/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

/** \file
   Vinax specific DSL CLI, access function implementation
*/

#include "dsl_cpe_control.h"

#if defined(INCLUDE_DSL_CPE_API_VINAX)

#include "dsl_cpe_cli.h"
#include "drv_dsl_cpe_api.h"
#include "drv_dsl_cpe_api_ioctl.h"

#ifdef INCLUDE_DSL_CPE_CLI_SUPPORT

#undef DSL_CCA_DBG_BLOCK
#define DSL_CCA_DBG_BLOCK DSL_CCA_DBG_CLI

extern const char *sFailureReturn;

/* for debugging: */
#ifdef DSL_CLI_LOCAL
#undef DSL_CLI_LOCAL
#endif
#if 0
#define DSL_CLI_LOCAL
#else
#define DSL_CLI_LOCAL static
#endif


static const DSL_char_t g_sDms[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint16_t uMsgID (hex)" DSL_CPE_CRLF
   "- rawMsg(sequence of 32-bit hex values according to message cat; leading 0's can be skipped)" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_DeviceMessage_t pMsg" DSL_CPE_CRLF DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_DBG_DeviceMessageSend(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   /* maximum message size includinf ID, Index and Length field */
   #define VNX_MSG_LEN  262
   /**
      This bit of the Message ID identifies 32 bit aligned TC messages */
   #define VDSL2_MBX_MSG_ID_IFX_MASK 0x0010

   DSL_Error_t ret = 0;
   DSL_DeviceMessage_t DevMsg;
   DSL_uint8_t msg[VNX_MSG_LEN];
   DSL_uint16_t *pMsg16 = (DSL_uint16_t*)&msg[0];
   DSL_uint16_t const *pMsgId = (DSL_uint16_t*)&msg[0];
   DSL_uint32_t *pMsg32 = (DSL_uint32_t*)&msg[6];
   DSL_char_t *pChar;
   DSL_uint32_t nParam = 0;
   DSL_uint32_t i = 0;
   DSL_boolean_t  bXtcMsg = DSL_FALSE;


   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 3, DSL_CLI_MIN) == DSL_FALSE)
   {
      return -1;
   }

   pChar = pCommands;

   if (pChar == DSL_NULL)
   {
      DSL_CPE_FPrintf(out, "nReturn=%d (error during processing of message data)" DSL_CPE_CRLF , DSL_ERROR );
      return -1;
   }

   /* get MsgId, Index and Length parameter */
   while (pChar != DSL_NULL)
   {
      /* Extract next parameter */
      if (DSL_CPE_sscanf(pChar, "%x", &nParam) > 0)
      {
         i += 2;
         *pMsg16 = (DSL_uint16_t)nParam;
         /* Find next parameter */
         pChar = strpbrk( pChar, " ");
         if (pChar != DSL_NULL)
         {
            pChar++;
            pMsg16++;
         }
         else
         {
            break;
         }
      }
      else
      {
         break;
      }

      if (i >= 6) break;
   }

   /* decide on the MsgId wether to treat 16 or 32bit params */
   if (*pMsgId & VDSL2_MBX_MSG_ID_IFX_MASK)
      bXtcMsg = DSL_TRUE;


   while (pChar != DSL_NULL)
   {
      /* Extract next parameter */
      if (DSL_CPE_sscanf(pChar, "%x", &nParam) > 0)
      {
         if (bXtcMsg == DSL_TRUE)
         {
            i += 4;
            *pMsg32 = (DSL_uint32_t)nParam;
         }
         else
         {
            i += 2;
            *pMsg16 = (DSL_uint16_t)nParam;
         }
         /* Find next parameter */
         pChar = strpbrk( pChar, " ");
         if (pChar != DSL_NULL)
         {
            pChar++;
            if (bXtcMsg == DSL_TRUE)
               pMsg32++;
            else
               pMsg16++;
         }
         else
         {
            break;
         }
      }
      else
      {
         break;
      }

      if (i >= VNX_MSG_LEN) break;
   }
   if ( (i > 0) && (i < VNX_MSG_LEN) )
   {
      /* Firmware message must consist of MsgID and two parameters minimum (6 bytes)*/
      if (i < 6)
      {
         DSL_CPE_FPrintf(out, "nReturn=%d (firmware message has to be at least 3 "
            "hex parameters long - '0x' does not have to be written)" DSL_CPE_CRLF , DSL_ERROR );
         return -1;
      }
      else
      {
         DevMsg.data.pMsg    = (DSL_uint8_t*)&msg;
         DevMsg.data.nSizeTx = (DSL_uint16_t)i;
         DevMsg.data.nSizeRx = (DSL_uint16_t)VNX_MSG_LEN;

         ret = DSL_CPE_Ioctl (fd, DSL_FIO_DBG_DEVICE_MESSAGE_SEND, (int) &DevMsg);

         if ((ret < 0) && (DevMsg.accessCtl.nReturn < DSL_SUCCESS))
         {
            DSL_CPE_FPrintf (out, sFailureReturn, ret);
            return ret;
         }
         else
         {
            DSL_CPE_FPrintf(out, "nReturn=%d nData=\"", ret );
            pMsg16 = (DSL_uint16_t*)&msg[0];
            pMsg32 = (DSL_uint32_t*)&msg[6];
            /* print Msg ID, Index and Length*/
            DSL_CPE_FPrintf(out, "%04X ", *pMsg16 );
            pMsg16++;
            DSL_CPE_FPrintf(out, "%04X ", *pMsg16 );
            pMsg16++;
            DSL_CPE_FPrintf(out, "%04X ", *pMsg16 );
            pMsg16++;
            i = 6;
            if (bXtcMsg == DSL_TRUE)
            {
               while (i < DevMsg.data.nSizeRx)
               {
                  DSL_CPE_FPrintf(out, "%08X ", *pMsg32 );
                  pMsg32++;
                  i += 4;
               }
            }
            else
            {
               while (i < DevMsg.data.nSizeRx)
               {
                  DSL_CPE_FPrintf(out, "%04X ", *pMsg16 );
                  pMsg16++;
                  i += 2;
               }
            }

            DSL_CPE_FPrintf(out, "\"" DSL_CPE_CRLF);
            return 0;
         }
      }

   }
   else
   {
      DSL_CPE_FPrintf(out, "DSL: error during processing of message data." DSL_CPE_CRLF );
      return -1;
   }

   return 0;
}

static const DSL_char_t g_sAlig[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_AccessDir_t nDirection" DSL_CPE_CRLF
   "   near-end = 0" DSL_CPE_CRLF
   "   far-end = 1" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_AuxLineInventory_t pData" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_AuxLineInventoryGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_int_t i = 0;
   DSL_AuxLineInventory_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%u", &pData.nDirection);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_AUX_LINE_INVENTORY_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%u nLength=%u pData=\"" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, (unsigned int)pData.data.nLength);

      for (i = 0; i < pData.data.nLength; i++)
      {
         if (i%10 == 0) DSL_CPE_FPrintf ( out, DSL_CPE_CRLF);
         DSL_CPE_FPrintf ( out, "%02x ", pData.data.pData[i]);
      }
      DSL_CPE_FPrintf ( out, "\""DSL_CPE_CRLF);
   }

   return 0;
}


static const DSL_char_t g_sBpsg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_boolean_t bSupported[ " _MKSTR(DSL_BANDPLAN_LAST) "][ " _MKSTR(DSL_PROFILE_LAST) "]" DSL_CPE_CRLF
   "   false = 0" DSL_CPE_CRLF
   "   true = 1" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_BandPlanSupportGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_BandPlanSupport_t pData;
   DSL_int_t nBandPlan = 0, nProfile = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 0, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_BAND_PLAN_SUPPORT_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d" DSL_CPE_CRLF, pData.accessCtl.nReturn);
      DSL_CPE_FPrintf (out, "nFormat=(nBandPlan(dec),nProfile(dec),bSupported(dec)) nData=\"" DSL_CPE_CRLF);

      for (nBandPlan = 0; nBandPlan < DSL_BANDPLAN_LAST; nBandPlan++)
      {
         for (nProfile = 0; nProfile < DSL_PROFILE_LAST; nProfile++)
         {
            DSL_CPE_FPrintf (out, "(%02d,%02d,%d) ", nBandPlan, nProfile,
               pData.data.bSupported[nBandPlan][nProfile]);
         }
         DSL_CPE_FPrintf (out, DSL_CPE_CRLF );
      }
      DSL_CPE_FPrintf(out, "\"" DSL_CPE_CRLF );
   }

   return 0;
}

static const DSL_char_t g_sVig[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_VersionInformation_t pData" DSL_CPE_CRLF DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_VersionInformationGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_VersionInformation_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 0, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_VERSION_INFORMATION_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, ret);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d DSL_DriverVersionApi=%s DSL_ChipSetFWVersion=%s "
         "DSL_ChipSetHWVersion=%s DSL_ChipSetType=%s DSL_DriverVersionMeiBsp=%s"
         DSL_CPE_CRLF, pData.accessCtl.nReturn, pData.data.DSL_DriverVersionApi,
         pData.data.DSL_ChipSetFWVersion, pData.data.DSL_ChipSetHWVersion,
         pData.data.DSL_ChipSetType, pData.data.DSL_DriverVersionMeiBsp);
   }

   return 0;
}

static const DSL_char_t g_sLpctg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsTx" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsRx" DSL_CPE_CRLF
   "- DSL_uint32_t FrameCheckSequenceErrors" DSL_CPE_CRLF
   "- DSL_uint32_t AlignmentErrors" DSL_CPE_CRLF
   "- DSL_uint32_t OctetsTx" DSL_CPE_CRLF
   "- DSL_uint32_t OctetsRx" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsTxErrors" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsRxErrors" DSL_CPE_CRLF
   "- DSL_uint32_t PauseFramesTx" DSL_CPE_CRLF
   "- DSL_uint32_t PauseFramesRx" DSL_CPE_CRLF
   "- DSL_uint32_t FramesTooLong" DSL_CPE_CRLF
   "- DSL_uint32_t FramesTooShort" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_LinePathCountersTotalGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_LinePathCounterTotal_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   memset(&pData, 0x00, sizeof(DSL_LinePathCounterTotal_t));

   DSL_CPE_sscanf (pCommands, "%u", &pData.nChannel);

   ret = DSL_CPE_Ioctl(fd, DSL_FIO_LINE_PATH_COUNTER_TOTAL_GET, (int)&pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d "
         "nChannel=%d "
         "UnitsTx=%d "
         "UnitsRx=%d "
         "FrameCheckSequenceErrors=%d "
         "AlignmentErrors=%d "
         "OctetsTx=%d "
         "OctetsRx=%d "
         "UnitsTxErrors=%d "
         "UnitsRxErrors=%d "
         "PauseFramesTx=%d "
         "PauseFramesRx=%d "
         "FramesTooLong=%d "
         "FramesTooShort=%d ",
         pData.accessCtl.nReturn,
         pData.nChannel,
         pData.data.UnitsTx,
         pData.data.UnitsRx,
         pData.data.FrameCheckSequenceErrors,
         pData.data.AlignmentErrors,
         pData.data.OctetsTx,
         pData.data.OctetsRx,
         pData.data.UnitsTxErrors,
         pData.data.UnitsRxErrors,
         pData.data.PauseFramesTx,
         pData.data.PauseFramesRx,
         pData.data.FramesTooLong,
         pData.data.FramesTooShort);
   }

   return 0;
}

static const DSL_char_t g_sDpctg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsTx" DSL_CPE_CRLF
   "- DSL_uint32_t SingleCollisionFrames" DSL_CPE_CRLF
   "- DSL_uint32_t MultipleCollisionFrames" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsRx" DSL_CPE_CRLF
   "- DSL_uint32_t FrameCheckSequenceErrors" DSL_CPE_CRLF
   "- DSL_uint32_t AlignmentErrors" DSL_CPE_CRLF
   "- DSL_uint32_t OctetsTx" DSL_CPE_CRLF
   "- DSL_uint32_t OctetsRx" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsTxErrors" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsRxErrors" DSL_CPE_CRLF
   "- DSL_uint32_t PauseFramesTx" DSL_CPE_CRLF
   "- DSL_uint32_t PauseFramesRx" DSL_CPE_CRLF
   "- DSL_uint32_t FramesTooLong" DSL_CPE_CRLF
   "- DSL_uint32_t FramesTooShort" DSL_CPE_CRLF
   "- DSL_uint32_t UnitsCrc" DSL_CPE_CRLF   
   "- DSL_uint32_t UnitsAlign" DSL_CPE_CRLF   
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_DataPathCountersTotalGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_DataPathCounterTotal_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   memset(&pData, 0x00, sizeof(DSL_DataPathCounterTotal_t));

   DSL_CPE_sscanf (pCommands, "%u", &pData.nChannel);

   ret = DSL_CPE_Ioctl(fd, DSL_FIO_DATA_PATH_COUNTER_TOTAL_GET, (int)&pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d "
         "nChannel=%d "
         "UnitsTx=%d "
         "SingleCollisionFrames=%d "
         "MultipleCollisionFrames=%d "
         "UnitsRx=%d "
         "FrameCheckSequenceErrors=%d "
         "AlignmentErrors=%d "
         "OctetsTx=%d "
         "OctetsRx=%d "
         "UnitsTxErrors=%d "
         "UnitsRxErrors=%d "
         "PauseFramesTx=%d "
         "PauseFramesRx=%d "
         "FramesTooLong=%d "
         "FramesTooShort=%d "
         "UnitsCrc=%d "
         "UnitsAlign=%d",
         pData.accessCtl.nReturn,
         pData.nChannel,
         pData.data.UnitsTx,
         pData.data.SingleCollisionFrames,
         pData.data.MultipleCollisionFrames,
         pData.data.UnitsRx,
         pData.data.FrameCheckSequenceErrors,
         pData.data.AlignmentErrors,
         pData.data.OctetsTx,
         pData.data.OctetsRx,
         pData.data.UnitsTxErrors,
         pData.data.UnitsRxErrors,
         pData.data.PauseFramesTx,
         pData.data.PauseFramesRx,
         pData.data.FramesTooLong,
         pData.data.FramesTooShort,
         pData.data.UnitsCrc,
         pData.data.UnitsAlign);
   }

   return 0;
}

static const DSL_char_t g_sUbwcg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_UtopiaBusWidth_t nDataWidth (dec)" DSL_CPE_CRLF
   "   default(16-bit) = 0" DSL_CPE_CRLF
   "   8-bit           = 1" DSL_CPE_CRLF
   "   16-bit          = 2" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_UtopiaBusWidthConfigGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_UtopiaBusWidthConfig_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 0, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_UTOPIA_BUS_WIDTH_CONFIG_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d nDataWidth=%d" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.data.nDataWidth);
   }

   return 0;
}

static const DSL_char_t g_sUbwcs[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "- DSL_UtopiaBusWidth_t nDataWidth (dec)" DSL_CPE_CRLF
   "   default(16-bit) = 0" DSL_CPE_CRLF
   "   8-bit           = 1" DSL_CPE_CRLF
   "   16-bit          = 2" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_UtopiaBusWidthConfigSet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_UtopiaBusWidthConfig_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%d", &pData.data.nDataWidth);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_UTOPIA_BUS_WIDTH_CONFIG_SET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d" DSL_CPE_CRLF, pData.accessCtl.nReturn);
   }

   return 0;
}

static const DSL_char_t g_sLlcs[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_DEV_RxTxGainSelection_t nGainSelection" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   default = 0" DSL_CPE_CRLF
   "   user    = 1" DSL_CPE_CRLF
   "- DSL_int16_t nRxGain" DSL_CPE_CRLF
   "- DSL_int16_t nTxGain" DSL_CPE_CRLF
   "- DSL_DEV_Clockout_t nClockoutEnable" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   on      = 0" DSL_CPE_CRLF
   "   off     = 1" DSL_CPE_CRLF
   "- DSL_DEV_Filter_t nFilter" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   off     = 0" DSL_CPE_CRLF
   "   isdn    = 1" DSL_CPE_CRLF
   "   pots    = 2" DSL_CPE_CRLF
   "   pots_2  = 3" DSL_CPE_CRLF
   "   pots_3  = 4" DSL_CPE_CRLF
   "- DSL_DEV_HdlcMode_t nHdlcIfMode" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   od      = 0" DSL_CPE_CRLF
   "   pp      = 1" DSL_CPE_CRLF
   "- DSL_DEV_MiiClockSource_t nMiiClockSource" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   ext     = 0" DSL_CPE_CRLF
   "   int     = 1" DSL_CPE_CRLF
   "   cry     = 2" DSL_CPE_CRLF
   "- DSL_DEV_MiiClockChain_t nMiiClockChain" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   off     = 0" DSL_CPE_CRLF
   "   on      = 1" DSL_CPE_CRLF
   "- DSL_DEV_TcMode_t nTcMode" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   mac     = 0" DSL_CPE_CRLF
   "   vphy    = 1" DSL_CPE_CRLF
   "- DSL_DEV_MiiMode_t nMiiMode" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   mii     = 0" DSL_CPE_CRLF
   "   rmii    = 1" DSL_CPE_CRLF
   "   smii    = 2" DSL_CPE_CRLF
   "   sssmii  = 3" DSL_CPE_CRLF
   "   off     = 4" DSL_CPE_CRLF
   "- DSL_DEV_LineMode_t nLineMode (hex)" DSL_CPE_CRLF
   "   na            = -1" DSL_CPE_CRLF
   "   VDSL2 B43     = 0001" DSL_CPE_CRLF
   "   VDSL2 A43     = 0002" DSL_CPE_CRLF
   "   VDSL2 V43     = 0004" DSL_CPE_CRLF
   "   VDSL1 V43POTS = 0008" DSL_CPE_CRLF
   "   VDSL1 V43ISDN = 0010" DSL_CPE_CRLF
   "   ADSL1 C43     = 0020" DSL_CPE_CRLF
   "   ADSL2 J43     = 0040" DSL_CPE_CRLF
   "   ADSL2 B43C    = 0080" DSL_CPE_CRLF
   "   ADSL2 A43C    = 0100" DSL_CPE_CRLF
   "- DSL_DEV_Hybrid_t nHybrid" DSL_CPE_CRLF
   "   na                 = -1" DSL_CPE_CRLF
   "   AD1_138_17         = 0" DSL_CPE_CRLF
   "   AD1_25_17          = 1" DSL_CPE_CRLF
   "   AD1_138_30         = 2" DSL_CPE_CRLF
   "   AD1_138_17_CPE     = 3" DSL_CPE_CRLF
   "   F1_998             = 4" DSL_CPE_CRLF
   "   F1_998_U0          = 5" DSL_CPE_CRLF
   "   F1_998_U017_o_ISDN = 6" DSL_CPE_CRLF
   "   F1_JAP30           = 7" DSL_CPE_CRLF
   "- DSL_uint32_t nBaseAddr (hex)" DSL_CPE_CRLF
   "- DSL_uint8_t nIrqNum" DSL_CPE_CRLF
   "- DSL_Parity_t nUtopiaParity" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   odd     = 0" DSL_CPE_CRLF
   "   even    = 1" DSL_CPE_CRLF
   "- DSL_uint8_t nUtopiaPhyAdr (hex)" DSL_CPE_CRLF
   "   na          = FF" DSL_CPE_CRLF
   "   valid range = 0..1E" DSL_CPE_CRLF      
   "- DSL_UtopiaBusWidth_t nUtopiaBusWidth (dec)" DSL_CPE_CRLF
   "   na     = -1" DSL_CPE_CRLF
   "   8-bit  = 1" DSL_CPE_CRLF
   "   16-bit = 2" DSL_CPE_CRLF
   "- DSL_Parity_t nPosPhyParity" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   odd     = 0" DSL_CPE_CRLF
   "   even    = 1" DSL_CPE_CRLF
   "- DSL_uint8_t nPosPhyAdr (hex)" DSL_CPE_CRLF
   "   na          = FF" DSL_CPE_CRLF
   "   valid range = 0..1E" DSL_CPE_CRLF
   "- DSL_int32_t nPosphyChunkSize" DSL_CPE_CRLF
   "   na          = -1" DSL_CPE_CRLF
   "   valid range = 0..64" DSL_CPE_CRLF   
   "- DSL_int32_t nEapsTimeout (msec)" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_LowLevelConfigurationSet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_LowLevelConfiguration_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 21, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%d %d %d %d %d %d %d %d %d %d %x %d %x %u %d %x %d %d %x %d %d",
      &pData.data.nGainSelection,
      &pData.data.nUserGainSettings.nRxGain,
      &pData.data.nUserGainSettings.nTxGain,
      &pData.data.nClockoutEnable,
      &pData.data.nFilter,
      &pData.data.nHdlcIfMode,
      &pData.data.nMiiClockSource,
      &pData.data.nMiiClockChain,
      &pData.data.nTcMode,
      &pData.data.nMiiMode,
      &pData.data.nLineMode,
      &pData.data.nHybrid,
      &pData.data.nBaseAddr,
      &pData.data.nIrqNum,
      &pData.data.nUtopiaParity,
      &pData.data.nUtopiaPhyAdr,
      &pData.data.nUtopiaBusWidth,
      &pData.data.nPosPhyParity,
      &pData.data.nPosPhyAdr,
      &pData.data.nPosphyChunkSize,
      &pData.data.nEapsTimeout);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_LOW_LEVEL_CONFIGURATION_SET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d" DSL_CPE_CRLF,
         pData.accessCtl.nReturn);
   }

   return 0;
}

static const DSL_char_t g_sLlcg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_DEV_RxTxGainSelection_t nGainSelection" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   default = 0" DSL_CPE_CRLF
   "   user    = 1" DSL_CPE_CRLF
   "- DSL_int16_t nRxGain" DSL_CPE_CRLF
   "- DSL_int16_t nTxGain" DSL_CPE_CRLF
   "- DSL_DEV_Clockout_t nClockoutEnable" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   on      = 0" DSL_CPE_CRLF
   "   off     = 1" DSL_CPE_CRLF
   "- DSL_DEV_Filter_t nFilter" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   off     = 0" DSL_CPE_CRLF
   "   isdn    = 1" DSL_CPE_CRLF
   "   pots    = 2" DSL_CPE_CRLF
   "   pots_2  = 3" DSL_CPE_CRLF
   "- DSL_DEV_HdlcMode_t nHdlcIfMode" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   od      = 0" DSL_CPE_CRLF
   "   pp      = 1" DSL_CPE_CRLF
   "- DSL_DEV_MiiClockSource_t nMiiClockSource"
   "   na      = -1" DSL_CPE_CRLF
   "   ext     = 0" DSL_CPE_CRLF
   "   int     = 1" DSL_CPE_CRLF
   "   cry     = 2" DSL_CPE_CRLF
   "- DSL_DEV_MiiClockChain_t nMiiClockChain" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   off     = 0" DSL_CPE_CRLF
   "   on      = 1" DSL_CPE_CRLF
   "- DSL_DEV_TcMode_t nTcMode" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   mac     = 0" DSL_CPE_CRLF
   "   vphy    = 1" DSL_CPE_CRLF
   "- DSL_DEV_MiiMode_t nMiiMode" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   mii     = 0" DSL_CPE_CRLF
   "   rmii    = 1" DSL_CPE_CRLF
   "   smii    = 2" DSL_CPE_CRLF
   "   sssmii  = 3" DSL_CPE_CRLF
   "   off     = 4" DSL_CPE_CRLF
   "- DSL_DEV_LineMode_t nLineMode (hex)" DSL_CPE_CRLF
   "   na            = -1" DSL_CPE_CRLF
   "   VDSL2 B43     = 0001" DSL_CPE_CRLF
   "   VDSL2 A43     = 0002" DSL_CPE_CRLF
   "   VDSL2 V43     = 0004" DSL_CPE_CRLF
   "   VDSL1 V43POTS = 0008" DSL_CPE_CRLF
   "   VDSL1 V43ISDN = 0010" DSL_CPE_CRLF
   "   ADSL1 C43     = 0020" DSL_CPE_CRLF
   "   ADSL2 J43     = 0040" DSL_CPE_CRLF
   "   ADSL2 B43C    = 0080" DSL_CPE_CRLF
   "   ADSL2 A43C    = 0100" DSL_CPE_CRLF
   "- DSL_DEV_Hybrid_t nHybrid" DSL_CPE_CRLF
   "   na                 = -1" DSL_CPE_CRLF
   "   AD1_138_17         = 0" DSL_CPE_CRLF
   "   AD1_25_17          = 1" DSL_CPE_CRLF
   "   AD1_138_30         = 2" DSL_CPE_CRLF
   "   AD1_138_17_CPE     = 3" DSL_CPE_CRLF
   "   F1_998             = 4" DSL_CPE_CRLF
   "   F1_998_U0          = 5" DSL_CPE_CRLF
   "   F1_998_U017_o_ISDN = 6" DSL_CPE_CRLF
   "   F1_JAP30           = 7" DSL_CPE_CRLF
   "- DSL_uint32_t nBaseAddr (hex)" DSL_CPE_CRLF
   "- DSL_uint8_t nIrqNum" DSL_CPE_CRLF
   "- DSL_Parity_t nUtopiaParity" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   odd     = 0" DSL_CPE_CRLF
   "   even    = 1" DSL_CPE_CRLF
   "- DSL_uint8_t nUtopiaPhyAdr (hex)" DSL_CPE_CRLF
   "- DSL_UtopiaBusWidth_t nUtopiaBusWidth (dec)" DSL_CPE_CRLF
   "   na     = -1" DSL_CPE_CRLF
   "   8-bit  = 1" DSL_CPE_CRLF
   "   16-bit = 2" DSL_CPE_CRLF
   "- DSL_Parity_t nPosPhyParity" DSL_CPE_CRLF
   "   na      = -1" DSL_CPE_CRLF
   "   odd     = 0" DSL_CPE_CRLF
   "   even    = 1" DSL_CPE_CRLF
   "- DSL_uint8_t nPosPhyAdr (hex)" DSL_CPE_CRLF
   "- DSL_int32_t nPosphyChunkSize" DSL_CPE_CRLF
   "- DSL_int32_t nEapsTimeout (msec)" DSL_CPE_CRLF   
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_LowLevelConfigurationGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_LowLevelConfiguration_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 0, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   memset(&pData, 0, sizeof(DSL_LowLevelConfiguration_t));

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_LOW_LEVEL_CONFIGURATION_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d "
         "nGainSelection=%d "
         "nUserRxGain=%d "
         "nUserTxGain=%d "
         "nClockoutEnable=%d "
         "nFilter=%d "
         "nHdlcIfMode=%d "
         "nMiiClockSource=%d "
         "nMiiClockChain=%d "
         "nTcMode=%d "
         "nMiiMode=%d "
         "nLineMode=%x "
         "nHybrid=%d "
         "nBaseAddr=%x "
         "nIrqNum=%u "
         "nUtopiaParity=%d "
         "nUtopiaPhyAdr=%x "
         "nUtopiaBusWidth=%d "         
         "nPosPhyParity=%d "
         "nPosPhyAdr=%x "
         "nPosphyChunkSize=%d "
         "nEapsTimeout=%d",
         pData.accessCtl.nReturn,
         pData.data.nGainSelection,
         pData.data.nUserGainSettings.nRxGain,
         pData.data.nUserGainSettings.nTxGain,
         pData.data.nClockoutEnable,
         pData.data.nFilter,
         pData.data.nHdlcIfMode,
         pData.data.nMiiClockSource,
         pData.data.nMiiClockChain,
         pData.data.nTcMode,
         pData.data.nMiiMode,
         pData.data.nLineMode,
         pData.data.nHybrid,
         pData.data.nBaseAddr,
         pData.data.nIrqNum,
         pData.data.nUtopiaParity,
         pData.data.nUtopiaPhyAdr,
         pData.data.nUtopiaBusWidth,
         pData.data.nPosPhyParity,
         pData.data.nPosPhyAdr,
         pData.data.nPosphyChunkSize,
         pData.data.nEapsTimeout);
   }

   return 0;
}

static const DSL_char_t g_sG997lspbg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_AccessDir_t nDirection" DSL_CPE_CRLF
   "   upstream = 0" DSL_CPE_CRLF
   "   downstream = 1" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_AccessDir_t nDirection" DSL_CPE_CRLF
   "   upstream = 0" DSL_CPE_CRLF
   "   downstream = 1" DSL_CPE_CRLF DSL_CPE_CRLF
   "- DSL_int16_t[" _MKSTR(DSL_G997_MAX_NUMBER_OF_BAND) "] LATN" DSL_CPE_CRLF
   "- DSL_int16_t[" _MKSTR(DSL_G997_MAX_NUMBER_OF_BAND) "] SATN" DSL_CPE_CRLF
   "- DSL_int16_t[" _MKSTR(DSL_G997_MAX_NUMBER_OF_BAND) "] SNR" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_G997_LineStatusPerBandGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_G997_LineStatusPerBand_t pData;
   DSL_int_t i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%u", &pData.nDirection);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_G997_LINE_STATUS_PER_BAND_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf(out, "nReturn=%d nDirection=%u ", pData.accessCtl.nReturn,
         pData.nDirection );

      for (i = 0; i < DSL_G997_MAX_NUMBER_OF_BANDS; i++)
      {
         DSL_CPE_FPrintf(out, "LATN[%d]=%hd ", i, pData.data.LATN[i]);
      }
      for (i = 0; i < DSL_G997_MAX_NUMBER_OF_BANDS; i++)
      {
         DSL_CPE_FPrintf(out, "SATN[%d]=%hd ", i, pData.data.SATN[i]);
      }
      for (i = 0; i < DSL_G997_MAX_NUMBER_OF_BANDS; i++)
      {
         DSL_CPE_FPrintf(out, "SNR[%d]=%hd ", i, pData.data.SNR[i]);
      }
      DSL_CPE_FPrintf(out, DSL_CPE_CRLF );
   }

   return 0;
}

static const DSL_char_t g_sRaCs[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_AccessDir_t nDirection" DSL_CPE_CRLF
   "   near-end = 0" DSL_CPE_CRLF
   "   far-end = 1" DSL_CPE_CRLF
   "- DSL_G997_RA_MODE_t RA_MODE" DSL_CPE_CRLF
   "   at init = 2" DSL_CPE_CRLF
   "   dynamic = 3" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nDirection" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_G997_RateAdaptationConfigSet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_G997_RateAdaptationConfig_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 2, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%u %u", &pData.nDirection, &pData.data.RA_MODE);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_G997_RATE_ADAPTATION_CONFIG_SET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d nDirection=%hu" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nDirection);
   }

   return 0;
}

static const DSL_char_t g_sRaCg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_AccessDir_t nDirection" DSL_CPE_CRLF
   "   near-end = 0" DSL_CPE_CRLF
   "   far-end = 1" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nDirection" DSL_CPE_CRLF
   "- DSL_G997_RA_MODE_t RA_MODE" DSL_CPE_CRLF
   "   at init = 2" DSL_CPE_CRLF
   "   dynamic = 3" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_G997_RateAdaptationConfigGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_G997_RateAdaptationConfig_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%u", &pData.nDirection);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_G997_RATE_ADAPTATION_CONFIG_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d nDirection=%hu RA_MODE=%u" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nDirection, pData.data.RA_MODE);
   }

   return 0;
}

static const DSL_char_t g_sSarCg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_SAR_SegClassFilter_t nSegClassFilter" DSL_CPE_CRLF
   "   sar filter mac addr = 0" DSL_CPE_CRLF
   "   sar filter vlan id = 1" DSL_CPE_CRLF
   "   sar filter vlan prio = 2" DSL_CPE_CRLF
   "- DSL_uint16_t nDefVpi" DSL_CPE_CRLF
   "- DSL_uint16_t nDefVci" DSL_CPE_CRLF
   "- SAR_MuxType_t nMuxType" DSL_CPE_CRLF
   "   sar mux vc mux = 0" DSL_CPE_CRLF
   "   sar mux llc = 1" DSL_CPE_CRLF
   "- DSL_boolean_t bFcsPreserve" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_ConfigGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_Config_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu", &pData.nChannel);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_CONFIG_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d nChannel=%hu nSegClassFilter=%u nDefVpi=%hu nDefVci=%hu "
         "nMuxType=%u bFcsPreserve=%u" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel, pData.data.nSegClassFilter,
         pData.data.nDefVpi, pData.data.nDefVci, pData.data.nMuxType,
         pData.data.bFcsPreserve);
   }

   return 0;
}

static const DSL_char_t g_sSarCs[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_SAR_SegClassFilter_t nSegClassFilter" DSL_CPE_CRLF
   "   sar filter mac addr = 0" DSL_CPE_CRLF
   "   sar filter vlan id = 1" DSL_CPE_CRLF
   "   sar filter vlan prio = 2" DSL_CPE_CRLF
   "- DSL_uint16_t nDefVpi" DSL_CPE_CRLF
   "- DSL_uint16_t nDefVci" DSL_CPE_CRLF
   "- SAR_MuxType_t nMuxType" DSL_CPE_CRLF
   "   sar mux vc mux = 0" DSL_CPE_CRLF
   "   sar mux llc = 1" DSL_CPE_CRLF
   "- DSL_boolean_t bFcsPreserve" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_ConfigSet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_Config_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 6, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu %u %hu %hu %u %u", &pData.nChannel,
      &pData.data.nSegClassFilter, &pData.data.nDefVpi, &pData.data.nDefVci,
      &pData.data.nMuxType, &pData.data.bFcsPreserve);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_CONFIG_SET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d nChannel=%hu" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel);
   }

   return 0;
}

static const DSL_char_t g_sSarRcg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint32_t nInvalidAtm" DSL_CPE_CRLF
   "- DSL_uint32_t nInvalidFrames" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "- DSL_uint32_t nCrc" DSL_CPE_CRLF
   "- DSL_uint32_t nFrames" DSL_CPE_CRLF
   "- ... n times previous 2 params [0 <= n <=" _MKSTR((DSL_SAR_MAX_REA_TABLE_ENTRIES -1)) "]" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_ReassemblyCountersGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_ReassemblyCounters_t pData;
   DSL_int_t i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu", &pData.nChannel);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_REASSEMBLY_COUNTERS_GET, (int) &pData);
   
   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d nChannel=%hu nInvalidAtm=%u, nInvalidFrames=%u" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel, pData.data.nInvalidAtm,
         pData.data.nInvalidFrames);

      DSL_CPE_FPrintf(out, "nFormat=(idx(dec),nCrc(dec),nFrames(dec)) nData=\"");

      for (i = 0; i < DSL_SAR_MAX_REA_TABLE_ENTRIES; i++)
      {
         DSL_CPE_FPrintf(out, "(%d,%u,%u) ", i, pData.data.nCrc[i],
            pData.data.nFrames[i]);
      }
      DSL_CPE_FPrintf(out, "\"" DSL_CPE_CRLF);
   }

   return 0;
}

static const DSL_char_t g_sSarRtg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint8_t nNumEntries" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "- DSL_uint16_t nVpi" DSL_CPE_CRLF
   "- DSL_uint16_t nVci" DSL_CPE_CRLF
   "- ... n times previous 2 params [0 <= n <=" _MKSTR((DSL_SAR_MAX_REA_TABLE_ENTRIES -1)) "]" DSL_CPE_CRLF
DSL_CPE_CRLF \
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_ReassemblyTableGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_ReassemblyTable_t pData;
   DSL_int_t i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu", &pData.nChannel);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_REASSEMBLY_TABLE_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d nChannel=%hu" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel);

      DSL_CPE_FPrintf(out, "nFormat=(idx(dec),nVpi(dec),nVci(dec)) nData=\"");

      for (i = 0; i < pData.data.nNumEntries; i++)
      {
         DSL_CPE_FPrintf(out, "(%d,%d,%d) ", i, pData.data.pTableEntry[i].nVpi,
            pData.data.pTableEntry[i].nVci );
      }
      DSL_CPE_FPrintf(out, "\"" DSL_CPE_CRLF);
   }

   return 0;
}

static const DSL_char_t g_sSarRts[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint8_t nNumEntries" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "- DSL_uint16_t nVpi" DSL_CPE_CRLF
   "- DSL_uint16_t nVci" DSL_CPE_CRLF
   "- ... n times previous 2 params [0 <= n <=" _MKSTR((DSL_SAR_MAX_REA_TABLE_ENTRIES -1)) "]" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
DSL_CPE_CRLF \
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_ReassemblyTableSet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_ReassemblyTable_t pData;
   DSL_char_t *pChar = DSL_NULL, *pTmp = DSL_NULL;
   DSL_int_t nMax = 0;
   DSL_int_t i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 4, DSL_CLI_MIN) == DSL_FALSE)
   {
      return -1;
   }

   nMax = (DSL_SAR_MAX_REA_TABLE_ENTRIES * 2) + 2;
   if (DSL_CPE_CLI_CheckParamNumber(pCommands, nMax, DSL_CLI_MAX) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu %bu", &pData.nChannel, &pData.data.nNumEntries);
   pTmp = pCommands;
   ret = DSL_CPE_MoveCharPtr(pTmp, 2, " ", &pChar);
   pTmp = pChar;

   for (i = 0; i < (DSL_int_t)(pData.data.nNumEntries); i++)
   {
      if ( pChar == DSL_NULL )
      {
         /* no more entries, leave the loop */
         break;
      }

      ret = DSL_CPE_sscanf(pChar, "%hu %hu", &pData.data.pTableEntry[i].nVpi,
         &pData.data.pTableEntry[i].nVci );

      if ( ret != 2 )
      {
         DSL_CPE_FPrintf(out, DSL_CPE_PREFIX "invalid table entry data!" DSL_CPE_CRLF);
         DSL_CPE_FPrintf(out, "nReturn=%d" DSL_CPE_CRLF, (DSL_int_t)DSL_ERROR);
         return 0;
      }
      ret = DSL_CPE_MoveCharPtr(pTmp, 2, " ", &pChar);
      pTmp = pChar;
   }

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_REASSEMBLY_TABLE_SET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d nChannel=%hu" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel);
   }

   return 0;
}

static const DSL_char_t g_sSarScg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint32_t nInvalidFramesRx" DSL_CPE_CRLF
   "- DSL_uint32_t nDefaultFramesRx" DSL_CPE_CRLF
   "- DSL_uint32_t nDefaultAtm" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "- DSL_uint32_t nFramesRx" DSL_CPE_CRLF
   "- DSL_uint32_t nAtm" DSL_CPE_CRLF
   "- ... n times previous 2 params [0 <= n <=" _MKSTR((DSL_SAR_MAX_SEG_TABLE_ENTRIES -1)) "]" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_SegmentationCountersGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_SegmentationCounters_t pData;
   DSL_int_t i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu", &pData.nChannel);

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_SEGMENTATION_COUNTERS_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d nChannel=%hu nInvalidFramesRx=%u, nDefaultFramesRx=%u"
         "nDefaultAtm=%u" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel, pData.data.nInvalidFramesRx,
         pData.data.nDefaultFramesRx, pData.data.nDefaultAtm);

      DSL_CPE_FPrintf(out, "nFormat=(idx(dec),nFramesRx(dec),nAtm(dec)) nData=\"");

      for (i = 0; i < DSL_SAR_MAX_SEG_TABLE_ENTRIES; i++)
      {
         DSL_CPE_FPrintf(out, "(%d,%u,%u) ", i, pData.data.nFramesRx[i],
            pData.data.nAtm[i]);
      }
      DSL_CPE_FPrintf(out, "\"" DSL_CPE_CRLF);
   }

   return 0;
}

static const DSL_char_t g_sSarStg[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint8_t nNumEntries" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "- DSL_uint8_t[6] nMacAddr" DSL_CPE_CRLF
   "   used hex format xx:xx:xx:xx:xx:xx" DSL_CPE_CRLF
   "- DSL_uint32_t nVlanId" DSL_CPE_CRLF
   "- DSL_uint32_t nVlanPrio" DSL_CPE_CRLF
   "- DSL_uint16_t nVpi" DSL_CPE_CRLF
   "- DSL_uint16_t nVci" DSL_CPE_CRLF
   "- ... n times previous 5 params [0 <= n <=" _MKSTR((DSL_SAR_MAX_SEG_TABLE_ENTRIES -1)) "]" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_SegmentationTableGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_SegmentationTable_t pData;
   DSL_int_t i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 1, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu", &pData.nChannel);

   memset(&pData, 0, sizeof(pData));
   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_SEGMENTATION_TABLE_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d nChannel=%hu nNumEntries=%hu" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel, pData.data.nNumEntries);

      DSL_CPE_FPrintf(out, "nFormat=(idx(dec),MacAddr(hex),nVlanId(dec),nVlanPrio(dec),"
         "nVpi(dec),nVci(dec)) nData=\"" DSL_CPE_CRLF );

      for (i = 0; i < pData.data.nNumEntries; i++)
      {
         DSL_CPE_FPrintf(out, "(%d,%02X:%02X:%02X:%02X:%02X:%02X,%u,%u,%hu,%hu)" DSL_CPE_CRLF,
            i, pData.data.pTableEntry[i].pMacAddress[0],
            pData.data.pTableEntry[i].pMacAddress[1],
            pData.data.pTableEntry[i].pMacAddress[2],
            pData.data.pTableEntry[i].pMacAddress[3],
            pData.data.pTableEntry[i].pMacAddress[4],
            pData.data.pTableEntry[i].pMacAddress[5],
            pData.data.pTableEntry[i].nVlanId,
            pData.data.pTableEntry[i].nVlanPrio,
            pData.data.pTableEntry[i].nVpi,
            pData.data.pTableEntry[i].nVci );
      }
      DSL_CPE_FPrintf(out, "\"" DSL_CPE_CRLF);
   }

   return 0;
}

static const DSL_char_t g_sSarSts[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   "- DSL_uint8_t nNumEntries" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "- DSL_uint8_t[6] nMacAddr" DSL_CPE_CRLF
   "   used hex format xx:xx:xx:xx:xx:xx" DSL_CPE_CRLF
   "- DSL_uint32_t nVlanId" DSL_CPE_CRLF
   "- DSL_uint32_t nVlanPrio" DSL_CPE_CRLF
   "- DSL_uint16_t nVpi" DSL_CPE_CRLF
   "- DSL_uint16_t nVci" DSL_CPE_CRLF
   "- ... n times previous 5 params [0 <= n <=" _MKSTR((DSL_SAR_MAX_SEG_TABLE_ENTRIES -1)) "]" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_uint8_t nChannel" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_SAR_SegmentationTableSet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_SAR_SegmentationTable_t pData;
   DSL_int_t nMax = 0;
   DSL_char_t sMacAdr[20];
   DSL_CPE_MacAddress_t macAdr;
   DSL_char_t *pChar = DSL_NULL, *pTmp = DSL_NULL;
   DSL_int_t i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 7, DSL_CLI_MIN) == DSL_FALSE)
   {
      return -1;
   }

   nMax = (DSL_SAR_MAX_SEG_TABLE_ENTRIES * 5) + 2;
   if (DSL_CPE_CLI_CheckParamNumber(pCommands, nMax, DSL_CLI_MAX) == DSL_FALSE)
   {
      return -1;
   }

   DSL_CPE_sscanf (pCommands, "%bu %bu", &pData.nChannel, &pData.data.nNumEntries);

   pTmp = pCommands;
   ret = DSL_CPE_MoveCharPtr(pTmp, 2, " ", &pChar);
   pTmp = pChar;

   for (i = 0; i < (DSL_int_t)(pData.data.nNumEntries); i++)
   {
      if ( pChar == DSL_NULL )
      {
         /* no more entries, leave the loop */
         break;
      }
      sMacAdr[0] = '\0';
      
      ret = DSL_CPE_sscanf (pChar, "%s %u %u %hu %hu",
         sMacAdr,
         &pData.data.pTableEntry[i].nVlanId,
         &pData.data.pTableEntry[i].nVlanPrio,
         &pData.data.pTableEntry[i].nVpi,
         &pData.data.pTableEntry[i].nVci );

      ret = DSL_CPE_GetMacAdrFromString(sMacAdr, &macAdr);
      if (ret < DSL_SUCCESS)
      {
         DSL_CPE_FPrintf(out, DSL_CPE_PREFIX "inconsistent MAC address!" DSL_CPE_CRLF);
         DSL_CPE_FPrintf(out, "nReturn=%d" DSL_CPE_CRLF, (DSL_int_t)DSL_ERROR);
         return 0;
      }
      else
      {
         memcpy (pData.data.pTableEntry[i].pMacAddress,
            &macAdr, DSL_MAC_ADDRESS_OCTETS);
      }
      ret = DSL_CPE_MoveCharPtr(pTmp, 5, " ", &pChar);
      pTmp = pChar;
   }

   if (i != pData.data.nNumEntries)
   {
      DSL_CPE_FPrintf(out, DSL_CPE_PREFIX "inconsistent number of entries!" DSL_CPE_CRLF);
      DSL_CPE_FPrintf(out, "nReturn=%d" DSL_CPE_CRLF, (DSL_int_t)DSL_ERROR);
      return 0;
   }

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_SAR_SEGMENTATION_TABLE_SET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d nChannel=%hu" DSL_CPE_CRLF,
         pData.accessCtl.nReturn, pData.nChannel);
   }

   return 0;
}


DSL_void_t DSL_CPE_CLI_DeviceCommandsRegister (DSL_void_t)
{
   /* Debug functionalities */
   DSL_CPE_CLI_CommandAdd ("dms", "DeviceMessageSend", DSL_CPE_CLI_DBG_DeviceMessageSend, g_sDms);

   /* Common functionalities */
   DSL_CPE_CLI_CommandAdd ("alig", "AuxLineInventoryGet", DSL_CPE_CLI_AuxLineInventoryGet, g_sAlig);
   DSL_CPE_CLI_CommandAdd ("bpsg", "BandPlanSupportGet", DSL_CPE_CLI_BandPlanSupportGet, g_sBpsg);
   DSL_CPE_CLI_CommandAdd ("vig", "VersionInformationGet", DSL_CPE_CLI_VersionInformationGet, g_sVig);
   DSL_CPE_CLI_CommandAdd ("lpctg", "LinePathCountersTotalGet", DSL_CPE_CLI_LinePathCountersTotalGet, g_sLpctg);
   DSL_CPE_CLI_CommandAdd ("dpctg", "DataPathCountersTotalGet", DSL_CPE_CLI_DataPathCountersTotalGet, g_sDpctg);

   DSL_CPE_CLI_CommandAdd ("ubwcg", "UtopiaBusWidthConfigGet", DSL_CPE_CLI_UtopiaBusWidthConfigGet, g_sUbwcg);
   DSL_CPE_CLI_CommandAdd ("ubwcs", "UtopiaBusWidthConfigSet", DSL_CPE_CLI_UtopiaBusWidthConfigSet, g_sUbwcs);   
   
   DSL_CPE_CLI_CommandAdd ("llcs", "LowLevelConfigurationSet", DSL_CPE_CLI_LowLevelConfigurationSet, g_sLlcs);
   DSL_CPE_CLI_CommandAdd ("llcg", "LowLevelConfigurationGet", DSL_CPE_CLI_LowLevelConfigurationGet, g_sLlcg);

   /* G.997.1 related functionlities */
   DSL_CPE_CLI_CommandAdd ("g997lspbg", "G997_LineStatusPerBandGet", DSL_CPE_CLI_G997_LineStatusPerBandGet, g_sG997lspbg);
   DSL_CPE_CLI_CommandAdd ("g997racs", "G997_RateAdaptationConfigSet", DSL_CPE_CLI_G997_RateAdaptationConfigSet, g_sRaCs);
   DSL_CPE_CLI_CommandAdd ("g997racg", "G997_RateAdaptationConfigGet", DSL_CPE_CLI_G997_RateAdaptationConfigGet, g_sRaCg);
   
   /* SAR related funtionalities */
   DSL_CPE_CLI_CommandAdd ("sarcg", "SAR_ConfigGet", DSL_CPE_CLI_SAR_ConfigGet, g_sSarCg);
   DSL_CPE_CLI_CommandAdd ("sarcs", "SAR_ConfigSet", DSL_CPE_CLI_SAR_ConfigSet, g_sSarCs);
   DSL_CPE_CLI_CommandAdd ("sarrcg", "SAR_ReassemblyCountersGet", DSL_CPE_CLI_SAR_ReassemblyCountersGet, g_sSarRcg);
   DSL_CPE_CLI_CommandAdd ("sarrtg", "SAR_ReassemblyTableGet", DSL_CPE_CLI_SAR_ReassemblyTableGet, g_sSarRtg);
   DSL_CPE_CLI_CommandAdd ("sarrts", "SAR_ReassemblyTableSet", DSL_CPE_CLI_SAR_ReassemblyTableSet, g_sSarRts);
   DSL_CPE_CLI_CommandAdd ("sarscg", "SAR_SegmentationCountersGet", DSL_CPE_CLI_SAR_SegmentationCountersGet, g_sSarScg);
   DSL_CPE_CLI_CommandAdd ("sarstg", "SAR_SegmentationTableGet", DSL_CPE_CLI_SAR_SegmentationTableGet, g_sSarStg);
   DSL_CPE_CLI_CommandAdd ("sarsts", "SAR_SegmentationTableSet", DSL_CPE_CLI_SAR_SegmentationTableSet, g_sSarSts);
}

#endif /* INCLUDE_DSL_CPE_CLI_SUPPORT */

#endif /* INCLUDE_DSL_CPE_API_VINAX */
