/******************************************************************************
**
** FILE NAME    : amazon_se_mei_bsp.c
** PROJECT      : Amazon_SE
** MODULES     	: MEI
**
** DATE         : 1 Jan 2006
** AUTHOR       : TC Chen
** DESCRIPTION  : MEI Driver
** COPYRIGHT    : 	Copyright (c) 2006
**			Infineon Technologies AG
**			Am Campeon 1-12, 85579 Neubiberg, Germany
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Version $Date      $Author     $Comment
   1.00.00  19/12/2006 TC Chen	   First version(same as danube_mei_bsp 2.00.03)
   2.00.00  24/04/2007 KH Teh	   Integrated DSL CPE API Driver 2.5.x
   2.00.01  25/04/2007 KH Teh	   Fix major number and device node name
   2.01.00  04/05/2007 KH Teh	   Add fw_date and fw_version
   2.02.00  04/05/2007 KH Teh	   bug fixes
   2.03.00  04/07/2007 KH Teh	   Free all memories mared FREE_ALL after all code swaps completed.
   2.04.00  04/08/2007 KH Teh	   Free all memories mared FREE_SHOWTIME after all code swaps completed.
*******************************************************************************/
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/proc_fs.h>
#include <asm/semaphore.h>
#include <asm/hardirq.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/bootmem.h>
#include <asm/uaccess.h>
#include <asm/amazon_se/amazon_se.h>
#include <asm/amazon_se/irq.h>
#include <asm/amazon_se/port.h>
#include <asm/amazon_se/amazon_se_led.h>
#define AMAZON_SE_MEI_BSP
#include <asm/amazon_se/amazon_se_mei_bsp.h>

#define AMAZON_SE_MEI_EMSG(fmt, args...) printk(KERN_ERR  "[%s %d]: " fmt,__FUNCTION__, __LINE__, ## args)
#define AMAZON_SE_MEI_DMSG(fmt, args...) printk(KERN_INFO "[%s %d]: " fmt,__FUNCTION__, __LINE__, ## args)

//TODO
//#define DFE_LOOPBACK
#undef DFE_LOOPBACK		// testing code //undefined by Henry , start to real link test.

#ifdef DFE_LOOPBACK
//#define DFE_MEM_TEST
//#define DFE_PING_TEST
#define DFE_ATM_LOOPBACK
#endif

#ifdef DFE_LOOPBACK
#ifndef UINT32
#define UINT32 unsigned long
#endif
#ifdef DFE_PING_TEST
#include "dsp_xmem_arb_rand_em.h"
#endif
#ifdef DFE_MEM_TEST
#include "aai_mem_test.h"
#endif
#ifdef DFE_ATM_LOOPBACK
#include "aai_lpbk_dyn_rate.h"
#endif
#endif

DSL_DEV_Version_t amazon_se_mei_version = { 
	major:	2,
	minor:	4,
	revision:0
};
DSL_DEV_HwVersion_t amazon_se_chip_info;
#define AMAZON_SE_MEI_DEVNAME "ifx_mei"
#define AMAZON_SE_MAX_DEVICES 1

DSL_DEV_MeiError_t DSL_BSP_FWDownload (DSL_DEV_Device_t *, const char *, unsigned long, long *, long *);
DSL_DEV_MeiError_t DSL_BSP_Showtime (DSL_DEV_Device_t *, DSL_uint32_t, DSL_uint32_t);
DSL_DEV_MeiError_t DSL_BSP_AdslLedInit (DSL_DEV_Device_t *, DSL_DEV_LedId_t, DSL_DEV_LedType_t, DSL_DEV_LedHandler_t);
DSL_DEV_MeiError_t DSL_BSP_AdslLedSet (DSL_DEV_Device_t *, DSL_DEV_LedId_t, DSL_DEV_LedMode_t);
DSL_DEV_MeiError_t DSL_BSP_MemoryDebugAccess (DSL_DEV_Device_t *, DSL_BSP_MemoryAccessType_t, DSL_uint32_t, DSL_uint32_t*, DSL_uint32_t);

int DSL_BSP_KernelIoctls (DSL_DEV_Device_t *, unsigned int, unsigned long);

static DSL_DEV_MeiError_t AMAZON_SE_MEI_RunAdslModem (DSL_DEV_Device_t *);
DSL_DEV_MeiError_t DSL_BSP_SendCMV (DSL_DEV_Device_t *, u16 *, int, u16 *);
static DSL_DEV_MeiError_t AMAZON_SE_MEI_CpuModeSet (DSL_DEV_Device_t *, DSL_DEV_CpuMode_t);
static DSL_DEV_MeiError_t AMAZON_SE_MEI_DownloadBootCode (DSL_DEV_Device_t *);
static DSL_DEV_MeiError_t AMAZON_SE_MEI_ArcJtagEnable (DSL_DEV_Device_t *, int);
static DSL_DEV_MeiError_t AMAZON_SE_MEI_AdslMailboxIRQEnable (DSL_DEV_Device_t *, int);

static int AMAZON_SE_MEI_GetPage (DSL_DEV_Device_t *, u32, u32, u32, u32 *, u32 *);
static int AMAZON_SE_MEI_BarUpdate (DSL_DEV_Device_t *, int);

static ssize_t AMAZON_SE_MEI_Write (DSL_DRV_file_t *, const char *, size_t, loff_t *);
static int AMAZON_SE_MEI_UserIoctls (DSL_DRV_inode_t *, DSL_DRV_file_t *, unsigned int, unsigned long);
static int AMAZON_SE_MEI_Open (DSL_DRV_inode_t *, DSL_DRV_file_t *);
static int AMAZON_SE_MEI_Release (DSL_DRV_inode_t *, DSL_DRV_file_t *);

#ifdef CONFIG_PROC_FS
static int AMAZON_SE_MEI_ProcRead (struct file *, char *, size_t, loff_t *);
static ssize_t AMAZON_SE_MEI_ProcWrite (struct file *, const char *, size_t, loff_t *);
#endif

void AMAZON_SE_MEI_ARC_MUX_Test(void);

static u32 *mei_arc_swap_buff = NULL;	//  holding swap pages

//#if defined(CONFIG_ATM_AMAZON_SE) || defined(CONFIG_ATM_AMAZON_SE_MODULE)
//extern int g_showtime; //defined in amazon_se_mei_bsp.c :Sumedh
int pt_g_showtime=0;
EXPORT_SYMBOL(pt_g_showtime);


int g_showtime=0; //defined in amazon_se_mei_bsp.c :Sumedh
EXPORT_SYMBOL(g_showtime);

//#endif

#define MEI_MASK_AND_ACK_IRQ mask_and_ack_amazon_se_irq

static int dev_major = 105;
#ifdef CONFIG_PROC_FS
	#define PROC_ITEMS 10
	#define MEI_DIRNAME "amazon_se_mei"
#endif /* CONFIG_PROC_FS */

#undef CONFIG_DEVFS_FS
#ifdef CONFIG_DEVFS_FS
static devfs_handle_t amazon_se_mei_devfs_handle[AMAZON_SE_MAX_DEVICES];
#endif
static struct file_operations amazon_se_mei_operations = {
      owner:THIS_MODULE,
      open:AMAZON_SE_MEI_Open,
      release:AMAZON_SE_MEI_Release,
      write:AMAZON_SE_MEI_Write,
      ioctl:AMAZON_SE_MEI_UserIoctls,
};

#ifdef CONFIG_PROC_FS
static struct proc_dir_entry *meidir;
static struct file_operations AMAZON_SE_MEI_ProcOperations = {
      read:AMAZON_SE_MEI_ProcRead,
      write:AMAZON_SE_MEI_ProcWrite,
};
static reg_entry_t regs[AMAZON_SE_MAX_DEVICES][PROC_ITEMS];	//total items to be monitored by /proc/mei
#define NUM_OF_REG_ENTRY	(sizeof(regs[0])/sizeof(reg_entry_t))
#endif //#ifdef CONFIG_PROC_FS

static DSL_DEV_Device_t dsl_devices[AMAZON_SE_MAX_DEVICES];

static amazon_se_mei_device_private_t
	sAmazon_SE_Mei_Private[AMAZON_SE_MAX_DEVICES];

static DSL_BSP_EventCallBack_t dsl_bsp_event_callback[DSL_BSP_CB_LAST + 1];

/////////////////               mei access Rd/Wr methods       ///////////////
/**
 * Write a value to register
 * This function writes a value to amazon_se register
 *
 * \param  	ul_address	The address to write
 * \param  	ul_data		The value to write
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_LongWordWrite (u32 ul_address, u32 ul_data)
{
	AMAZON_SE_WRITE_REGISTER_L (ul_data, ul_address);
	wmb();
	return;
}				//    end of "AMAZON_SE_MEI_LongWordWrite(..."

/**
 * Write a value to register
 * This function writes a value to amazon_se register
 *
 * \param 	pDev		the device pointer
 * \param  	ul_address	The address to write
 * \param  	ul_data		The value to write
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_LongWordWriteOffset (DSL_DEV_Device_t * pDev, u32 ul_address,
				   u32 ul_data)
{
	AMAZON_SE_WRITE_REGISTER_L (ul_data, pDev->base_address + ul_address);
	wmb();
	return;
}				//    end of "AMAZON_SE_MEI_LongWordWrite(..."

/**
 * Read the amazon_se register
 * This function read the value from amazon_se register
 *
 * \param  	ul_address	The address to write
 * \param  	pul_data	Pointer to the data
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_LongWordRead (u32 ul_address, u32 * pul_data)
{
	//*pul_data = *((volatile u32 *)ul_address);
	*pul_data = AMAZON_SE_READ_REGISTER_L (ul_address);
	rmb();
	return;
}				//    end of "AMAZON_SE_MEI_LongWordRead(..."

/**
 * Read the amazon_se register
 * This function read the value from amazon_se register
 *
 * \param 	pDev		the device pointer
 * \param  	ul_address	The address to write
 * \param  	pul_data	Pointer to the data
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_LongWordReadOffset (DSL_DEV_Device_t * pDev, u32 ul_address,
				  u32 * pul_data)
{
	//*pul_data = *((volatile u32 *)ul_address);
	*pul_data = AMAZON_SE_READ_REGISTER_L (pDev->base_address + ul_address);
	rmb();
	return;
}				//    end of "AMAZON_SE_MEI_LongWordRead(..."

/**
 * Write several DWORD datas to ARC memory via ARC DMA interface
 * This function writes several DWORD datas to ARC memory via DMA interface.
 *
 * \param 	pDev		the device pointer
 * \param  	destaddr	The address to write
 * \param  	databuff	Pointer to the data buffer
 * \param  	databuffsize	Number of DWORDs to write
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_DMAWrite (DSL_DEV_Device_t * pDev, u32 destaddr,
			u32 * databuff, u32 databuffsize)
{
	u32 *p = databuff;
	u32 temp;
#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_intstat_t flags;
#endif

	if (destaddr & 3)
		return DSL_DEV_MEI_ERR_FAILURE;

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_LOCKINT (flags);
#endif

	//      Set the write transfer address
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, MEI_XFR_ADDR_OFFSET,
					   destaddr);

	//      Write the data pushed across DMA
	while (databuffsize--) {
		temp = *p;
		if (destaddr == MEI_TO_ARC_MAILBOX)
			MEI_HALF_WORD_SWAP (temp);
		AMAZON_SE_MEI_LongWordWriteOffset (pDev,
						   (u32) MEI_DATA_XFR_OFFSET,
						   temp);
		p++;
	}			//    end of "while(..."

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_UNLOCKINT (flags);
#endif

	return DSL_DEV_MEI_ERR_SUCCESS;

}				//    end of "AMAZON_SE_MEI_DMAWrite(..."

/**
 * Read several DWORD datas from ARC memory via ARC DMA interface
 * This function reads several DWORD datas from ARC memory via DMA interface.
 *
 * \param 	pDev		the device pointer
 * \param  	srcaddr		The address to read
 * \param  	databuff	Pointer to the data buffer
 * \param  	databuffsize	Number of DWORDs to read
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_DMARead (DSL_DEV_Device_t * pDev, u32 srcaddr, u32 * databuff,
		       u32 databuffsize)
{
	u32 *p = databuff;
	u32 temp;
#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_intstat_t flags;
#endif
	if (srcaddr & 3)
		return DSL_DEV_MEI_ERR_FAILURE;

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_LOCKINT (flags);
#endif

	//      Set the read transfer address
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_XFR_ADDR_OFFSET,
					   srcaddr);

	//      Read the data popped across DMA
	while (databuffsize--) {
		AMAZON_SE_MEI_LongWordReadOffset (pDev,
						  (u32) MEI_DATA_XFR_OFFSET,
						  &temp);
		if (databuff == (u32 *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->CMV_RxMsg)	// swap half word
			MEI_HALF_WORD_SWAP (temp);
		*p = temp;
		p++;
	}			//    end of "while(..."

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_UNLOCKINT (flags);
#endif

	return DSL_DEV_MEI_ERR_SUCCESS;

}				//    end of "AMAZON_SE_MEI_DMARead(..."

/**
 * Switch the ARC control mode
 * This function switchs the ARC control mode to JTAG mode or MEI mode
 *
 * \param 	pDev		the device pointer
 * \param  	mode		The mode want to switch: JTAG_MASTER_MODE or MEI_MASTER_MODE.
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_ControlModeSet (DSL_DEV_Device_t * pDev, int mode)
{
	u32 temp = 0x0;
	AMAZON_SE_MEI_LongWordReadOffset (pDev, (u32) MEI_DBG_MASTER_OFFSET,
					  &temp);
	switch (mode) {
	case JTAG_MASTER_MODE:
		temp &= ~(HOST_MSTR);
		break;
	case MEI_MASTER_MODE:
		temp |= (HOST_MSTR);
		break;
	default:
		AMAZON_SE_MEI_EMSG
			("AMAZON_SE_MEI_ControlModeSet: unkonwn mode [%d]\n",
			 mode);
		return;
	}
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DBG_MASTER_OFFSET,
					   temp);
}

/**
 * Disable ARC to MEI interrupt
 *
 * \param 	pDev		the device pointer
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_IRQDisable (DSL_DEV_Device_t * pDev)
{
	AMAZON_SE_MEI_LongWordWriteOffset (pDev,
					   (u32) ARC_TO_MEI_INT_MASK_OFFSET,
					   0x0);
}				//    end of "AMAZON_SE_MEI_IRQDisable(..."

/**
 * Eable ARC to MEI interrupt
 *
 * \param 	pDev		the device pointer
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_IRQEnable (DSL_DEV_Device_t * pDev)
{
	AMAZON_SE_MEI_LongWordWriteOffset (pDev,
					   (u32) ARC_TO_MEI_INT_MASK_OFFSET,
					   MSGAV_EN);
}				//    end of "AMAZON_SE_MEI_IRQEnable(..."

/**
 * Poll for transaction complete signal
 * This function polls and waits for transaction complete signal.
 *
 * \param 	pDev		the device pointer
 * \ingroup	Internal
 */
static void
meiPollForDbgDone (DSL_DEV_Device_t * pDev)
{
	u32 query = 0;
	int i = 0;
	while (i < WHILE_DELAY) {
		AMAZON_SE_MEI_LongWordReadOffset (pDev,
						  (u32) ARC_TO_MEI_INT_OFFSET,
						  &query);
		query &= (ARC_TO_MEI_DBG_DONE);
		if (query)
			break;
		i++;
		if (i == WHILE_DELAY) {
			AMAZON_SE_MEI_EMSG ("PollforDbg fail!\n");
		}
	}
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_DBG_DONE);	// to clear this interrupt
}				//    end of "meiPollForDbgDone(..."

/**
 * ARC Debug Memory Access for a single DWORD reading.
 * This function used for direct, address-based access to ARC memory.
 *
 * \param 	pDev		the device pointer
 * \param  	DEC_mode	ARC memory space to used
 * \param  	address	  	Address to read
 * \param  	data	  	Pointer to data
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
_AMAZON_SE_MEI_DBGLongWordRead (DSL_DEV_Device_t * pDev, u32 DEC_mode,
				u32 address, u32 * data)
{
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DEBUG_DEC_OFFSET,
					   DEC_mode);
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DEBUG_RAD_OFFSET,
					   address);
	meiPollForDbgDone (pDev);
	AMAZON_SE_MEI_LongWordReadOffset (pDev, (u32) MEI_DEBUG_DATA_OFFSET,
					  data);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
 * ARC Debug Memory Access for a single DWORD writing.
 * This function used for direct, address-based access to ARC memory.
 *
 * \param 	pDev		the device pointer
 * \param  	DEC_mode	ARC memory space to used
 * \param  	address	  	The address to write
 * \param  	data	  	The data to write
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
_AMAZON_SE_MEI_DBGLongWordWrite (DSL_DEV_Device_t * pDev, u32 DEC_mode,
				 u32 address, u32 data)
{
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DEBUG_DEC_OFFSET,
					   DEC_mode);
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DEBUG_WAD_OFFSET,
					   address);
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DEBUG_DATA_OFFSET,
					   data);
	meiPollForDbgDone (pDev);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
 * ARC Debug Memory Access for writing.
 * This function used for direct, address-based access to ARC memory.
 *
 * \param 	pDev		the device pointer
 * \param  	destaddr	The address to read
 * \param  	databuffer  	Pointer to data
 * \param	databuffsize	The number of DWORDs to read
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */

static DSL_DEV_MeiError_t
AMAZON_SE_MEI_DebugWrite (DSL_DEV_Device_t * pDev, u32 destaddr,
			  u32 * databuff, u32 databuffsize)
{
	u32 i;
	u32 temp = 0x0;
	u32 address = 0x0;
	u32 *buffer = 0x0;
#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_intstat_t flags;
#endif

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_LOCKINT (flags);
#endif

	//      Open the debug port before DMP memory write
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);

	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DEBUG_DEC_OFFSET,
					   MEI_DEBUG_DEC_DMP1_MASK);

	//      For the requested length, write the address and write the data
	address = destaddr;
	buffer = databuff;
	for (i = 0; i < databuffsize; i++) {
		temp = *buffer;
		_AMAZON_SE_MEI_DBGLongWordWrite (pDev,
						 MEI_DEBUG_DEC_DMP1_MASK,
						 address, temp);
		address += 4;
		buffer++;
	}			//    end of "for(..."

	//      Close the debug port after DMP memory write
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_UNLOCKINT (flags);
#endif

	//      Return
	return DSL_DEV_MEI_ERR_SUCCESS;

}				//    end of "AMAZON_SE_MEI_DebugWrite(..."

/**
 * ARC Debug Memory Access for reading.
 * This function used for direct, address-based access to ARC memory.
 *
 * \param 	pDev		the device pointer
 * \param  	srcaddr	  	The address to read
 * \param  	databuffer  	Pointer to data
 * \param	databuffsize	The number of DWORDs to read
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_DebugRead (DSL_DEV_Device_t * pDev, u32 srcaddr, u32 * databuff, u32 databuffsize)
{
	u32 i;
	u32 temp = 0x0;
	u32 address = 0x0;
	u32 *buffer = 0x0;
#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_intstat_t flags;
#endif

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_LOCKINT (flags);
#endif

	//      Open the debug port before DMP memory read
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);

	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_DEBUG_DEC_OFFSET,
					   MEI_DEBUG_DEC_DMP1_MASK);

	//      For the requested length, write the address and read the data
	address = srcaddr;
	buffer = databuff;
	for (i = 0; i < databuffsize; i++) {
		_AMAZON_SE_MEI_DBGLongWordRead (pDev, MEI_DEBUG_DEC_DMP1_MASK,
						address, &temp);
		*buffer = temp;
		address += 4;
		buffer++;
	}			//    end of "for(..."

	//      Close the debug port after DMP memory read
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);

#ifdef AMAZON_SE_DMA_DEBUG_MUTEX
	MEI_UNLOCKINT (flags);
#endif

	//      Return
	return DSL_DEV_MEI_ERR_SUCCESS;

}				//    end of "AMAZON_SE_MEI_DebugRead(..."

/**
 * Send a message to ARC MailBox.
 * This function sends a message to ARC Mailbox via ARC DMA interface.
 *
 * \param 	pDev		the device pointer
 * \param  	msgsrcbuffer  	Pointer to message.
 * \param	msgsize		The number of words to write.
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_MailboxWrite (DSL_DEV_Device_t * pDev, u16 * msgsrcbuffer,
			    u16 msgsize)
{
	int i;
	u32 arc_mailbox_status = 0x0;
	u32 temp = 0;
	DSL_DEV_MeiError_t meiMailboxError = DSL_DEV_MEI_ERR_SUCCESS;

	//      Write to mailbox
	meiMailboxError =
		AMAZON_SE_MEI_DMAWrite (pDev, MEI_TO_ARC_MAILBOX,
					(u32 *) msgsrcbuffer, msgsize / 2);
	meiMailboxError =
		AMAZON_SE_MEI_DMAWrite (pDev, MEI_TO_ARC_MAILBOXR,
					(u32 *) (&temp), 1);

	//      Notify arc that mailbox write completed
	((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_waiting = 1;
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_TO_ARC_INT_OFFSET,
					   MEI_TO_ARC_MSGAV);

	i = 0;
	while (i < WHILE_DELAY) {	// wait for ARC to clear the bit
		AMAZON_SE_MEI_LongWordReadOffset (pDev,
						  (u32) MEI_TO_ARC_INT_OFFSET,
						  &arc_mailbox_status);
		if ((arc_mailbox_status & MEI_TO_ARC_MSGAV) != MEI_TO_ARC_MSGAV)
			break;
		i++;
		if (i == WHILE_DELAY) {
			AMAZON_SE_MEI_EMSG (">>> Timeout waiting for ARC to clear MEI_TO_ARC_MSGAV!!! MEI_TO_ARC message size = %d DWORDs <<<\n", msgsize/2);
			meiMailboxError = DSL_DEV_MEI_ERR_FAILURE;
		}
	}

	return meiMailboxError;
}

/**
 * Read a message from ARC MailBox.
 * This function reads a message from ARC Mailbox via ARC DMA interface.
 *
 * \param 	pDev		the device pointer
 * \param  	msgsrcbuffer  	Pointer to message.
 * \param	msgsize		The number of words to read
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_MailboxRead (DSL_DEV_Device_t * pDev, u16 * msgdestbuffer,
			   u16 msgsize)
{
	DSL_DEV_MeiError_t meiMailboxError = DSL_DEV_MEI_ERR_SUCCESS;
	//      Read from mailbox
	meiMailboxError =
		AMAZON_SE_MEI_DMARead (pDev, ARC_TO_MEI_MAILBOX,
				       (u32 *) msgdestbuffer, msgsize / 2);

	//      Notify arc that mailbox read completed
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) ARC_TO_MEI_INT_OFFSET,
					   ARC_TO_MEI_MSGAV);

	return meiMailboxError;
}

/**
 * Download boot pages to ARC.
 * This function downloads boot pages to ARC.
 *
 * \param 	pDev		the device pointer
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_DownloadBootPages (DSL_DEV_Device_t * pDev)
{
	int boot_loop;
	int page_size;
	u32 dest_addr;

	/*
	 **     DMA the boot code page(s)
	 */

	for (boot_loop = 1;
	     boot_loop <
	     (((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr-> count); boot_loop++) {
		if ((((amazon_se_mei_device_private_t *) pDev->pPriv)-> img_hdr->page[boot_loop].p_size) & BOOT_FLAG) {
			page_size = AMAZON_SE_MEI_GetPage (pDev, boot_loop,
						       GET_PROG, MAXSWAPSIZE,
						       mei_arc_swap_buff,
						       &dest_addr);
			if (page_size > 0) {
				AMAZON_SE_MEI_DMAWrite (pDev, dest_addr,
							mei_arc_swap_buff,
							page_size);
			}
		}
		if ((((amazon_se_mei_device_private_t *) pDev->pPriv)-> img_hdr->page[boot_loop].d_size) & BOOT_FLAG) {
			page_size = AMAZON_SE_MEI_GetPage (pDev, boot_loop,
						       GET_DATA, MAXSWAPSIZE,
						       mei_arc_swap_buff,
						       &dest_addr);
			if (page_size > 0) {
				AMAZON_SE_MEI_DMAWrite (pDev, dest_addr,
							mei_arc_swap_buff,
							page_size);
			}
		}
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
 * Initial efuse rar.
 **/
static void
AMAZON_SE_MEI_FuseInit (DSL_DEV_Device_t * pDev)
{
#if 1
	u32 data = 0;
	AMAZON_SE_MEI_DMAWrite (pDev, IRAM0_BASE, &data, 1);
	AMAZON_SE_MEI_DMAWrite (pDev, IRAM0_BASE + 4, &data, 1);
	AMAZON_SE_MEI_DMAWrite (pDev, IRAM1_BASE, &data, 1);
	AMAZON_SE_MEI_DMAWrite (pDev, IRAM1_BASE + 4, &data, 1);
	AMAZON_SE_MEI_DMAWrite (pDev, BRAM_BASE, &data, 1);
	AMAZON_SE_MEI_DMAWrite (pDev, BRAM_BASE + 4, &data, 1);
	AMAZON_SE_MEI_DMAWrite (pDev, ADSL_DILV_BASE, &data, 1);
	AMAZON_SE_MEI_DMAWrite (pDev, ADSL_DILV_BASE + 4, &data, 1);
#endif
}

/**
 * efuse rar program
 **/
static void
AMAZON_SE_MEI_FuseProg (DSL_DEV_Device_t * pDev)
{
#if 1
	u32 reg_data, fuse_value;
	int i = 0;
	AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ, &reg_data);
	while ((reg_data & 0x10000000) == 0) {
		AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ,
					    &reg_data);
		//add a watchdog
		i++;
		/* 0x4000 translate to  about 16 ms@111M, so should be enough */
		if (i == 0x4000)
			return;
	}
	// STEP a: Prepare memory for external accesses
	// Write fuse_en bit24
	AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ, &reg_data);
	AMAZON_SE_MEI_LongWordWrite ((u32) AMAZON_SE_RCU_REQ, reg_data | (1 << 24));

	AMAZON_SE_MEI_FuseInit (pDev);
	for (i = 0; i < 4; i++) {
		AMAZON_SE_MEI_LongWordRead ((u32) (AMAZON_SE_FUSE_BASE_ADDR) +
					    i * 4, &fuse_value);
		switch (fuse_value & 0xF0000) {
		case 0x80000:
			reg_data =
				((fuse_value & RX_DILV_ADDR_BIT_MASK) |
				 (RX_DILV_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, ADSL_DILV_BASE,
						&reg_data, 1);
			break;
		case 0x90000:
			reg_data =
				((fuse_value & RX_DILV_ADDR_BIT_MASK) |
				 (RX_DILV_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, ADSL_DILV_BASE + 4,
						&reg_data, 1);
			break;
		case 0xA0000:
			reg_data =
				((fuse_value & IRAM0_ADDR_BIT_MASK) |
				 (IRAM0_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, IRAM0_BASE, &reg_data,
						1);
			break;
		case 0xB0000:
			reg_data =
				((fuse_value & IRAM0_ADDR_BIT_MASK) |
				 (IRAM0_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, IRAM0_BASE + 4,
						&reg_data, 1);
			break;
		case 0xC0000:
			reg_data =
				((fuse_value & IRAM1_ADDR_BIT_MASK) |
				 (IRAM1_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, IRAM1_BASE, &reg_data,
						1);
			break;
		case 0xD0000:
			reg_data =
				((fuse_value & IRAM1_ADDR_BIT_MASK) |
				 (IRAM1_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, IRAM1_BASE + 4,
						&reg_data, 1);
			break;
		case 0xE0000:
			reg_data =
				((fuse_value & BRAM_ADDR_BIT_MASK) |
				 (BRAM_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, BRAM_BASE, &reg_data,
						1);
			break;
		case 0xF0000:
			reg_data =
				((fuse_value & BRAM_ADDR_BIT_MASK) |
				 (BRAM_ADDR_BIT_MASK + 0x1));
			AMAZON_SE_MEI_DMAWrite (pDev, BRAM_BASE + 4,
						&reg_data, 1);
			break;
		default:	// PPE efuse
			break;
		}
	}
	AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ, &reg_data);
	AMAZON_SE_MEI_LongWordWrite ((u32) AMAZON_SE_RCU_REQ, reg_data & ~(1 << 24));
	AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ, &reg_data);
#endif
}

/**
 * Enable DFE Clock
 * This function enables DFE Clock
 *
 * \param 	pDev		the device pointer
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_EnableCLK (DSL_DEV_Device_t * pDev)
{
	u32 arc_debug_data = 0;
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);
	//enable ac_clk signal
	_AMAZON_SE_MEI_DBGLongWordRead (pDev, MEI_DEBUG_DEC_DMP1_MASK,
					CRI_CCR0, &arc_debug_data);
	arc_debug_data |= ACL_CLK_MODE_ENABLE;
	_AMAZON_SE_MEI_DBGLongWordWrite (pDev, MEI_DEBUG_DEC_DMP1_MASK,
					 CRI_CCR0, arc_debug_data);
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
 * Halt the ARC.
 * This function halts the ARC.
 *
 * \param 	pDev		the device pointer
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_HaltArc (DSL_DEV_Device_t * pDev)
{
	u32 arc_debug_data = 0x0;

	//      Switch arc control from JTAG mode to MEI mode
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);
	_AMAZON_SE_MEI_DBGLongWordRead (pDev, MEI_DEBUG_DEC_AUX_MASK,
					ARC_DEBUG, &arc_debug_data);
	arc_debug_data |= ARC_DEBUG_HALT;
	_AMAZON_SE_MEI_DBGLongWordWrite (pDev, MEI_DEBUG_DEC_AUX_MASK,
					 ARC_DEBUG, arc_debug_data);
	//      Switch arc control from MEI mode to JTAG mode
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);

	MEI_WAIT (10);
	//      Return
	return DSL_DEV_MEI_ERR_SUCCESS;

}				//    end of "meiHalt(..."

/**
 * Run the ARC.
 * This function runs the ARC.
 *
 * \param 	pDev		the device pointer
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_RunArc (DSL_DEV_Device_t * pDev)
{
	u32 arc_debug_data = 0x0;

	//      Switch arc control from JTAG mode to MEI mode- write '1' to bit0
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);
	_AMAZON_SE_MEI_DBGLongWordRead (pDev, MEI_DEBUG_DEC_AUX_MASK,
					AUX_STATUS, &arc_debug_data);

	//      Write debug data reg with content ANDd with 0xFDFFFFFF (halt bit cleared)
	arc_debug_data &= ~ARC_AUX_HALT;
	_AMAZON_SE_MEI_DBGLongWordWrite (pDev, MEI_DEBUG_DEC_AUX_MASK,
					 AUX_STATUS, arc_debug_data);

	//      Switch arc control from MEI mode to JTAG mode- write '0' to bit0
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);
	//      Enable mask for arc codeswap interrupts
	AMAZON_SE_MEI_IRQEnable (pDev);

	//      Return
	return DSL_DEV_MEI_ERR_SUCCESS;

}				//    end of "meiActivate(..."
#define FPI_BUG_FIX
/**
 * Reset the ARC.
 * This function resets the ARC.
 *
 * \param 	pDev		the device pointer
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_ResetARC (DSL_DEV_Device_t * pDev)
{
	u32 arc_debug_data = 0, bar, i;

	AMAZON_SE_MEI_HaltArc (pDev);
	#ifdef FPI_BUG_FIX
	for (i = 0; i < 5; i++) {
		AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ, &arc_debug_data);
		AMAZON_SE_MEI_LongWordWrite ((u32) AMAZON_SE_RCU_REQ,
					     arc_debug_data | DANBUE_RCU_RST_REQ_DFE | DANBUE_RCU_RST_REQ_AFE);
		do {
			AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_STAT, &arc_debug_data);
		} while ((arc_debug_data & (DANBUE_RCU_RST_REQ_DFE | DANBUE_RCU_RST_REQ_AFE)) !=
			(DANBUE_RCU_RST_REQ_DFE | DANBUE_RCU_RST_REQ_AFE));
		AMAZON_SE_MEI_LongWordReadOffset (pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET, &bar);
		if (bar == 0x1FFFFFE0)
			break;
	}
	if (i >= 5)
		AMAZON_SE_MEI_EMSG ("AMAZON_SE_MEI_ResetARC failed!\n");
	#else
		AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ, &arc_debug_data);
		AMAZON_SE_MEI_LongWordWrite ((u32) AMAZON_SE_RCU_REQ,
					     arc_debug_data | DANBUE_RCU_RST_REQ_DFE | DANBUE_RCU_RST_REQ_AFE);
	#endif
	// reset ARC
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_RST_CONTROL_OFFSET, MEI_SOFT_RESET);
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_RST_CONTROL_OFFSET, 0);

	AMAZON_SE_MEI_IRQDisable (pDev);

	AMAZON_SE_MEI_EnableCLK (pDev);

#if 0
	// reset part of PPE
	*(unsigned long *) (AMAZON_SE_PPE32_SRST) = 0xC30;
	*(unsigned long *) (AMAZON_SE_PPE32_SRST) = 0xFFF;
#endif

	((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready = 0;

	return DSL_DEV_MEI_ERR_SUCCESS;
}

DSL_DEV_MeiError_t
DSL_BSP_Showtime (DSL_DEV_Device_t * dev, DSL_uint32_t rate_fast, DSL_uint32_t rate_intl)
{
        if (pt_g_showtime==0)
                return DSL_DEV_MEI_ERR_FAILURE;

#if defined(CONFIG_ATM_AMAZON_SE) && defined (CONFIG_IFX_DSL_CPE_API)

	AMAZON_SE_MEI_EMSG ("Datarate US intl = %d, fast = %d\n", rate_intl,
			    rate_fast);
	if (rate_intl && rate_fast) {
		ifx_atm_set_cell_rate (0, rate_fast / (53 * 8));
		ifx_atm_set_cell_rate (1, rate_intl / (53 * 8));
	}
	else if (rate_fast) {
		ifx_atm_set_cell_rate (0, rate_fast / (53 * 8));
	}
	else if (rate_intl) {
		ifx_atm_set_cell_rate (0, rate_intl / (53 * 8));
	}
	else {
		AMAZON_SE_MEI_EMSG ("Got rate fail.\n");
	}
#endif
	/*Sumedh: crash handle: set global variable, use from ppe driver */
#if defined(CONFIG_ATM_AMAZON_SE) || defined(CONFIG_ATM_AMAZON_SE_MODULE)
	g_showtime=1;
	printk("g_showtime set to 1\n");
#endif
	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
 * Reset/halt/run the DFE.
 * This function provide operations to reset/halt/run the DFE.
 *
 * \param 	pDev		the device pointer
 * \param	mode		which operation want to do
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_CpuModeSet (DSL_DEV_Device_t *pDev,
			  DSL_DEV_CpuMode_t mode)
{
	DSL_DEV_MeiError_t err_ret = DSL_DEV_MEI_ERR_FAILURE;
	switch (mode) {
	case DSL_CPU_HALT:
		err_ret = AMAZON_SE_MEI_HaltArc (pDev);
		break;
	case DSL_CPU_RUN:
		err_ret = AMAZON_SE_MEI_RunArc (pDev);
		break;
	case DSL_CPU_RESET:
		err_ret = AMAZON_SE_MEI_ResetARC (pDev);
		break;
	default:
		break;
	}
	return err_ret;
}

/**
 * Accress DFE memory.
 * This function provide a way to access DFE memory;
 *
 * \param 	pDev		the device pointer
 * \param	type		read or write
 * \param	destaddr	destination address
 * \param	databuff	pointer to hold data
 * \param	databuffsize	size want to read/write
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
DSL_DEV_MeiError_t
DSL_BSP_MemoryDebugAccess (DSL_DEV_Device_t * pDev,
				DSL_BSP_MemoryAccessType_t type,
				DSL_uint32_t destaddr, DSL_uint32_t *databuff,
				DSL_uint32_t databuffsize)
{
	DSL_DEV_MeiError_t meierr = DSL_DEV_MEI_ERR_SUCCESS;
	switch (type) {
	case DSL_BSP_MEMORY_READ:
		meierr = AMAZON_SE_MEI_DebugRead (pDev, destaddr, databuff,
						  databuffsize);
		break;
	case DSL_BSP_MEMORY_WRITE:
		meierr = AMAZON_SE_MEI_DebugWrite (pDev, destaddr, databuff,
						   databuffsize);
		break;
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
 * Download boot code to ARC.
 * This function downloads boot code to ARC.
 *
 * \param 	pDev		the device pointer
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_DownloadBootCode (DSL_DEV_Device_t *pDev)
{
	AMAZON_SE_MEI_IRQDisable (pDev);

	AMAZON_SE_MEI_EnableCLK (pDev);

	AMAZON_SE_MEI_FuseProg (pDev);	//program fuse rar

	AMAZON_SE_MEI_DownloadBootPages (pDev);

	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
 * Enable Jtag debugger interface
 * This function setups mips gpio to enable jtag debugger
 *
 * \param 	pDev		the device pointer
 * \param 	enable		enable or disable
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_ArcJtagEnable (DSL_DEV_Device_t *dev, int enable)
{
	int meierr;
	u32 reg_data;

	switch (enable) {
	case 1:
		//reserve gpio 7, 8, 9, 10, 11  for ARC JTAG
		meierr = amazon_se_port_reserve_pin (0, 7,
						     PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 7 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_reserve_pin (0, 8,
						     PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 8 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_reserve_pin (0, 9,
						     PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 9 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_reserve_pin (0, 10,
						     PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 10 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_reserve_pin (1, 11,
						     PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 11 Fail.\n");
			goto jtag_end;
		}
		// set gpio11 as output, gpio 7,8,9,10 as input
		*(AMAZON_SE_GPIO_P0_DIR) &= ~((1 << 10) | (1 << 9) | (1 << 8) | (1 << 7)); 
		*(AMAZON_SE_GPIO_P0_DIR) |= (1 << 11);	

		*(AMAZON_SE_GPIO_P0_ALTSEL0) |= (1 << 7) | (1 << 8) | (1 << 9) | (1 << 10) | (1 << 11);
		#if 0
		*(AMAZON_SE_GPIO_P0_ALTSEL1) |= (1 << 7) | (1 << 8) | (1 << 9) | (1 << 10);
		*(AMAZON_SE_GPIO_P0_ALTSEL1) &= ~(1 << 11);
		#endif
		*(AMAZON_SE_GPIO_P0_ALTSEL1) |= (1 << 7) | (1 << 8) | (1 << 9) | (1 << 10) | (1 << 11);
		//*AMAZON_SE_GPIO_P0_OD = (*AMAZON_SE_GPIO_P0_OD) | 0x800;

		//enable ARC JTAG
		AMAZON_SE_MEI_LongWordRead ((u32) AMAZON_SE_RCU_REQ,
					    &reg_data);
		AMAZON_SE_MEI_LongWordWrite ((u32) AMAZON_SE_RCU_REQ,
					     reg_data |
					     AMAZON_SE_RCU_RST_REQ_ARC_JTAG);
		printk(KERN_INFO "%s %s %d. AMAZON_SE_GPIO_P0_DIR %#x, AMAZON_SE_GPIO_P0_ALTSEL0 %#x, AMAZON_SE_GPIO_P0_ALTSEL1 %#x\n", __FILE__, __func__, __LINE__, *AMAZON_SE_GPIO_P0_DIR, *AMAZON_SE_GPIO_P0_ALTSEL0, *AMAZON_SE_GPIO_P0_ALTSEL1);
		break;
	case 0:
	default:
		//reserve gpio 9, 10, 11, 14, 19 for ARC JTAG
		meierr = amazon_se_port_free_pin (0, 9, PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 9 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_free_pin (0, 10,
						  PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 10 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_free_pin (0, 11,
						  PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 11 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_free_pin (0, 14,
						  PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 14 Fail.\n");
			goto jtag_end;
		}
		meierr = amazon_se_port_free_pin (1, 3, PORT_MODULE_MEI_JTAG);
		if (meierr < 0) {
			AMAZON_SE_MEI_EMSG ("Reserve GPIO 19 Fail.\n");
			goto jtag_end;
		}
		break;
	}

      jtag_end:
	if (meierr)
		return DSL_DEV_MEI_ERR_FAILURE;
	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
 * Enable DFE to MIPS interrupt
 * This function enable DFE to MIPS interrupt
 *
 * \param 	pDev		the device pointer
 * \param 	enable		enable or disable
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_AdslMailboxIRQEnable (DSL_DEV_Device_t *pDev, int enable)
{
	DSL_DEV_MeiError_t meierr;
	switch (enable) {
	case 0:
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
		AMAZON_SE_MEI_IRQDisable (pDev);
		break;
	case 1:
		AMAZON_SE_MEI_IRQEnable (pDev);
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
		break;
	default:
		meierr = DSL_DEV_MEI_ERR_FAILURE;
		break;

	}
	return meierr;
}

/**
 * Get the modem status
 * This function return the modem status
 *
 * \param 	pDev		the device pointer
 * \return	1: modem ready 0: not ready
 * \ingroup	Internal
 */
static int
AMAZON_SE_MEI_IsModemReady (DSL_DEV_Device_t * pDev)
{
	return ((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready;
}

DSL_DEV_MeiError_t
DSL_BSP_AdslLedInit (DSL_DEV_Device_t * dev,
			  DSL_DEV_LedId_t led_number,
			  DSL_DEV_LedType_t type,
			  DSL_DEV_LedHandler_t handler)
{
	u32 arc_debug_data = 0x0, i;
	if (led_number == DSL_LED_LINK_ID && type == DSL_LED_LINK_TYPE && handler == DSL_LED_HD_CPU) {
#ifdef CONFIG_AMAZON_SE_LED
		struct led_config_param param;
		param.operation_mask = CONFIG_OPERATION_UPDATE_SOURCE;
		param.led = 0x03;
		param.source = 0x03;
		printk(KERN_INFO "[%s %d]: -> amazon_se_led_config()\n", __func__, __LINE__);
		amazon_se_led_config (&param);

#endif
	} else if (led_number == DSL_LED_DATA_ID && type == DSL_LED_DATA_TYPE &&
		(handler == DSL_LED_HD_CPU || handler == DSL_LED_HD_FW)) {
		#if 0
		/* Enable ADSL LEDs by access to AUX_ARC_GPIO_CTRL Auxiliary register.
		 * Note that this can only be done when the ARC is idle */
		AMAZON_SE_MEI_ControlModeSet (dev, MEI_MASTER_MODE);
		_AMAZON_SE_MEI_DBGLongWordRead (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_CTRL, &arc_debug_data);
		AMAZON_SE_MEI_DMSG("ARC_GPIO_CTRL %#x\n", arc_debug_data);
		arc_debug_data |= (3 << 9);
		_AMAZON_SE_MEI_DBGLongWordWrite (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_CTRL, arc_debug_data);
		_AMAZON_SE_MEI_DBGLongWordRead (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_CTRL, &arc_debug_data);
		AMAZON_SE_MEI_DMSG("ARC_GPIO_CTRL %#x\n", arc_debug_data);
		#if 0
		for (i = 0; i < 10; i++) {
			_AMAZON_SE_MEI_DBGLongWordRead (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, &arc_debug_data);
			arc_debug_data |= (1 << 9);
			_AMAZON_SE_MEI_DBGLongWordWrite (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, arc_debug_data);
			mdelay(500);
			arc_debug_data &= ~(1 << 9);
			_AMAZON_SE_MEI_DBGLongWordWrite (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, arc_debug_data);
			mdelay(500);
		}
		for (i = 0; i < 10; i++) {
			_AMAZON_SE_MEI_DBGLongWordRead (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, &arc_debug_data);
			arc_debug_data |= (2 << 9);
			_AMAZON_SE_MEI_DBGLongWordWrite (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, arc_debug_data);
			mdelay(500);
			arc_debug_data &= ~(2 << 9);
			_AMAZON_SE_MEI_DBGLongWordWrite (dev, MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, arc_debug_data);
			mdelay(500);
		}
		#endif
		AMAZON_SE_MEI_ControlModeSet (dev, JTAG_MASTER_MODE);
		#endif
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
};

DSL_DEV_MeiError_t
DSL_BSP_AdslLedSet (DSL_DEV_Device_t * dev, DSL_DEV_LedId_t led_number, DSL_DEV_LedMode_t mode)
{
	printk(KERN_INFO "[%s %d]: mode = %#x, led_number = %d\n", __func__, __LINE__, mode, led_number);
	switch (mode) {
	case DSL_LED_OFF:
		switch (led_number) {
		case DSL_LED_LINK_ID:
#ifdef CONFIG_AMAZON_SE_LED
			amazon_se_led_set_blink (1, 0);
			amazon_se_led_set_data (1, 0);
#endif
			break;
		case DSL_LED_DATA_ID:
#ifdef CONFIG_AMAZON_SE_LED
			amazon_se_led_set_blink (0, 0);
			amazon_se_led_set_data (0, 0);
#endif
			break;
		}
		break;
	case DSL_LED_FLASH:
		switch (led_number) {
		case DSL_LED_LINK_ID:
#ifdef CONFIG_AMAZON_SE_LED
			amazon_se_led_set_blink (1, 1);	// data
#endif
			break;
		case DSL_LED_DATA_ID:
#ifdef CONFIG_AMAZON_SE_LED
			amazon_se_led_set_blink (0, 1);	// data 
#endif
			break;
		}
		break;
	case DSL_LED_ON:
		switch (led_number) {
		case DSL_LED_LINK_ID:
#ifdef CONFIG_AMAZON_SE_LED
			amazon_se_led_set_blink (1, 0);
			amazon_se_led_set_data (1, 1);
#endif
			break;
		case DSL_LED_DATA_ID:
#ifdef CONFIG_AMAZON_SE_LED
			amazon_se_led_set_blink (0, 0);
			amazon_se_led_set_data (0, 1);
#endif
			break;
		}
		break;
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
};

////////////////makeCMV(Opcode, Group, Address, Index, Size, Data), CMV in u16 TxMessage[MSG_LENGTH]///////////////////////////
/**
* Compose a message.
* This function compose a message from opcode, group, address, index, size, and data
*
* \param       opcode          The message opcode
* \param       group           The message group number
* \param       address         The message address.
* \param       index           The message index.
* \param       size            The number of words to read/write.
* \param       data            The pointer to data.
* \param       CMVMSG          The pointer to message buffer.
* \ingroup     Internal
*/
void
makeCMV (u8 opcode, u8 group, u16 address, u16 index, int size, u16 * data, u16 *CMVMSG)
{
        memset (CMVMSG, 0, MSG_LENGTH * 2);
        CMVMSG[0] = (opcode << 4) + (size & 0xf);
        CMVMSG[1] = (((index == 0) ? 0 : 1) << 7) + (group & 0x7f);
        CMVMSG[2] = address;
        CMVMSG[3] = index;
        if (opcode == H2D_CMV_WRITE)
                memcpy (CMVMSG + 4, data, size * 2);
        return;
}

/**
 * Send a message to ARC and read the response
 * This function sends a message to arc, waits the response, and reads the responses.
 *
 * \param 	pDev		the device pointer
 * \param	request		Pointer to the request
 * \param	reply		Wait reply or not.
 * \param	response	Pointer to the response
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
DSL_DEV_MeiError_t
DSL_BSP_SendCMV (DSL_DEV_Device_t * pDev, u16 * request, int reply, u16 * response)	// write cmv to arc, if reply needed, wait for reply
{
	DSL_DEV_MeiError_t meierror;
#if defined(AMAZON_SE_PORT_RTEMS)
	int delay_counter = 0;
#endif

	if (MEI_MUTEX_LOCK (((amazon_se_mei_device_private_t *) pDev->pPriv)->mei_cmv_sema))
		return -ERESTARTSYS;

	((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_reply = reply;
	memset (((amazon_se_mei_device_private_t *) pDev->pPriv)->CMV_RxMsg, 0,
		sizeof (((amazon_se_mei_device_private_t *) pDev->pPriv)->
			CMV_RxMsg));
	((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav = 0;

	meierror = AMAZON_SE_MEI_MailboxWrite (pDev, request, MSG_LENGTH);

	if (meierror != DSL_DEV_MEI_ERR_SUCCESS) {
		((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_waiting = 0;
		((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav = 0;
		AMAZON_SE_MEI_EMSG ("MailboxWrite Fail!\n");
		AMAZON_SE_MEI_EMSG ("Resetting ARC...\n");
		AMAZON_SE_MEI_ResetARC(pDev);
		MEI_MUTEX_UNLOCK (((amazon_se_mei_device_private_t *) pDev->pPriv)->mei_cmv_sema);
		return meierror;
	}
	else {
		((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_count++;
	}

	if (((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_reply ==
	    NO_REPLY) {
		MEI_MUTEX_UNLOCK (((amazon_se_mei_device_private_t *) pDev->pPriv)->mei_cmv_sema);
		return DSL_DEV_MEI_ERR_SUCCESS;
	}

#if !defined(AMAZON_SE_PORT_RTEMS)
	if (((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav == 0)
		MEI_WAIT_EVENT_TIMEOUT (((amazon_se_mei_device_private_t *)
					 pDev->pPriv)->wait_queue_arcmsgav,
					CMV_TIMEOUT);
#else
	while (((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav == 0 && delay_counter < CMV_TIMEOUT / 5) {
		MEI_WAIT (5);
		delay_counter++;
	}
#endif

	((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_waiting = 0;
	if (((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav == 0) {	//CMV_timeout
		((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav = 0;
		AMAZON_SE_MEI_EMSG ("\%s: DSL_DEV_MEI_ERR_MAILBOX_TIMEOUT\n",
				    __FUNCTION__);
		MEI_MUTEX_UNLOCK (((amazon_se_mei_device_private_t *) pDev->pPriv)->mei_cmv_sema);
		return DSL_DEV_MEI_ERR_MAILBOX_TIMEOUT;
	}
	else {
		((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav = 0;
		((amazon_se_mei_device_private_t *) pDev->pPriv)->
			reply_count++;
		memcpy (response,
			((amazon_se_mei_device_private_t *) pDev->pPriv)->
			CMV_RxMsg, MSG_LENGTH * 2);
		MEI_MUTEX_UNLOCK (((amazon_se_mei_device_private_t *) pDev->pPriv)->mei_cmv_sema);
		return DSL_DEV_MEI_ERR_SUCCESS;
	}
	MEI_MUTEX_UNLOCK (((amazon_se_mei_device_private_t *) pDev->pPriv)->mei_cmv_sema);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
 * Reset the ARC, download boot codes, and run the ARC.
 * This function resets the ARC, downloads boot codes to ARC, and runs the ARC.
 *
 * \param 	pDev		the device pointer
 * \return	DSL_DEV_MEI_ERR_SUCCESS or DSL_DEV_MEI_ERR_FAILURE
 * \ingroup	Internal
 */
static DSL_DEV_MeiError_t
AMAZON_SE_MEI_RunAdslModem (DSL_DEV_Device_t *pDev)
{
	int nSize = 0, idx = 0;
	uint32_t im0_register, im2_register;
	DSL_DEV_WinHost_Message_t m;

	if (mei_arc_swap_buff == NULL) {
		mei_arc_swap_buff =
			(u32 *) kmalloc (MAXSWAPSIZE * 4, GFP_KERNEL);
		if (mei_arc_swap_buff == NULL) {
			AMAZON_SE_MEI_EMSG (">>> malloc fail for codeswap buff!!! <<<\n");
			return DSL_DEV_MEI_ERR_FAILURE;
		}
	}

	((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr =
		(ARC_IMG_HDR *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[0].address;
	//printk("img_hdr->count=[%x] each[%x]\n", ((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->count, sizeof(ARC_SWP_PAGE_HDR) );
	if ((((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->
	     count) * sizeof (ARC_SWP_PAGE_HDR) > SDRAM_SEGMENT_SIZE) {
		AMAZON_SE_MEI_EMSG ("firmware header size is bigger than 64K segment size\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	}
	// check image size
	for (idx = 0; idx < MAX_BAR_REGISTERS; idx++) {
		nSize += ((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[idx].nCopy;
	}
	if (nSize !=
	    ((amazon_se_mei_device_private_t *) pDev->pPriv)->image_size) {
		AMAZON_SE_MEI_EMSG ("Firmware download is not completed. Please download firmware again!\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	}
	// TODO: check crc
	///

	AMAZON_SE_MEI_ResetARC (pDev);
	AMAZON_SE_MEI_HaltArc (pDev);
	//printk(KERN_INFO "[%s %d] -> AMAZON_SE_MEI_BarUpdate()\n", __func__, __LINE__);
	AMAZON_SE_MEI_BarUpdate (pDev,
				 ((amazon_se_mei_device_private_t *) pDev->pPriv)->nBar);

	//AMAZON_SE_MEI_DMSG("Starting to meiDownloadBootCode\n");

	AMAZON_SE_MEI_DownloadBootCode (pDev);

	im0_register = (*AMAZON_SE_ICU_IM0_IER) & (1 << 20);
	im2_register = (*AMAZON_SE_ICU_IM2_IER) & (1 << 20);
	/* Turn off irq */
	disable_irq (AMAZON_SE_USB_OC_INT0);
	disable_irq (AMAZON_SE_USB_OC_INT2);
	disable_irq (pDev->nIrq[AMAZON_SE_DYING_GASP]);

	AMAZON_SE_MEI_RunArc (pDev);
#if 0
	if (AMAZON_SE_MPS_CHIPID_VERSION_GET(chipid) > 1 ) {
////////////////makeCMV(Opcode, Group, Address, Index, Size, Data), CMV in u16 TxMessage[MSG_LENGTH]///////////////////////////
                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_INFO, 103, 5, 1, 1, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS) {
			printk(KERN_ERR "[%s %d]: cmvwrite INFO 103 5 1 failed!\n", __func__, __LINE__);
			return -EIO;
		}
	}
#endif
	MEI_WAIT_EVENT_TIMEOUT (((amazon_se_mei_device_private_t *) pDev->pPriv)->wait_queue_modemready, 1000);

	MEI_MASK_AND_ACK_IRQ (AMAZON_SE_USB_OC_INT0);
	MEI_MASK_AND_ACK_IRQ (AMAZON_SE_USB_OC_INT2);
	MEI_MASK_AND_ACK_IRQ (pDev->nIrq[AMAZON_SE_DYING_GASP]);

	/* Re-enable irq */
	enable_irq(pDev->nIrq[AMAZON_SE_DYING_GASP]);
	*AMAZON_SE_ICU_IM0_IER |= im0_register;
	*AMAZON_SE_ICU_IM2_IER |= im2_register;

	if (((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready != 1) {
		AMAZON_SE_MEI_EMSG ("Modem failed to be ready!\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	} else {
		//AMAZON_SE_MEI_DMSG("Modem is ready.\n");
		return DSL_DEV_MEI_ERR_SUCCESS;
	}
}

/**
 * Get the page's data pointer
 * This function caculats the data address from the firmware header.
 *
 * \param 	pDev		the device pointer
 * \param	Page		The page number.
 * \param	data		Data page or program page.
 * \param	MaxSize		The maximum size to read.
 * \param	Buffer		Pointer to data.
 * \param	Dest		Pointer to the destination address.
 * \return	The number of bytes to read.
 * \ingroup	Internal
 */
static int
AMAZON_SE_MEI_GetPage (DSL_DEV_Device_t * pDev, u32 Page, u32 data,
		       u32 MaxSize, u32 * Buffer, u32 * Dest)
{
	u32 size;
	u32 i;
	u32 *p;
	u32 idx, offset, nBar = 0;

	if (Page >
	    ((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->count)
		return -2;
	/*
	 **     Get program or data size, depending on "data" flag
	 */
	size = (data ==
		GET_DATA) ? (((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->page[Page].d_size) : 
			     (((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->page[Page].p_size);
	size &= BOOT_FLAG_MASK;	//      Clear boot bit!
	if (size > MaxSize)
		return -1;

	if (size == 0)
		return 0;
	/*
	 **     Get program or data offset, depending on "data" flag
	 */
	i = data ? (((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->page[Page].d_offset) :
			(((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->page[Page].p_offset);

	/*
	 **     Copy data/program to buffer
	 */

	idx = i / SDRAM_SEGMENT_SIZE;
	offset = i % SDRAM_SEGMENT_SIZE;
	p = (u32 *) ((u8 *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[idx].address + offset);

	for (i = 0; i < size; i++) {
		if (offset + i * 4 - (nBar * SDRAM_SEGMENT_SIZE) >= SDRAM_SEGMENT_SIZE) {
			idx++;
			nBar++;
			p = (u32 *) ((u8 *) KSEG1ADDR ((u32)
						((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[idx].address));
		}
		Buffer[i] = *p++;
	}

	/*
	 **     Pass back data/program destination address
	 */
	*Dest = data ? (((amazon_se_mei_device_private_t *) pDev->pPriv)-> img_hdr->page[Page].d_dest) : 
				(((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr->page[Page].p_dest);

	return size;
}

/**
 * Free the memory for ARC firmware
 *
 * \param 	pDev		the device pointer
 * \param	type	Free all memory or free the unused memory after showtime
 * \ingroup	Internal
 */
const char *free_str[4] = {"NONE", "Free_Reload", "Free_Showtime", "Free_All"};
static int
AMAZON_SE_MEI_DFEMemoryFree (DSL_DEV_Device_t * pDev, int type)
{
	int idx = 0;
	smmu_mem_info_t *adsl_mem_info =
		((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info;

	for (idx = 0; idx < MAX_BAR_REGISTERS; idx++) {
		if (type == FREE_ALL ||adsl_mem_info[idx].type == type) {
			if (adsl_mem_info[idx].size > 0) {
				AMAZON_SE_MEI_DMSG ("Freeing memory %p (%s)\n", adsl_mem_info[idx].org_address, free_str[adsl_mem_info[idx].type]);
				kfree (adsl_mem_info[idx].org_address);
				adsl_mem_info[idx].org_address = 0;
				adsl_mem_info[idx].address = 0;
				adsl_mem_info[idx].size = 0;
				adsl_mem_info[idx].type = 0;
				adsl_mem_info[idx].nCopy = 0;
			}
		}
	}
	return 0;
}

static int
AMAZON_SE_MEI_DFEMemoryAlreadyAllocated (DSL_DEV_Device_t * pDev, long size, int *retval)
{
	int idx = 0;
	long total_size = 0;
	smmu_mem_info_t *adsl_mem_info =
		((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info;
	int allocate_size = SDRAM_SEGMENT_SIZE;
	int mem_allocated_flag = 1;

	
	//printk(KERN_INFO "[%s %d]: image_size = %d\n", __func__, __LINE__, size);
	for (idx = 0; size > 0 && idx < MAX_BAR_REGISTERS && (mem_allocated_flag == 1); idx++) {
		// skip bar15 for XDATA usage.
#ifndef DFE_LOOPBACK
		if (idx == XDATA_REGISTER)
			continue;
#endif
		if (size < SDRAM_SEGMENT_SIZE)
			allocate_size = size;
		else
			allocate_size = SDRAM_SEGMENT_SIZE;
		mem_allocated_flag = (adsl_mem_info[idx].org_address == 0) ? 0 : 1;
		//AMAZON_SE_MEI_EMSG ("mem_allocated_flag for %d register :%d !\n", idx, mem_allocated_flag);
		size -= allocate_size;
		total_size += allocate_size;
	}
	//AMAZON_SE_MEI_EMSG ("[%s] return value :%d !\n", __func__, mem_allocated_flag);
	*retval = idx;
	return mem_allocated_flag;

}
static int
AMAZON_SE_MEI_DFEMemoryAlloc (DSL_DEV_Device_t * pDev, long size)
{
	unsigned long mem_ptr;
	char *org_mem_ptr = NULL;
	int idx = 0;
	long total_size = 0;
	int err = 0;
	smmu_mem_info_t *adsl_mem_info =
		((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info;
	int allocate_size = SDRAM_SEGMENT_SIZE;

	//printk(KERN_INFO "[%s %d]: image_size = %d\n", __func__, __LINE__, size);
	// Alloc Swap Pages
	for (idx = 0; size > 0 && idx < MAX_BAR_REGISTERS; idx++) {
		// skip bar15 for XDATA usage.
#ifndef DFE_LOOPBACK
		if (idx == XDATA_REGISTER)
			continue;
#endif
#if 0
                if (size < SDRAM_SEGMENT_SIZE) {
                        allocate_size = size;
                        if (allocate_size < 1024)
                                allocate_size = 1024;
                }
#endif
#if 0 //BingTao
		if (idx == (MAX_BAR_REGISTERS - 1))
#endif
		if (size < SDRAM_SEGMENT_SIZE)
			allocate_size = size;
		else
			allocate_size = SDRAM_SEGMENT_SIZE;
		org_mem_ptr = kmalloc (allocate_size, GFP_KERNEL);
		if (org_mem_ptr == NULL) {
			//XXX: BingTao
			//AMAZON_SE_MEI_EMSG ("%s: kmalloc %d bytes memory fail!\n", in_interrupt()?"INTERRUPT":"NOT INTERRUPT",size);
			AMAZON_SE_MEI_EMSG ("%s: kmalloc %d bytes memory fail!\n", in_interrupt()?"INTERRUPT":"NOT INTERRUPT",allocate_size);
			err = -ENOMEM;
			machine_restart(""); //XXX:
			goto allocate_error;
		}
		mem_ptr = (unsigned long) (org_mem_ptr + 1023) & ~(1024 -1);
		adsl_mem_info[idx].address = (char *) mem_ptr;
		adsl_mem_info[idx].org_address = org_mem_ptr;
		adsl_mem_info[idx].size = allocate_size;
		size -= allocate_size;
		total_size += allocate_size;
		//printk("alloc memory idx=%d,size=%ld,addr=%X\n",idx,size,(int)adsl_mem_info[idx].address);
	}
	if (size > 0) {
		AMAZON_SE_MEI_EMSG ("Image size is too large!\n");
		err = -EFBIG;
		goto allocate_error;
	}
	err = idx;
	return err;

      allocate_error:
	AMAZON_SE_MEI_DFEMemoryFree (pDev, FREE_ALL);
	return err;
}

/**
 * Program the BAR registers
 *
 * \param 	pDev		the device pointer
 * \param	nTotalBar	The number of bar to program.
 * \ingroup	Internal
 */
static int
AMAZON_SE_MEI_BarUpdate (DSL_DEV_Device_t * pDev, int nTotalBar)
{
	int idx = 0, temp;
	smmu_mem_info_t *adsl_mem_info =
		((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info;

	for (idx = 0; idx < nTotalBar; idx++) {
		//skip XDATA register
		if (idx == XDATA_REGISTER)
			continue;
		AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, 
								(((uint32_t) adsl_mem_info[idx].address) & 0x0FFFFFFF));
		AMAZON_SE_MEI_LongWordReadOffset (pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, &temp);
		//AMAZON_SE_MEI_DMSG("BAR%d: write %08X read %08X\n", idx, (((uint32_t)adsl_mem_info[idx].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
		#if 0
		if ((temp & 0x0FFFFFFF) != (((uint32_t)adsl_mem_info[idx].address) & 0x0FFFFFFF))
			panic ("[%s %d]: BAR%d: write %08X read %08X\n", __func__, __LINE__, idx, (((uint32_t)adsl_mem_info[idx].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
		#endif
	}
	for (idx = nTotalBar; idx < MAX_BAR_REGISTERS; idx++) {
#if 0 //XXX: BingTao
		if (idx == XDATA_REGISTER)
			continue;
#endif
		AMAZON_SE_MEI_LongWordWriteOffset (pDev,
						   (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, 
						   (((uint32_t)adsl_mem_info[nTotalBar - 1].address) & 0x0FFFFFFF));
		AMAZON_SE_MEI_LongWordReadOffset (pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, &temp);
		//AMAZON_SE_MEI_DMSG("BAR%d: write %08X read %08X\n", idx, (((uint32_t)adsl_mem_info[nTotalBar - 1].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
		#if 0
		if ((temp & 0x0FFFFFFF) != (((uint32_t)adsl_mem_info[nTotalBar - 1].address) & 0x0FFFFFFF))
			panic ("[%s %d]: BAR%d: write %08X read %08X\n", __func__, __LINE__, idx, (((uint32_t)adsl_mem_info[nTotalBar - 1].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
		#endif
		/* These are for /proc/amazon_se_mei/meminfo purpose */
		adsl_mem_info[idx].address = adsl_mem_info[nTotalBar - 1].address;	
		adsl_mem_info[idx].org_address = adsl_mem_info[nTotalBar - 1].org_address;	
		adsl_mem_info[idx].size = 0; /* Prevent it from being freed */
	}

	AMAZON_SE_MEI_LongWordWriteOffset (pDev,
					   (u32) MEI_XMEM_BAR_BASE_OFFSET + XDATA_REGISTER * 4, 
					   (((uint32_t) adsl_mem_info [XDATA_REGISTER].address) & 0x0FFFFFFF));
	AMAZON_SE_MEI_LongWordReadOffset (pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + XDATA_REGISTER * 4, &temp);
	//AMAZON_SE_MEI_DMSG("BAR%d: write %08X read %08X\n", XDATA_REGISTER, (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
	#if 0
	if ((temp & 0x0FFFFFFF) != (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF))
		panic ("[%s %d]: BAR%d: write %08X read %08X\n", __func__, __LINE__, XDATA_REGISTER, (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
	#endif
	// update MEI_XDATA_BASE_SH
	AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_XDATA_BASE_SH_OFFSET,
					   ((unsigned long)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF);
	AMAZON_SE_MEI_LongWordReadOffset (pDev, (u32) MEI_XDATA_BASE_SH_OFFSET, &temp);
	//AMAZON_SE_MEI_DMSG("MEI_XDATA_BASE_SH_OFFSET: write %08X read %08X\n", (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
	#if 0
	if ((temp & 0x0FFFFFFF) != (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF))
		panic ("[%s %d]: MEI_XDATA_BASE_SH_OFFSET:: write %08X read %08X\n", __func__, __LINE__, (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
	#endif
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/* This copies the firmware from secondary storage to 64k memory segment in SDRAM */
DSL_DEV_MeiError_t
DSL_BSP_FWDownload (DSL_DEV_Device_t * pDev, const char *buf,
			 unsigned long size, long *loff, long *current_offset)
{
	ARC_IMG_HDR img_hdr_tmp;
	smmu_mem_info_t *adsl_mem_info =
		((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info;

	size_t nRead = 0, nCopy = 0, nCopy_big = 0;
	char *mem_ptr;
	ssize_t retval = -ENOMEM;
	int idx = 0;

#if defined(CONFIG_ATM_AMAZON_SE) || defined(CONFIG_ATM_AMAZON_SE_MODULE)
	/*Sumedh: crash handle: set global variable, use from ppe driver */
	g_showtime=0;
	//printk("g_showtime set to 0\n");
#endif

	if (*loff == 0) {
		if (size < sizeof (img_hdr_tmp)) {
			AMAZON_SE_MEI_EMSG ("Firmware size is too small!\n");
			return retval;
		}
		copy_from_user ((char *) &img_hdr_tmp, buf, sizeof (img_hdr_tmp));
		// header of image_size and crc are not included.
		((amazon_se_mei_device_private_t *) pDev->pPriv)->image_size = le32_to_cpu (img_hdr_tmp.size) + 8;

		if (((amazon_se_mei_device_private_t *) pDev->pPriv)->image_size > 1024 * 1024) {
			AMAZON_SE_MEI_EMSG ("Firmware size is too large!\n");
			return retval;
		}
		// check if arc is halt
		AMAZON_SE_MEI_ResetARC (pDev);
		AMAZON_SE_MEI_HaltArc (pDev);

		//Kishore: Retain the memory allocated for next download, if
		//memory is already allocated.
		if (!AMAZON_SE_MEI_DFEMemoryAlreadyAllocated(pDev,  ((amazon_se_mei_device_private_t *) pDev->pPriv)->image_size, &retval)) {
			AMAZON_SE_MEI_DFEMemoryFree (pDev, FREE_ALL);	//free all

			retval = AMAZON_SE_MEI_DFEMemoryAlloc (pDev,
						       ((amazon_se_mei_device_private_t *) pDev->pPriv)->image_size);
		}
		else {
			for (idx = 0; idx < retval; idx++) 
				adsl_mem_info[idx].nCopy = 0;
		}

		if (retval < 0) {
			AMAZON_SE_MEI_EMSG ("Error: No memory space left.\n");
			machine_restart(""); //XXX:
			goto error;
		}

		for (idx = 0; idx < retval; idx++) {
			//skip XDATA register
			if (idx == XDATA_REGISTER)
				continue;
			if (idx * SDRAM_SEGMENT_SIZE < le32_to_cpu (img_hdr_tmp.page[0].p_offset))
				adsl_mem_info[idx].type = FREE_RELOAD;
			else
				adsl_mem_info[idx].type = FREE_SHOWTIME;
		}
		((amazon_se_mei_device_private_t *) pDev->pPriv)->nBar = retval;

		((amazon_se_mei_device_private_t *) pDev->pPriv)->img_hdr =
			(ARC_IMG_HDR *) adsl_mem_info[0].address;

#if 0 //BingTao XXX:
		adsl_mem_info[XDATA_REGISTER].org_address = kmalloc (SDRAM_SEGMENT_SIZE, GFP_KERNEL);
		adsl_mem_info[XDATA_REGISTER].address =
			(char *) ((unsigned long) (adsl_mem_info[XDATA_REGISTER].org_address + 1023) & 0xFFFFFC00);

		adsl_mem_info[XDATA_REGISTER].size = SDRAM_SEGMENT_SIZE;

		if (adsl_mem_info[XDATA_REGISTER].address == NULL) {
			AMAZON_SE_MEI_EMSG ("kmalloc memory fail! RESET the device..\n");
			/*XXX: */
			machine_restart("");
			retval = -ENOMEM;
			goto error;
		}
		adsl_mem_info[XDATA_REGISTER].type = FREE_RELOAD;
#endif
		//printk(KERN_INFO "[%s %d] -> AMAZON_SE_MEI_BarUpdate()\n", __func__, __LINE__);
		AMAZON_SE_MEI_BarUpdate (pDev, ((amazon_se_mei_device_private_t *)pDev->pPriv)->nBar);
	}
	else if (((amazon_se_mei_device_private_t *) pDev->pPriv)-> image_size == 0) {
		AMAZON_SE_MEI_EMSG ("Error: Firmware size=0! \n");
		goto error;
	}

	nRead = 0;
#if 1
	while (nRead < size) {
		long offset = ((long) (*loff) + nRead) % SDRAM_SEGMENT_SIZE;
		idx = (((long) (*loff)) + nRead) / SDRAM_SEGMENT_SIZE;
		mem_ptr = (char *) KSEG1ADDR ((unsigned long) (adsl_mem_info[idx].address) + offset);
		if ((size - nRead + offset) > SDRAM_SEGMENT_SIZE)
			nCopy = SDRAM_SEGMENT_SIZE - offset;
		else
			nCopy = size - nRead;
		copy_from_user (mem_ptr, buf + nRead, nCopy);
		for (offset = 0; offset < (nCopy / 4); offset++) {
			((unsigned long *) mem_ptr)[offset] = le32_to_cpu (((unsigned long *) mem_ptr)[offset]);
		}
		nRead += nCopy;
		adsl_mem_info[idx].nCopy += nCopy;
	}
#else
	printk(KERN_INFO "[%s %d] -> About to start copy_from_user\n", __func__, __LINE__);
	#define CHUNK_SIZE (16*1024)
	while (nRead < size) {
		int i=0;
		long offset = ((long) (*loff) + nRead) % SDRAM_SEGMENT_SIZE;
		idx = (((long) (*loff)) + nRead) / SDRAM_SEGMENT_SIZE;
		mem_ptr = (char *) KSEG1ADDR ((unsigned long) (adsl_mem_info[idx].address) + offset);
		/* Sumedh: now dont read 64K. read 16K, 4 times */
		nCopy_big = 0;
		for(i=0; i<4, nRead < size; i++) {
			if ((size - nRead + offset) > CHUNK_SIZE) {
				nCopy = CHUNK_SIZE - offset;
				printk(KERN_INFO "!size[%x] nRead[%x] offset[%x] chunk[%x] nCopy[%x]\n", size, nRead, offset,CHUNK_SIZE, nCopy);
			}
			else {
				nCopy = size - nRead;
				printk(KERN_INFO "!!size[%x] nRead[%x] offset[%x] chunk[%x] nCopy[%x]\n", size, nRead, offset, CHUNK_SIZE, nCopy);
			}
			printk(KERN_INFO "memptr[%p] buf+nRead[%x] nCopy[%x]\n", mem_ptr, buf+nRead, nCopy);
			copy_from_user (mem_ptr, buf + nRead, nCopy);
			nRead += nCopy;
			nCopy_big += nCopy;
		}
		printk(KERN_INFO "nCopy_big[%x]\n", nCopy_big);
		for (offset = 0; offset < (nCopy_big / 4); offset++) {
			((unsigned long *) mem_ptr)[offset] = le32_to_cpu (((unsigned long *) mem_ptr)[offset]);
			//printk(KERN_INFO "[%x] ", offset);
		}
		printk(KERN_INFO "\n");
		adsl_mem_info[idx].nCopy += nCopy_big;
	}
#endif
	*loff += size;
	*current_offset = size;
	return DSL_DEV_MEI_ERR_SUCCESS;
error:
	AMAZON_SE_MEI_DFEMemoryFree (pDev, FREE_ALL);
	return DSL_DEV_MEI_ERR_FAILURE;
}
/*
 * Register a callback event.
 * Return:
 * -1 if the event already has a callback function registered.
 *  0 success
 */
int DSL_BSP_EventCBRegister(DSL_BSP_EventCallBack_t *p)
{
	if (!p) {
                AMAZON_SE_MEI_EMSG("Invalid parameter!\n");
                return -EINVAL;
	}
        if (p->event > DSL_BSP_CB_LAST || p->event < DSL_BSP_CB_FIRST) {
                AMAZON_SE_MEI_EMSG("Invalid Event %d\n", p->event);
                return -EINVAL;
        }
        if (dsl_bsp_event_callback[p->event].function) {
                AMAZON_SE_MEI_EMSG("Event %d already has a callback function registered!\n", p->event);
                return -1;
        } else {
                dsl_bsp_event_callback[p->event].function = p->function;
                dsl_bsp_event_callback[p->event].event    = p->event;
                dsl_bsp_event_callback[p->event].pData    = p->pData;
        }
        return 0;
}
int DSL_BSP_EventCBUnregister(DSL_BSP_EventCallBack_t *p)
{
	if (!p) {
                AMAZON_SE_MEI_EMSG("Invalid parameter!\n");
                return -EINVAL;
	}
        if (p->event > DSL_BSP_CB_LAST || p->event < DSL_BSP_CB_FIRST) {
                AMAZON_SE_MEI_EMSG("Invalid Event %d\n", p->event);
                return -EINVAL;
        }
        if (dsl_bsp_event_callback[p->event].function) {
                AMAZON_SE_MEI_EMSG("Unregistering Event %d...\n", p->event);
                dsl_bsp_event_callback[p->event].function = NULL;
                dsl_bsp_event_callback[p->event].pData    = NULL;
        } else {
                AMAZON_SE_MEI_EMSG("Event %d is not registered!\n", p->event);
                return -1;
        }
        return 0;
}

/**
 * MEI Dying Gasp interrupt handler
 *
 * \param int1
 * \param void0
 * \param regs	Pointer to the structure of amazon_se mips registers
 * \ingroup	Internal
 */
static void
AMAZON_SE_MEI_Dying_Gasp_IrqHandle (int int1, void *void0, struct pt_regs *regs)
{
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) void0;
        DSL_BSP_CB_Type_t event;

	if (pDev == NULL)
		AMAZON_SE_MEI_EMSG("Error: Got Interrupt but pDev is NULL!!!!\n");

	disable_irq (pDev->nIrq[AMAZON_SE_DYING_GASP]);
	event = DSL_BSP_CB_DYING_GASP;

	if (dsl_bsp_event_callback[event].function)
		(*dsl_bsp_event_callback[event].function)(pDev, event, dsl_bsp_event_callback[event].pData);

	AMAZON_SE_MEI_EMSG("Dying Gasp! Shutting Down...\n");
	kill_proc (1, SIGINT, 1);       /* Ask init to reboot us */
}
/**
 * MEI interrupt handler
 *
 * \param int1
 * \param void0
 * \param regs	Pointer to the structure of amazon_se mips registers
 * \ingroup	Internal
 */
#if defined(AMAZON_SE_PORT_RTEMS)
static void
AMAZON_SE_MEI_IrqHandle (unsigned long int1)
#else
static void
AMAZON_SE_MEI_IrqHandle (int int1, void *void0, struct pt_regs *regs)
#endif				//defined(AMAZON_SE_PORT_RTEMS)
{
	u32 scratch, i;
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) void0;
#if defined(DFE_LOOPBACK) && defined(DFE_PING_TEST)
	dfe_loopback_irq_handler ();
	return;
#endif //DFE_LOOPBACK
        DSL_BSP_CB_Type_t event;

	if (pDev == NULL)
		AMAZON_SE_MEI_EMSG("Error: Got Interrupt but pDev is NULL!!!!\n");

	AMAZON_SE_MEI_DebugRead (pDev, ARC_MEI_MAILBOXR, &scratch, 1);
	if (scratch & OMB_CODESWAP_MESSAGE_MSG_TYPE_MASK) {
		AMAZON_SE_MEI_EMSG("Receive Code Swap Request interrupt!!!\n");
		return;
	}
	else if (scratch & OMB_CLEAREOC_INTERRUPT_CODE)	// clear eoc message interrupt
	{
		AMAZON_SE_MEI_DMSG("OMB_CLEAREOC_INTERRUPT_CODE\n");
                event = DSL_BSP_CB_CEOC_IRQ;
		AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_MSGAV);
                if (dsl_bsp_event_callback[event].function)
			(*dsl_bsp_event_callback[event].function)(pDev, event, dsl_bsp_event_callback[event].pData);
        } else if (scratch & OMB_REBOOT_INTERRUPT_CODE) // Reboot
	{
		AMAZON_SE_MEI_DMSG("OMB_CLEAREOC_INTERRUPT_CODE\n");
		//AMAZON_SE_MEI_DMSG("OMB_REBOOT_INTERRUPT_CODE\n"); // 805071:<IFTW-Jelly>
		event = DSL_BSP_CB_FIRMWARE_REBOOT;
		//AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_MSGAV); // 805071:<IFTW-Jelly>
		if (dsl_bsp_event_callback[event].function)
			(*dsl_bsp_event_callback[event].function)(pDev, event, dsl_bsp_event_callback[event].pData);
	} else {			// normal message
		AMAZON_SE_MEI_MailboxRead (pDev,
					   ((amazon_se_mei_device_private_t *)
					    pDev->pPriv)->CMV_RxMsg,
					   MSG_LENGTH);
		if (((amazon_se_mei_device_private_t *) pDev->pPriv)-> cmv_waiting == 1) {
			((amazon_se_mei_device_private_t *) pDev->pPriv)-> arcmsgav = 1;
			((amazon_se_mei_device_private_t *) pDev->pPriv)-> cmv_waiting = 0;
#if !defined(AMAZON_SE_PORT_RTEMS)
			MEI_WAKEUP_EVENT (((amazon_se_mei_device_private_t *)
					   pDev->pPriv)->wait_queue_arcmsgav);
#endif
		}
		else {
			((amazon_se_mei_device_private_t *) pDev->pPriv)-> modem_ready_cnt++;
			memcpy ((char *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->Recent_indicator,
				(char *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->CMV_RxMsg,
				MSG_LENGTH * 2);
			if (((((amazon_se_mei_device_private_t *) pDev->pPriv)->CMV_RxMsg[0] & 0xff0) >> 4) == D2H_AUTONOMOUS_MODEM_READY_MSG)	// arc ready
			{	//check ARC ready message

				//AMAZON_SE_MEI_DMSG ("Got MODEM_READY_MSG\n");
				((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready = 1;
				MEI_WAKEUP_EVENT (((amazon_se_mei_device_private_t *) pDev->pPriv)->wait_queue_modemready);
			}
		}
	}

//	MEI_MASK_AND_ACK_IRQ (pDev->nIrq[AMAZON_SE_DFEIR]);
	return;
}

int
DSL_BSP_ATMLedCBRegister (int (*ifx_adsl_ledcallback) (void))
{
#ifdef CONFIG_ATM_AMAZON_SE
	return IFX_ATM_LED_Callback_Register (ifx_adsl_ledcallback);
#endif
}

int
DSL_BSP_ATMLedCBUnregister (int (*ifx_adsl_ledcallback) (void))
{
#ifdef CONFIG_ATM_AMAZON_SE
	return IFX_ATM_LED_Callback_Unregister (ifx_adsl_ledcallback);
#endif
}

#if 0
int
DSL_BSP_EventCBRegister (int (*ifx_adsl_callback)
			        (DSL_BSP_CB_Event_t * param))
{
	int error = 0;
	if (DSL_EventCB == NULL) {
		DSL_EventCB = ifx_adsl_callback;
	}
	else {
		error = -EIO;
	}
	return error;
}

int
DSL_BSP_EventCBUnregister (int (*ifx_adsl_callback)
				  (DSL_BSP_CB_Event_t * param))
{
	int error = 0;
	if (DSL_EventCB == ifx_adsl_callback) {
		DSL_EventCB = NULL;
	}
	else {
		error = -EIO;
	}
	return error;
}

static int
DSL_BSP_GetEventCB (int (**ifx_adsl_callback)
			   (DSL_BSP_CB_Event_t * param))
{
	*ifx_adsl_callback = DSL_EventCB;
	return 0;
}
#endif
//TODO, for loopback test
#ifdef DFE_LOOPBACK
#define mte_reg_base	(0x4800*4+0x20000)

/* Iridia Registers Address Constants */
#define MTE_Reg(r)    	(int)(mte_reg_base + (r*4))

#define IT_AMODE       	MTE_Reg(0x0004)

#define TIMER_DELAY   	(1024)
#define BC0_BYTES     	(32)
#define BC1_BYTES     	(30)
#define NUM_MB        	(12)
#define TIMEOUT_VALUE 	2000

static void
BFMWait (u32 cycle)
{
	u32 i;
	for (i = 0; i < cycle; i++);
}

static void
WriteRegLong (u32 addr, u32 data)
{
	//*((volatile u32 *)(addr)) =  data;
	AMAZON_SE_WRITE_REGISTER_L (data, addr);
}

static u32
ReadRegLong (u32 addr)
{
	// u32  rd_val;
	//rd_val = *((volatile u32 *)(addr));
	// return rd_val;
	return AMAZON_SE_READ_REGISTER_L (addr);
}

/* This routine writes the mailbox with the data in an input array */
static void
WriteMbox (u32 * mboxarray, u32 size)
{
	AMAZON_SE_MEI_DebugWrite (&dsl_devices[0], IMBOX_BASE, mboxarray,
				  size);
	printk ("write to %X\n", IMBOX_BASE);
	AMAZON_SE_MEI_LongWordWriteOffset (&dsl_devices[0],
					   (u32) MEI_TO_ARC_INT_OFFSET,
					   MEI_TO_ARC_MSGAV);
}

/* This routine reads the output mailbox and places the results into an array */
static void
ReadMbox (u32 * mboxarray, u32 size)
{
	AMAZON_SE_MEI_DebugRead (&dsl_devices[0], OMBOX_BASE, mboxarray,
				 size);
	printk ("read from %X\n", OMBOX_BASE);
}

static void
MEIWriteARCValue (u32 address, u32 value)
{
	u32 i, check = 0;

	/* Write address register */
	AMAZON_SE_WRITE_REGISTER_L (address,
				    MEI_DEBUG_WAD_OFFSET + MEI_SPACE_ACCESS);

	/* Write data register */
	AMAZON_SE_WRITE_REGISTER_L (value,
				    MEI_DEBUG_DATA_OFFSET + MEI_SPACE_ACCESS);

	/* wait until complete - timeout at 40 */
	for (i = 0; i < 40; i++) {
		check = AMAZON_SE_READ_REGISTER_L (ARC_TO_MEI_INT_OFFSET +
						   MEI_SPACE_ACCESS);

		if ((check & ARC_TO_MEI_DBG_DONE))
			break;
	}
	/* clear the flag */
	AMAZON_SE_WRITE_REGISTER_L (ARC_TO_MEI_DBG_DONE,
				    ARC_TO_MEI_INT_OFFSET + MEI_SPACE_ACCESS);
}

void
arc_code_page_download (uint32_t arc_code_length, uint32_t * start_address)
{
	int count;
	printk ("try to download pages,size=%d\n", arc_code_length);
	AMAZON_SE_MEI_ControlModeSet (&dsl_devices[0], MEI_MASTER_MODE);
	AMAZON_SE_MEI_HaltArc (&dsl_devices[0]);
	AMAZON_SE_MEI_LongWordWriteOffset (&dsl_devices[0],
					   (u32) MEI_XFR_ADDR_OFFSET, 0);
	for (count = 0; count < arc_code_length; count++) {
		AMAZON_SE_MEI_LongWordWriteOffset (&dsl_devices[0],
						   (u32) MEI_DATA_XFR_OFFSET,
						   *(start_address + count));
	}
	AMAZON_SE_MEI_ControlModeSet (&dsl_devices[0], JTAG_MASTER_MODE);
}
static int
load_jump_table (unsigned long addr)
{
	int i;
	uint32_t addr_le, addr_be;
	uint32_t jump_table[32];
	for (i = 0; i < 16; i++) {
		addr_le = i * 8 + addr;
		addr_be = ((addr_le >> 16) & 0xffff);
		addr_be |= ((addr_le & 0xffff) << 16);
		jump_table[i * 2 + 0] = 0x0f802020;
		jump_table[i * 2 + 1] = addr_be;
		//printk("jt %X %08X %08X\n",i,jump_table[i*2+0],jump_table[i*2+1]);
	}
	arc_code_page_download (32, &jump_table[0]);
	return 0;
}

int got_int = 0;

void
dfe_loopback_irq_handler (void)
{
	uint32_t rd_mbox[10];

	memset (&rd_mbox[0], 0, 10 * 4);
	ReadMbox (&rd_mbox[0], 6);
	if (rd_mbox[0] == 0x0) {
		printk ("Get ARC_ACK\n");
		got_int = 1;
	}
	else if (rd_mbox[0] == 0x5) {
		printk ("Get ARC_BUSY\n");
		got_int = 2;
	}
	else if (rd_mbox[0] == 0x3) {
		printk ("Get ARC_EDONE\n");
		if (rd_mbox[1] == 0x0) {
			got_int = 3;
			printk ("Get E_MEMTEST\n");
			if (rd_mbox[2] != 0x1) {
				got_int = 4;
				printk ("Get Result %X\n", rd_mbox[2]);
			}
		}
	}
	AMAZON_SE_MEI_LongWordWriteOffset (&dsl_devices[0],
					   (u32) ARC_TO_MEI_INT_OFFSET,
					   ARC_TO_MEI_DBG_DONE);
	MEI_MASK_AND_ACK_IRQ (pDev->nIrq[AMAZON_SE_DFEIR]);
	disable_irq (pDev->nIrq[AMAZON_SE_DFEIR]);
	//got_int = 1;
	return;
}

static void
wait_mem_test_result (void)
{
	uint32_t mbox[5];
	mbox[0] = 0;
	printk ("Waiting Starting\n");
	while (mbox[0] == 0) {
		ReadMbox (&mbox[0], 5);
	}
	printk ("Try to get mem test result.\n");
	ReadMbox (&mbox[0], 5);
	if (mbox[0] == 0xA) {
		printk ("Success.\n");
	}
	else if (mbox[0] == 0xA) {
		printk ("Fail,address %X,except data %X,receive data %X\n",
			mbox[1], mbox[2], mbox[3]);
	}
	else {
		printk ("Fail\n");
	}
}

static int
arc_ping_testing (void)
{
#define MEI_PING 0x00000001
	uint32_t wr_mbox[10], rd_mbox[10];
	int i;
	for (i = 0; i < 10; i++) {
		wr_mbox[i] = 0;
		rd_mbox[i] = 0;
	}

	printk ("send ping msg\n");
	wr_mbox[0] = MEI_PING;
	WriteMbox (&wr_mbox[0], 10);

	while (got_int == 0) {
		MEI_WAIT (100);
	}

	printk ("send start event\n");
	got_int = 0;

	wr_mbox[0] = 0x4;
	wr_mbox[1] = 0;
	wr_mbox[2] = 0;
	wr_mbox[3] = (uint32_t) 0xf5acc307e;
	wr_mbox[4] = 5;
	wr_mbox[5] = 2;
	wr_mbox[6] = 0x1c000;
	wr_mbox[7] = 64;
	wr_mbox[8] = 0;
	wr_mbox[9] = 0;
	WriteMbox (&wr_mbox[0], 10);
	DSL_ENABLE_IRQ (pDev->nIrq[AMAZON_SE_DFEIR]);
	//printk("AMAZON_SE_MEI_MailboxWrite ret=%d\n",i);
	AMAZON_SE_MEI_LongWordWriteOffset (&dsl_devices[0],
					   (u32) MEI_TO_ARC_INT_OFFSET,
					   MEI_TO_ARC_MSGAV);
	printk ("sleeping\n");
	while (1) {
		if (got_int > 0) {

			if (got_int > 3)
				printk ("got_int >>>> 3\n");
			else
				printk ("got int = %d\n", got_int);
			got_int = 0;
			//schedule();
			DSL_ENABLE_IRQ (pDev->nIrq[AMAZON_SE_DFEIR]);
		}
		//mbox_read(&rd_mbox[0],6);
		MEI_WAIT (100);
	}
}

static DSL_DEV_MeiError_t
DFE_Loopback_Test (void)
{
	int i = 0;
	u32 arc_debug_data = 0, temp;
	DSL_DEV_Device_t *pDev = &dsl_devices[0];
	uint32_t wr_mbox[10];

	AMAZON_SE_MEI_ResetARC (pDev);
	// start the clock
	arc_debug_data = ACL_CLK_MODE_ENABLE;
	AMAZON_SE_MEI_DebugWrite (pDev, CRI_CCR0, &arc_debug_data, 1);

#if defined( DFE_PING_TEST )|| defined( DFE_ATM_LOOPBACK)
	// WriteARCreg(AUX_XMEM_LTEST,0);
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);
#define AUX_XMEM_LTEST 0x128
	_AMAZON_SE_MEI_DBGLongWordWrite (pDev, MEI_DEBUG_DEC_AUX_MASK,
					 AUX_XMEM_LTEST, 0);
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);

	// WriteARCreg(AUX_XDMA_GAP,0);
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);
#define AUX_XDMA_GAP 0x114
	_AMAZON_SE_MEI_DBGLongWordWrite (pDev, MEI_DEBUG_DEC_AUX_MASK,
					 AUX_XDMA_GAP, 0);
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);

	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);
	temp = 0;
	_AMAZON_SE_MEI_DBGLongWordWrite (pDev, MEI_DEBUG_DEC_AUX_MASK,
					 (u32) MEI_XDATA_BASE_SH_OFFSET +
					 MEI_SPACE_ACCESS, temp);
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);

	i = AMAZON_SE_MEI_DFEMemoryAlloc (pDev, SDRAM_SEGMENT_SIZE * 16);
	if (i >= 0) {
		int idx;

		for (idx = 0; idx < i; idx++) {
			((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[idx].type = FREE_RELOAD;
			AMAZON_SE_WRITE_REGISTER_L ((((uint32_t) ((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[idx].address) & 0x0fffffff), 
							MEI_SPACE_ACCESS + MEI_XMEM_BAR_BASE_OFFSET + idx * 4);
			printk ("bar%d(%X)=%X\n", idx,
				MEI_SPACE_ACCESS + MEI_XMEM_BAR_BASE_OFFSET +
				idx * 4, (((uint32_t)
					   ((amazon_se_mei_device_private_t *)
					    pDev->pPriv)->adsl_mem_info[idx].
					   address) & 0x0fffffff));
			memset ((u8 *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[idx].address, 0, SDRAM_SEGMENT_SIZE);
		}

		AMAZON_SE_MEI_LongWordWriteOffset (pDev, (u32) MEI_XDATA_BASE_SH_OFFSET,
					   ((unsigned long) ((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF);
	}
	else {
		AMAZON_SE_MEI_EMSG ("cannot load image: no memory\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	}
	//WriteARCreg(AUX_IC_CTRL,2);
	AMAZON_SE_MEI_ControlModeSet (pDev, MEI_MASTER_MODE);
#define AUX_IC_CTRL 0x11
	_AMAZON_SE_MEI_DBGLongWordWrite (pDev, MEI_DEBUG_DEC_AUX_MASK,
					 AUX_IC_CTRL, 2);
	AMAZON_SE_MEI_ControlModeSet (pDev, JTAG_MASTER_MODE);

	AMAZON_SE_MEI_HaltArc (&dsl_devices[0]);

#ifdef DFE_PING_TEST

	printk ("ping test image size=%d\n", sizeof (arc_ahb_access_code));
	memcpy ((u8 *) (((amazon_se_mei_device_private_t *) pDev->pPriv)->
			adsl_mem_info[0].address + 0x1004),
		&arc_ahb_access_code[0], sizeof (arc_ahb_access_code));
	load_jump_table (0x80000 + 0x1004);

#endif //DFE_PING_TEST

	printk ("ARC ping test code download complete\n");
#endif //defined( DFE_PING_TEST )|| defined( DFE_ATM_LOOPBACK)
#ifdef DFE_MEM_TEST
	AMAZON_SE_MEI_LongWordWriteOffset (&dsl_devices[0],
					   (u32) ARC_TO_MEI_INT_MASK_OFFSET,
					   MSGAV_EN);

	arc_code_page_download (1537, &code_array[0]);
	printk ("ARC mem test code download complete\n");
#endif //DFE_MEM_TEST
#ifdef DFE_ATM_LOOPBACK
	arc_debug_data = 0xf;
	arc_code_page_download (1077, &code_array[0]);
	wr_mbox[0] = 1024;	//TIMER_DELAY
	wr_mbox[1] = 0;		//TXFB_START0
	wr_mbox[2] = 49;	//TXFB_END0
	wr_mbox[3] = 50;	//TXFB_START1
	wr_mbox[4] = 109;	//TXFB_END1
	wr_mbox[5] = 0;		//RXFB_START0
	wr_mbox[6] = 49;	//RXFB_END0
	wr_mbox[7] = 256;	//RXFB_START1
	wr_mbox[8] = 315;	//RXFB_END1
	WriteMbox (&wr_mbox[0], 9);
	// Start Iridia IT_AMODE (in dmp access) why is it required?
	AMAZON_SE_MEI_DebugWrite (&dsl_devices[0], 0x32010, &arc_debug_data,
				  1);
#endif //DFE_ATM_LOOPBACK
	AMAZON_SE_MEI_IRQEnable (pDev);
	AMAZON_SE_MEI_RunArc (&dsl_devices[0]);

#ifdef DFE_PING_TEST
	arc_ping_testing ();
#endif //DFE_PING_TEST
#ifdef DFE_MEM_TEST
	wait_mem_test_result ();
#endif //DFE_MEM_TEST

	AMAZON_SE_MEI_DFEMemoryFree (pDev, FREE_ALL);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

#endif //DFE_LOOPBACK
//end of TODO, for loopback test

static int
AMAZON_SE_MEI_InitDevNode (int num)
{
#ifdef CONFIG_DEVFS_FS
	char buf[36];
	sprintf (buf, "%s%d", AMAZON_SE_MEI_DEVNAME, num);
	if ((amazon_se_mei_devfs_handle[num] =
	     devfs_register (NULL, buf, DEVFS_FL_DEFAULT, dev_major, 0,
			     S_IFCHR | S_IRUGO | S_IWUGO,
			     &amazon_se_mei_operations,
			     (void *) 0)) == NULL) {
		AMAZON_SE_MEI_EMSG ("Register mei devfs error.\n");
		return -ENODEV;
	}

#else
	if (num == 0) {
		if ((dev_major = register_chrdev (dev_major, AMAZON_SE_MEI_DEVNAME, &amazon_se_mei_operations)) < 0) {
			AMAZON_SE_MEI_EMSG ("register_chrdev(%d %s) failed!\n", dev_major, AMAZON_SE_MEI_DEVNAME);
			return -ENODEV;
		}
	}
#endif
	return 0;
}

static int
AMAZON_SE_MEI_CleanUpDevNode (int num)
{
#ifdef CONFIG_DEVFS_FS
	devfs_unregister (amazon_semei_devfs_handle[num]);
#else
	if (num == 0)
		unregister_chrdev (dev_major, "amazon_se_mei");
#endif
	return 0;
}

static int
AMAZON_SE_MEI_InitDevice (int num)
{
	DSL_DEV_Device_t *pDev;
	pDev = &dsl_devices[num];
	if (pDev == NULL)
		return -ENOMEM;
	pDev->pPriv = &sAmazon_SE_Mei_Private[num];
	memset (pDev->pPriv, 0, sizeof (amazon_se_mei_device_private_t));

	memset (&((amazon_se_mei_device_private_t *) pDev->pPriv)->
		adsl_mem_info[0], 0,
		sizeof (smmu_mem_info_t) * MAX_BAR_REGISTERS);

	if (num == 0) {
		u32 pmu_reg = 0;

		pDev->nIrq[AMAZON_SE_DFEIR]      = AMAZON_SE_MEI_INT;
		pDev->nIrq[AMAZON_SE_DYING_GASP] = AMAZON_SE_MEI_DYING_GASP_INT;
		pDev->base_address = MEI_SPACE_ACCESS;

		// power up mei
		pmu_reg = AMAZON_SE_READ_REGISTER_L (AMAZON_SE_PMU_PWDCR);
		pmu_reg &= 0xffff7dfe; 
		AMAZON_SE_WRITE_REGISTER_L (pmu_reg, AMAZON_SE_PMU_PWDCR);
	}
	pDev->nInUse = 0;
	((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready = 0;
	((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav = 0;

	MEI_INIT_WAKELIST ("arcq", ((amazon_se_mei_device_private_t *) pDev->pPriv)->wait_queue_arcmsgav);	// for ARCMSGAV
	MEI_INIT_WAKELIST ("arcr", ((amazon_se_mei_device_private_t *) pDev->pPriv)->wait_queue_modemready);	// for arc modem ready

	MEI_MUTEX_INIT (((amazon_se_mei_device_private_t *) pDev->pPriv)->mei_cmv_sema, 1);	// semaphore initialization, mutex
#if 0
	MEI_MASK_AND_ACK_IRQ (pDev->nIrq[AMAZON_SE_DFEIR]);
	MEI_MASK_AND_ACK_IRQ (pDev->nIrq[AMAZON_SE_DYING_GASP]);
#endif
	if (request_irq (pDev->nIrq[AMAZON_SE_DFEIR], AMAZON_SE_MEI_IrqHandle, 0, "DFEIR", pDev) != 0) {
		AMAZON_SE_MEI_EMSG ("request_irq %d failed!\n", pDev->nIrq[AMAZON_SE_DFEIR]);
		return -1;
	}
	if (request_irq (pDev->nIrq[AMAZON_SE_DYING_GASP], AMAZON_SE_MEI_Dying_Gasp_IrqHandle, 0, "DYING_GASP", pDev) != 0) {
		AMAZON_SE_MEI_EMSG ("request_irq %d failed!\n", pDev->nIrq[AMAZON_SE_DYING_GASP]);
		return -1;
	}
//	AMAZON_SE_MEI_DMSG("Device %d initialized. IER %#x\n", num, amazon_se_get_irq_ier(pDev->nIrq[AMAZON_SE_DYING_GASP]));
	return 0;
}

static int
AMAZON_SE_MEI_ExitDevice (int num)
{
	DSL_DEV_Device_t *pDev;
	pDev = &dsl_devices[num];

	if (pDev == NULL)
		return -EIO;

	disable_irq (pDev->nIrq[AMAZON_SE_DFEIR]);
	disable_irq (pDev->nIrq[AMAZON_SE_DYING_GASP]);

	free_irq(pDev->nIrq[AMAZON_SE_DFEIR], pDev);
	free_irq(pDev->nIrq[AMAZON_SE_DYING_GASP], pDev);

	return 0;
}

static DSL_DEV_Device_t *
AMAZON_SE_BSP_HandleGet (int maj, int num)
{
	if (num > AMAZON_SE_MAX_DEVICES)
		return NULL;
	return &dsl_devices[num];
}

DSL_DEV_Device_t *
DSL_BSP_DriverHandleGet (int maj, int num)
{
	DSL_DEV_Device_t *pDev;
	if (num > AMAZON_SE_MAX_DEVICES)
		return NULL;

	pDev = &dsl_devices[num];
	pDev->nInUse++;
	MOD_INC_USE_COUNT;
	return pDev;
}

int
DSL_BSP_DriverHandleDelete (DSL_DEV_Device_t * nHandle)
{
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) nHandle;
	if (pDev->nInUse) {
		pDev->nInUse--;
		MOD_DEC_USE_COUNT;
	}
	return 0;
}
static int
AMAZON_SE_MEI_Open (DSL_DRV_inode_t * ino, DSL_DRV_file_t * fil)
{
	int maj = MAJOR (ino->i_rdev);
	int num = MINOR (ino->i_rdev);
	DSL_DEV_Device_t *pDev = NULL;
	if ((pDev = DSL_BSP_DriverHandleGet (maj, num)) == NULL) {
		AMAZON_SE_MEI_EMSG("open(%d:%d) fail!\n", maj, num);
		return -EIO;
	}
	fil->private_data = pDev;
	return 0;
}

static int
AMAZON_SE_MEI_Release (DSL_DRV_inode_t * ino, DSL_DRV_file_t * fil)
{
	//int maj = MAJOR(ino->i_rdev);
	int num = MINOR (ino->i_rdev);
	DSL_DEV_Device_t *pDev;

	pDev = &dsl_devices[num];
	if (pDev == NULL)
		return -EIO;
	DSL_BSP_DriverHandleDelete (pDev);
	return 0;
}

/**
 * Callback function for linux userspace program writing
 */
static ssize_t
AMAZON_SE_MEI_Write (DSL_DRV_file_t * filp, const char *buf, size_t size, loff_t * loff)
{
	DSL_DEV_MeiError_t mei_error = DSL_DEV_MEI_ERR_FAILURE;
	long offset = 0;
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) filp->private_data;

	if (pDev == NULL)
		return -EIO;

	mei_error =
		DSL_BSP_FWDownload (pDev, buf, size, (long *) loff,
					 &offset);
	if (mei_error == DSL_DEV_MEI_ERR_FAILURE)
		return -EIO;
	return (ssize_t) offset;
}

/**
 * Callback function for linux userspace program ioctling
 */
static int
AMAZON_SE_MEI_IoctlCopyFrom (int from_kernel, char *dest, char *from, int size)
{
	int ret = 0;

	if (!from_kernel)
		ret = copy_from_user ((char *) dest, (char *) from, size);
	else
		ret = memcpy ((char *) dest, (char *) from, size);
	return ret;
}

static int
AMAZON_SE_MEI_IoctlCopyTo (int from_kernel, char *dest, char *from, int size)
{
	int ret = 0;

	if (!from_kernel)
		ret = copy_to_user ((char *) dest, (char *) from, size);
	else
		ret = memcpy ((char *) dest, (char *) from, size);
	return ret;
}

static int
AMAZON_SE_MEI_Ioctls (DSL_DEV_Device_t * pDev, int from_kernel, unsigned int command, unsigned long lon)
{
	int i = 0;
	int meierr = DSL_DEV_MEI_ERR_SUCCESS;
	u32 base_address = MEI_SPACE_ACCESS;
	DSL_DEV_WinHost_Message_t winhost_msg, m;
	DSL_DEV_MeiDebug_t debugrdwr;
	DSL_DEV_MeiReg_t regrdwr;

	switch (command) {

	case DSL_FIO_BSP_CMV_WINHOST:
		AMAZON_SE_MEI_IoctlCopyFrom (from_kernel, (char *) winhost_msg.msg.TxMessage,
					     (char *) lon, MSG_LENGTH * 2);

		if ((meierr =
		     DSL_BSP_SendCMV (pDev, winhost_msg.msg.TxMessage, YES_REPLY,
					   winhost_msg.msg.RxMessage)) != DSL_DEV_MEI_ERR_SUCCESS) {
			AMAZON_SE_MEI_EMSG
				("WINHOST CMV fail :TxMessage:%X %X %X %X, RxMessage:%X %X %X %X %X\n",
				 winhost_msg.msg.TxMessage[0], winhost_msg.msg.TxMessage[1], winhost_msg.msg.TxMessage[2], winhost_msg.msg.TxMessage[3], 
				 winhost_msg.msg.RxMessage[0], winhost_msg.msg.RxMessage[1], winhost_msg.msg.RxMessage[2], winhost_msg.msg.RxMessage[3], 
				 winhost_msg.msg.RxMessage[4]);
			meierr = DSL_DEV_MEI_ERR_FAILURE;
		}
		else {
			AMAZON_SE_MEI_IoctlCopyTo (from_kernel, (char *) lon,
						   (char *) winhost_msg.msg.RxMessage,
						   MSG_LENGTH * 2);
		}
		break;

	case DSL_FIO_BSP_CMV_READ:
		AMAZON_SE_MEI_IoctlCopyFrom (from_kernel, (char *) (&regrdwr),
					     (char *) lon, sizeof (DSL_DEV_MeiReg_t));

		AMAZON_SE_MEI_LongWordRead ((u32) regrdwr.iAddress,
					    (u32 *) & (regrdwr.iData));

		AMAZON_SE_MEI_IoctlCopyTo (from_kernel, (char *) lon,
					   (char *) (&regrdwr),
					   sizeof (DSL_DEV_MeiReg_t));

		break;

	case DSL_FIO_BSP_CMV_WRITE:
		AMAZON_SE_MEI_IoctlCopyFrom (from_kernel, (char *) (&regrdwr),
					     (char *) lon, sizeof (DSL_DEV_MeiReg_t));

		AMAZON_SE_MEI_LongWordWrite ((u32) regrdwr.iAddress,
					     regrdwr.iData);
		break;

	case DSL_FIO_BSP_GET_BASE_ADDRESS:
		AMAZON_SE_MEI_IoctlCopyTo (from_kernel, (char *) lon,
					   (char *) (&base_address),
					   sizeof (base_address));
		break;

	case DSL_FIO_BSP_IS_MODEM_READY:
		i = AMAZON_SE_MEI_IsModemReady (pDev);
		AMAZON_SE_MEI_IoctlCopyTo (from_kernel, (char *) lon,
					   (char *) (&i), sizeof (int));
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
		break;
	case DSL_FIO_BSP_RESET:
	case DSL_FIO_BSP_REBOOT:
		meierr = AMAZON_SE_MEI_CpuModeSet (pDev, DSL_CPU_RESET);
		meierr = AMAZON_SE_MEI_CpuModeSet (pDev, DSL_CPU_HALT);
		break;

	case DSL_FIO_BSP_HALT:
		meierr = AMAZON_SE_MEI_CpuModeSet (pDev, DSL_CPU_HALT);
		break;

	case DSL_FIO_BSP_RUN:
		meierr = AMAZON_SE_MEI_CpuModeSet (pDev, DSL_CPU_RUN);
		break;
	case DSL_FIO_BSP_BOOTDOWNLOAD:
		meierr = AMAZON_SE_MEI_DownloadBootCode (pDev);
		break;
	case DSL_FIO_BSP_JTAG_ENABLE:
		meierr = AMAZON_SE_MEI_ArcJtagEnable (pDev, 1);
		break;

	case DSL_FIO_BSP_REMOTE:
		AMAZON_SE_MEI_IoctlCopyFrom (from_kernel, (char *) (&i),
					     (char *) lon, sizeof (int));

		meierr = AMAZON_SE_MEI_AdslMailboxIRQEnable (pDev, i);
		break;

	case DSL_FIO_BSP_DSL_START:
		if ((meierr = AMAZON_SE_MEI_RunAdslModem (pDev)) != DSL_DEV_MEI_ERR_SUCCESS) {
			AMAZON_SE_MEI_EMSG ("AMAZON_SE_MEI_RunAdslModem() error...");
			meierr = DSL_DEV_MEI_ERR_FAILURE;
		}
		break;

	case DSL_FIO_BSP_DEBUG_READ:
	case DSL_FIO_BSP_DEBUG_WRITE:
		AMAZON_SE_MEI_IoctlCopyFrom (from_kernel,
					     (char *) (&debugrdwr),
					     (char *) lon,
					     sizeof (debugrdwr));

		if (command == DSL_FIO_BSP_DEBUG_READ)
			meierr = DSL_BSP_MemoryDebugAccess (pDev,
								 DSL_BSP_MEMORY_READ,
								 debugrdwr.
								 iAddress,
								 (u32 *)
								 debugrdwr.
								 buffer,
								 debugrdwr.
								 iCount);
		else
			meierr = DSL_BSP_MemoryDebugAccess (pDev,
								 DSL_BSP_MEMORY_WRITE,
								 debugrdwr.
								 iAddress,
								 (u32 *)
								 debugrdwr.
								 buffer,
								 debugrdwr.
								 iCount);

		AMAZON_SE_MEI_IoctlCopyTo (from_kernel, (char *) lon, (char *) (&debugrdwr), sizeof (debugrdwr));
		break;
	case DSL_FIO_BSP_GET_VERSION:
		AMAZON_SE_MEI_IoctlCopyTo (from_kernel, (char *) lon, (char *) (&amazon_se_mei_version), sizeof (DSL_DEV_Version_t));
		break;

	case DSL_FIO_BSP_GET_CHIP_INFO:
		amazon_se_chip_info.major = 1;
		amazon_se_chip_info.minor = AMAZON_SE_MPS_CHIPID_VERSION_GET(chipid);
		AMAZON_SE_MEI_IoctlCopyTo (from_kernel, (char *) lon, (char *) (&amazon_se_chip_info), sizeof (DSL_DEV_HwVersion_t));
		break;

        case DSL_FIO_BSP_FREE_RESOURCE: /*Sumedh: removed code under this case!*/
		break;
	case DSL_FIO_BSP_FREE_RESOURCE_NEW:
                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_STAT, 4, 0, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		AMAZON_SE_MEI_DMSG("RxMessage[4] = %#x\n", m.msg.RxMessage[4]);
		if (!(m.msg.RxMessage[4] & DSL_DEV_STAT_CODESWAP_COMPLETE))
			return -EAGAIN;
		AMAZON_SE_MEI_DMSG("Freeing all memories marked FREE_SHOWTIME\n");
	        AMAZON_SE_MEI_DFEMemoryFree (pDev, FREE_SHOWTIME);
		break;
	/*Sumedh--ADD CASE TO FREE ALL */
        case DSL_FIO_BSP_FREE_ALL_RESOURCES:
                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_STAT, 4, 0, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		AMAZON_SE_MEI_DMSG("RxMessage[4] = %#x\n", m.msg.RxMessage[4]);
		if (!(m.msg.RxMessage[4] & DSL_DEV_STAT_CODESWAP_COMPLETE))
			return -EAGAIN;
		AMAZON_SE_MEI_DMSG("Freeing all memories marked FREE_SHOWTIME\n");
	        AMAZON_SE_MEI_DFEMemoryFree (pDev, FREE_ALL);
		break;
#ifdef CONFIG_AMAZON_SE
	case DSL_FIO_ARC_MUX_TEST:
		AMAZON_SE_MEI_ARC_MUX_Test();
		break;
#endif
	default:
//		AMAZON_SE_MEI_EMSG("Invalid IOCTL command: %d\n");
		break;
	}
	return meierr;
}
#ifdef CONFIG_AMAZON_SE
void AMAZON_SE_MEI_ARC_MUX_Test(void)
{
	u32 *p, i;
	*AMAZON_SE_RCU_RST_REQ |= AMAZON_SE_RCU_RST_REQ_MUX_ARC;

	p = (u32*)(DFE_LDST_BASE_ADDR + IRAM0_BASE);
	AMAZON_SE_MEI_DMSG("Writing to IRAM0(%p)...\n", p);
	for (i = 0; i < IRAM0_SIZE/sizeof(u32); i++, p++) {
		*p = 0xdeadbeef;
		if (*p != 0xdeadbeef)
			AMAZON_SE_MEI_DMSG("%p: %#x\n", p, *p);
	}

	p = (u32*)(DFE_LDST_BASE_ADDR + IRAM1_BASE);
	AMAZON_SE_MEI_DMSG("Writing to IRAM1(%p)...\n", p);
	for (i = 0; i < IRAM1_SIZE/sizeof(u32); i++, p++) {
		*p = 0xdeadbeef;
		if (*p != 0xdeadbeef)
			AMAZON_SE_MEI_DMSG("%p: %#x\n", p, *p);
	}

	p = (u32*)(DFE_LDST_BASE_ADDR + BRAM_BASE);
	AMAZON_SE_MEI_DMSG("Writing to BRAM(%p)...\n", p);
	for (i = 0; i < BRAM_SIZE/sizeof(u32); i++, p++) {
		*p = 0xdeadbeef;
		if (*p != 0xdeadbeef)
			AMAZON_SE_MEI_DMSG("%p: %#x\n", p, *p);
	}

	p = (u32*)(DFE_LDST_BASE_ADDR + XRAM_BASE);
	AMAZON_SE_MEI_DMSG("Writing to XRAM(%p)...\n", p);
	for (i = 0; i < XRAM_SIZE/sizeof(u32); i++, p++) {
		*p = 0xdeadbeef;
		if (*p != 0xdeadbeef)
			AMAZON_SE_MEI_DMSG("%p: %#x\n", p, *p);
	}

	p = (u32*)(DFE_LDST_BASE_ADDR + YRAM_BASE);
	AMAZON_SE_MEI_DMSG("Writing to YRAM(%p)...\n", p);
	for (i = 0; i < YRAM_SIZE/sizeof(u32); i++, p++) {
		*p = 0xdeadbeef;
		if (*p != 0xdeadbeef)
			AMAZON_SE_MEI_DMSG("%p: %#x\n", p, *p);
	}

	p = (u32*)(DFE_LDST_BASE_ADDR + EXT_MEM_BASE);
	AMAZON_SE_MEI_DMSG("Writing to EXT_MEM(%p)...\n", p);
	for (i = 0; i < EXT_MEM_SIZE/sizeof(u32); i++, p++) {
		*p = 0xdeadbeef;
		if (*p != 0xdeadbeef)
			AMAZON_SE_MEI_DMSG("%p: %#x\n", p, *p);
	}
	*AMAZON_SE_RCU_RST_REQ &= ~AMAZON_SE_RCU_RST_REQ_MUX_ARC;
}
#endif
int
DSL_BSP_KernelIoctls (DSL_DEV_Device_t * pDev, unsigned int command,
			   unsigned long lon)
{
	int error = 0;
	error = AMAZON_SE_MEI_Ioctls (pDev, 1, command, lon);
	return error;
}

static int
AMAZON_SE_MEI_UserIoctls (DSL_DRV_inode_t * ino, DSL_DRV_file_t * fil,
			  unsigned int command, unsigned long lon)
{
	int error = 0;
	int maj = MAJOR (ino->i_rdev);
	int num = MINOR (ino->i_rdev);
	DSL_DEV_Device_t *pDev;

	pDev = AMAZON_SE_BSP_HandleGet (maj, num);
	if (pDev == NULL)
		return -EIO;

	error = AMAZON_SE_MEI_Ioctls (pDev, 0, command, lon);
	return error;
}

////////////////////     procfs debug    ///////////////////////////

#ifdef CONFIG_PROC_FS
/*
 * Register a callback function for linux proc filesystem
 */
static int
AMAZON_SE_MEI_InitProcFS (int num)
{
	struct proc_dir_entry *entry;
	int i = 0;

	DSL_DEV_Device_t *pDev;
	pDev = &dsl_devices[num];
	if (pDev == NULL)
		return -ENOMEM;
	reg_entry_t regs_temp[PROC_ITEMS] =	// Items being debugged
	{
		/* flag, name,      description } */
		{NULL, "arcmsgav", "arc to mei message ", 0},
		{NULL, "cmv_reply", "cmv needs reply", 0},
		{NULL, "cmv_waiting", "waiting for cmv reply from arc", 0},
		{NULL, "modem_ready_cnt", "ARC to MEI indicator count", 0},
		{NULL, "cmv_count", "MEI to ARC CMVs", 0},
		{NULL, "reply_count", "ARC to MEI Reply", 0},
		{NULL, "Recent_indicator", "most recent indicator", 0},
		{NULL, "fw_version", "Firmware Version", 0},
		{NULL, "fw_date", "Firmware Date", 0},
		{NULL, "meminfo", "Memory Allocation Information", 0},
	};

	regs_temp[0].flag = &(((amazon_se_mei_device_private_t *) pDev->pPriv)->arcmsgav);
	regs_temp[1].flag = &(((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_reply);
	regs_temp[2].flag = &(((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_waiting);
	regs_temp[3].flag = &(((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready_cnt);
	regs_temp[4].flag = &(((amazon_se_mei_device_private_t *) pDev->pPriv)->cmv_count);
	regs_temp[5].flag = &(((amazon_se_mei_device_private_t *) pDev->pPriv)->reply_count);
	regs_temp[6].flag = (int *) &(((amazon_se_mei_device_private_t *) pDev->pPriv)->Recent_indicator);

	memcpy ((char *) regs[num], (char *) regs_temp, sizeof (regs_temp));
	// procfs
	meidir = proc_mkdir (MEI_DIRNAME, &proc_root);
	if (meidir == NULL) {
		AMAZON_SE_MEI_EMSG ("Failed to create /proc/%s\n" MEI_DIRNAME);
		return (-ENOMEM);
	}

	for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
		entry = create_proc_entry (regs[num][i].name,
					   S_IWUSR | S_IRUSR | S_IRGRP |
					   S_IROTH, meidir);
		if (entry) {
			regs[num][i].low_ino = entry->low_ino;
			entry->proc_fops = &AMAZON_SE_MEI_ProcOperations;
		}
		else {
			AMAZON_SE_MEI_EMSG ("Failed to create /proc/%s/%s\n" MEI_DIRNAME, regs[num][i].name);
			return (-ENOMEM);
		}
	}
	return 0;
}

/*
 * Reading function for linux proc filesystem
 */
static int
AMAZON_SE_MEI_ProcRead (struct file *file, char *buf, size_t nbytes, loff_t * ppos)
{
	int i_ino = (file->f_dentry->d_inode)->i_ino;
	char *p = buf;
	int i;
	int num;
	reg_entry_t *entry = NULL;
	DSL_DEV_Device_t *pDev = NULL;
	DSL_DEV_WinHost_Message_t m;

	for (num = 0; num < AMAZON_SE_MAX_DEVICES; num++) {
		for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
			if (regs[num][i].low_ino == i_ino) {
				entry = &regs[num][i];
				pDev = &dsl_devices[num];
				break;
			}
		}
	}
	if (entry == NULL)
		return -EINVAL;
	else if (strcmp(entry->name, "meminfo") == 0) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;
		p += sprintf (p, "No           Address     Size\n");
		for (i = 0; i < MAX_BAR_REGISTERS; i++) {
			p += sprintf (p, "BAR[%02d] Addr:0x%08X Size:%lu\n",
					  i, (u32) ((amazon_se_mei_device_private_t *) pDev->pPriv)->adsl_mem_info[i].address, 
					  ((amazon_se_mei_device_private_t *) pDev->pPriv)-> adsl_mem_info[i].size);
			//printk( "BAR[%02d] Addr:0x%08X Size:%d\n",i,adsl_mem_info[i].address,adsl_mem_info[i].size);
		}
		*ppos += (p - buf);
	} else if (strcmp(entry->name, "fw_version") == 0) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;
		if (((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready_cnt < 1)
			return -EAGAIN;
                //major:bits 0-7
                //minor:bits 8-15
                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_INFO, 54, 0, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		p += sprintf(p, "FW Version: %d.%d.", m.msg.RxMessage[4] & 0xFF, (m.msg.RxMessage[4] >> 8) & 0xFF);
                //sub_version:bits 4-7
                //int_version:bits 0-3
                //spl_appl:bits 8-13
                //rel_state:bits 14-15
                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_INFO, 54, 1, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		p += sprintf(p, "%d.%d.%d.%d\n",
			(m.msg.RxMessage[4] >> 4) & 0xF, m.msg.RxMessage[4] & 0xF, 
			(m.msg.RxMessage[4] >> 14) & 3, (m.msg.RxMessage[4] >> 8) & 0x3F);
                *ppos += (p - buf);
	} else if (strcmp(entry->name, "fw_date") == 0) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;
		if (((amazon_se_mei_device_private_t *) pDev->pPriv)->modem_ready_cnt < 1)
			return -EAGAIN;

                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_INFO, 55, 0, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		/* Day/Month */
		p += sprintf(p, "FW Date: %d.%d.", m.msg.RxMessage[4] & 0xFF, (m.msg.RxMessage[4] >> 8) & 0xFF);

                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_INFO, 55, 2, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		/* Year */
		p += sprintf(p, "%d ", m.msg.RxMessage[4]);

                makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_INFO, 55, 1, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV (pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		/* Hour:Minute */
		p += sprintf(p, "%d:%d\n", (m.msg.RxMessage[4] >> 8) & 0xFF, m.msg.RxMessage[4] & 0xFF);

                *ppos += (p - buf);
	} else if (entry->flag != (int *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->Recent_indicator) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;	// indicates end of file
		p += sprintf (p, "0x%08X\n\n", *(entry->flag));
		*ppos += (p - buf);
		if ((p - buf) > nbytes)	/* Assume output can be read at one time */
			return -EINVAL;
	} else {
		if ((int) (*ppos) / ((int) 7) == 16)
			return 0;	// indicate end of the message
		p += sprintf (p, "0x%04X\n\n", *(((u16 *) (entry->flag)) + (int) (*ppos) / ((int) 7)));
		*ppos += (p - buf);
	}
	return p -  buf;
}

/*
 * Writing function for linux proc filesystem
 */
static ssize_t
AMAZON_SE_MEI_ProcWrite (struct file *file, const char *buffer, size_t count, loff_t * ppos)
{
	int i_ino = (file->f_dentry->d_inode)->i_ino;
	reg_entry_t *current_reg = NULL;
	int i = 0;
	int num = 0;
	unsigned long newRegValue = 0;
	char *endp = NULL;
	DSL_DEV_Device_t *pDev = NULL;

	for (num = 0; num < AMAZON_SE_MAX_DEVICES; num++) {
		for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
			if (regs[num][i].low_ino == i_ino) {
				current_reg = &regs[num][i];
				pDev = &dsl_devices[num];
				break;
			}
		}
	}
	if ((current_reg == NULL)
	    || (current_reg->flag ==
		(int *) ((amazon_se_mei_device_private_t *) pDev->pPriv)->
		Recent_indicator))
		return -EINVAL;

	newRegValue = simple_strtoul (buffer, &endp, 0);
	*(current_reg->flag) = (int) newRegValue;
	return (count + endp - buffer);
}
#endif //CONFIG_PROC_FS

////////////////////////////////////////////////////////////////////////////
/*
 * Writing function for linux proc filesystem
 */
int __init
AMAZON_SE_MEI_ModuleInit (void)
{
	int i = 0;

	printk ("Amazon_SE MEI version:%d.%02d.%02d\n",
		amazon_se_mei_version.major, amazon_se_mei_version.minor,
		amazon_se_mei_version.revision);

	for (i = 0; i < AMAZON_SE_MAX_DEVICES; i++) {
		if (AMAZON_SE_MEI_InitDevice (i) != 0) {
			printk ("%s: Init device fail!\n", __FUNCTION__);
			return -EIO;
		}
		AMAZON_SE_MEI_InitDevNode (i);
#ifdef CONFIG_PROC_FS
		AMAZON_SE_MEI_InitProcFS (i);
#endif //CONFIG_PROC_FS
	}
        for (i = 0; i <= DSL_BSP_CB_LAST ; i++)
		dsl_bsp_event_callback[i].function = NULL;

#ifdef DFE_LOOPBACK
	DFE_Loopback_Test ();
#endif //DFE_LOOPBACK

	return 0;
}

void __exit
AMAZON_SE_MEI_ModuleExit (void)
{
	int i = 0;
	int num;

	for (num = 0; num < AMAZON_SE_MAX_DEVICES; num++) {
		AMAZON_SE_MEI_CleanUpDevNode (num);
#ifdef CONFIG_PROC_FS
		for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
			remove_proc_entry (regs[num][i].name, meidir);
		}
#endif
	}

	remove_proc_entry (MEI_DIRNAME, &proc_root);
	for (i = 0; i < AMAZON_SE_MAX_DEVICES; i++) {
		for (i = 0; i < AMAZON_SE_MAX_DEVICES; i++) {
			AMAZON_SE_MEI_ExitDevice (i);
		}
	}
}

/* export function for DSL Driver */

/* The functions of MEI_DriverHandleGet and MEI_DriverHandleDelete are 
something like open/close in kernel space , where the open could be used 
to register a callback for autonomous messages and returns a mei driver context pointer (comparable to the file descriptor in user space) 
   The context will be required for the multi line chips future! */

EXPORT_SYMBOL (DSL_BSP_DriverHandleGet);
EXPORT_SYMBOL (DSL_BSP_DriverHandleDelete);

EXPORT_SYMBOL (DSL_BSP_ATMLedCBRegister);
EXPORT_SYMBOL (DSL_BSP_ATMLedCBUnregister);
EXPORT_SYMBOL (DSL_BSP_KernelIoctls);
EXPORT_SYMBOL (DSL_BSP_AdslLedInit);
EXPORT_SYMBOL (DSL_BSP_AdslLedSet);
EXPORT_SYMBOL (DSL_BSP_FWDownload);
EXPORT_SYMBOL (DSL_BSP_Showtime);

EXPORT_SYMBOL (DSL_BSP_MemoryDebugAccess);
EXPORT_SYMBOL (DSL_BSP_SendCMV);

// provide a register/unregister function for DSL driver to register a event callback function
EXPORT_SYMBOL (DSL_BSP_EventCBRegister);
EXPORT_SYMBOL (DSL_BSP_EventCBUnregister);

module_init (AMAZON_SE_MEI_ModuleInit);
module_exit (AMAZON_SE_MEI_ModuleExit);
