/******************************************************************************
**
** FILE NAME    : amazon_se_rcu.c
** PROJECT      : Amazon_se
** MODULES      : RCU
**
** DATE         : 09 Aug 2005
** AUTHOR       : Huang Xiaogang
** DESCRIPTION  : Amazon_se Reset Control Logic(RCU) driver
** COPYRIGHT    :       Copyright (c) 2006
**                      Infineon Technologies AG
**                      Am Campeon 1-12, 85579 Neubiberg, Germany
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date        $Author         $Comment
** 01 Jan 2006  Huang Xiaogang  modification & verification on Amazon_se chip
*******************************************************************************/

#include<linux/module.h>
#include<linux/init.h>
#include<linux/proc_fs.h>
#include<linux/kdev_t.h>
#include<asm/atomic.h>
#include<linux/fs.h>

#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/amazon_se/amazon_se_rcu.h>
#include <asm/amazon_se/amazon_se.h>

typedef struct rcu_dev{
      char name[16];
      int major;
      int minor;
      int occupied;
      int count;
      char buff[10];
}rcu_dev;

#define AMAZON_SE_RCU_EMSG(fmt, args...)                                printk(KERN_DEBUG "%s: " fmt, __FUNCTION__ , ##args)
#define DEVICE_NAME                                             "amazon_se_rcu"

static struct rcu_dev *amazon_se_rcu_dev;
static struct proc_dir_entry* amazon_se_rcu_dir;

static int rcu_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
    int result=0;

    amazon_se_rst_req_parm_t rst_req;
    amazon_se_rst_stat_parm_t rst_stat;

    if(_IOC_TYPE(cmd)!= AMAZON_SE_RCU_IOC_MAGIC) return -ENOTTY;
    if(_IOC_NR(cmd) > AMAZON_SE_RCU_IOC_MAXNR ) return -ENOTTY;
    
    switch(cmd){
        case AMAZON_SE_RCU_IOC_SET_RCU_RST_REQ:
        {
          if(copy_from_user(&rst_req, (void*)arg, sizeof(rst_req)) < 0)
          {
                AMAZON_SE_RCU_EMSG("invalid argument \n "); 
                result = -EINVAL;
          }
          *AMAZON_SE_RCU_RST_REQ = rst_req.amazon_se_rst_req;
          break;
        }
        case AMAZON_SE_RCU_IOC_GET_RCU_RST_REQ:
        {
          rst_req.amazon_se_rst_req = *AMAZON_SE_RCU_RST_REQ;
          if(copy_to_user((void*)arg, &rst_req, sizeof(rst_req)) < 0){
                printk("KERN_ERR: AMAZON_SE_RCU_IOC_GET_RCU_RST_REQ \n");
                result = -EINVAL;
          }
          break;
        }
        case AMAZON_SE_RCU_IOC_GET_STATUS:
        {
          rst_stat.amazon_se_rst_stat = *AMAZON_SE_RCU_RST_STAT; 
          printk("rst_stat.amazon_se_rst_stat = 0x%08x\n", rst_stat.amazon_se_rst_stat);
          if(copy_to_user((void*)arg, &rst_stat, sizeof(rst_stat)) < 0){

                AMAZON_SE_RCU_EMSG("KERN_ERR: AMAZON_SE_RCU_IOC_GET_STATUS \n");
                result = -EINVAL;
          }
          break;
        }
        default:
          result = -ENOIOCTLCMD;
          break;
      }
      return result;
}

static int rcu_open(struct inode *inode, struct file *file)
{
        if (amazon_se_rcu_dev->occupied == 1) return -EBUSY;
        amazon_se_rcu_dev->occupied = 1;
        MOD_INC_USE_COUNT;
        return 0;
}

int rcu_release(struct inode *inode, struct file *file)
{
        amazon_se_rcu_dev->occupied = 0;
        MOD_DEC_USE_COUNT;
        return 0;
}

int rcu_register_proc_read(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{

        int len=0;
        unsigned int rcu_rst_stat;
        unsigned int rcu_rst_req;

        len+=sprintf(buf+len ,"AMAZON_SE_RCU_REGISTERS\n");
        /* print internals of rst_stat reg */
        rcu_rst_stat = *AMAZON_SE_RCU_RST_STAT;
        len+=sprintf(buf+len,"AMAZON_SE_RCU_STAT                    : 0x%08x\n", rcu_rst_stat);

        /* print internals of rst_req reg */
        rcu_rst_req = *AMAZON_SE_RCU_RST_REQ;
        len+=sprintf(buf+len,"AMAZON_SE_RCU_RST_REQ          : 0x%08x\n", rcu_rst_req);
 
        *eof = 1;
        return len;
}

static struct file_operations rcu_fops = {
        ioctl    :  rcu_ioctl,
        open     :  rcu_open,
        release  :  rcu_release,
};
static int __init
init_module_amazon_se_rcu(void)
{
        int result=0;   

        amazon_se_rcu_dev = (rcu_dev*)kmalloc(sizeof(rcu_dev),GFP_KERNEL);

        if (amazon_se_rcu_dev == NULL){
                return -ENOMEM;
        }
        memset(amazon_se_rcu_dev,0,sizeof(rcu_dev));
        strcpy(amazon_se_rcu_dev->name, DEVICE_NAME);
        amazon_se_rcu_dev->minor = 0;
        result = register_chrdev(0, amazon_se_rcu_dev->name, &rcu_fops);
        if (result < 0) {
             AMAZON_SE_RCU_EMSG("cannot register device\n");
             kfree(amazon_se_rcu_dev); 
             if(result == -EINVAL )
                AMAZON_SE_RCU_EMSG(" The specified number is not valid\n");                     
             if(result == -EBUSY)
                AMAZON_SE_RCU_EMSG(" The major number is busy \n");     
             return -EINVAL;                            
        }
        amazon_se_rcu_dev->major = result; 
        create_proc_read_entry( "amazon_se_rcu", 0, amazon_se_rcu_dir, rcu_register_proc_read, NULL);  
        amazon_se_rcu_dev->occupied=0;
       return 0;
}
static void __exit
amazon_se_rcu_cleanup_module(void)
{

        unregister_chrdev(amazon_se_rcu_dev->major,amazon_se_rcu_dev->name);
        kfree(amazon_se_rcu_dev);  
        remove_proc_entry("amazon_se_rcu",amazon_se_rcu_dir);
}

MODULE_LICENSE ("GPL");
MODULE_AUTHOR("Infineon IFAP DC COM");
MODULE_DESCRIPTION("AMAZON_SE RCU driver");

module_init(init_module_amazon_se_rcu);
module_exit(amazon_se_rcu_cleanup_module);



