/* ==========================================================================
 * $File: //dwh/usb_iip/dev/software/otg_ipmate/linux/drivers/dwc_otg_pcd.c $
 * $Revision: #8 $
 * $Date: 2006/04/21 $
 * $Change: 631780 $
 *
 * Synopsys HS OTG Linux Software Driver and documentation (hereinafter,
 * "Software") is an Unsupported proprietary work of Synopsys, Inc. unless
 * otherwise expressly agreed to in writing between Synopsys and you.
 *
 * The Software IS NOT an item of Licensed Software or Licensed Product under
 * any End User Software License Agreement or Agreement for Licensed Product
 * with Synopsys or any supplement thereto. You are permitted to use and
 * redistribute this Software in source and binary forms, with or without
 * modification, provided that redistributions of source code must retain this
 * notice. You may not view, use, disclose, copy or distribute this file or
 * any information contained herein except pursuant to this license grant from
 * Synopsys. If you do not agree with this notice, including the disclaimer
 * below, then you are not authorized to use the Software.
 *
 * THIS SOFTWARE IS BEING DISTRIBUTED BY SYNOPSYS SOLELY ON AN "AS IS" BASIS
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE HEREBY DISCLAIMED. IN NO EVENT SHALL SYNOPSYS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * ========================================================================== */
#ifndef DWC_HOST_ONLY

/** @file
 * This file implements the Peripheral Controller Driver.
 *
 * The Peripheral Controller Driver (PCD) is responsible for
 * translating requests from the Function Driver into the appropriate
 * actions on the DWC_otg controller. It isolates the Function Driver
 * from the specifics of the controller by providing an API to the
 * Function Driver.
 *
 * The Peripheral Controller Driver for Linux will implement the
 * Gadget API, so that the existing Gadget drivers can be used.
 * (Gadget Driver is the Linux terminology for a Function Driver.)
 *
 * The Linux Gadget API is defined in the header file
 * <code><linux/usb_gadget.h></code>.  The USB EP operations API is
 * defined in the structure <code>usb_ep_ops</code> and the USB
 * Controller API is defined in the structure
 * <code>usb_gadget_ops</code>.
 *
 * An important function of the PCD is managing interrupts generated
 * by the DWC_otg controller. The implementation of the DWC_otg device
 * mode interrupt service routines is in dwc_otg_pcd_intr.c.
 *
 * @todo Add Device Mode test modes (Test J mode, Test K mode, etc).
 * @todo Does it work when the request size is greater than DEPTSIZ
 * transfer size
 *
 */

//Make options
//#define USE_ORIG

//Compiling options
//#define GADGET_NEEDS_ALIGNED_BUFFERS



#ifdef USE_ORIG
#undef GADGET_NEEDS_ALIGNED_BUFFERS
#endif



#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#ifdef LINUX26
#include <linux/device.h>
#endif
#include <linux/errno.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/string.h>
#ifdef LINUX26
#include <linux/dma-mapping.h>
#include <asm/arch/lm.h>
#include <asm/arch/irqs.h>
#endif

#include <linux/usb_ch9.h>
#include <linux/usb_gadget.h>

#include "dwc_otg_driver.h"
#include "dwc_otg_pcd.h"

#ifdef CONFIG_IFX
#include "dwc_otg_ifx.h" // for Infineon platform specific.
#endif


#ifndef IRQ_NONE
typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)
#endif

/**
 * Static PCD pointer for use in usb_gadget_register_driver and
 * usb_gadget_unregister_driver.  Initialized in dwc_otg_pcd_init.
 */
static   dwc_otg_pcd_t *s_pcd = 0;


/* Display the contents of the buffer */
extern void dump_msg(const u8 *buf, unsigned int length);


/**
 * This function completes a request.  It call's the request call back.
 */
void request_done(dwc_otg_pcd_ep_t *_ep, dwc_otg_pcd_request_t *_req, int _status){
	unsigned stopped = _ep->stopped;

HDEBUG_SE(0);
	DWC_DEBUGPL(DBG_PCDV, "%s(%p)\n", __func__, _ep);
	list_del_init(&_req->queue);

	if (_req->req.status == -EINPROGRESS) {
		_req->req.status = _status;
	} else {
		_status = _req->req.status;
	}

	/* don't modify queue heads during completion callback */
	_ep->stopped = 1;
	SPIN_UNLOCK(&_ep->pcd->lock);
	_req->req.complete(&_ep->ep, &_req->req);
	SPIN_LOCK(&_ep->pcd->lock);

	if (_ep->pcd->request_pending > 0){
		--_ep->pcd->request_pending;
	}
	_ep->stopped = stopped;
}

/**
 * This function terminates all the requsts in the EP request queue.
 */
void request_nuke( dwc_otg_pcd_ep_t *_ep )
{
	dwc_otg_pcd_request_t *req;

HDEBUG_S;
	_ep->stopped = 1;
	/* called with irqs blocked?? */
/*
	while (!list_empty(&_ep->queue)) {
		req = list_entry(_ep->queue.next, dwc_otg_pcd_request_t,
                                 queue);
#if GADGET_NEEDS_ALIGNED_BUFFERS
		if(req->flags==1&&req->unaligned_buf) {
			memcpy(req->unaligned_buf, req->req.buf, req->req.length);
			kfree(req->req.buf);
			req->req.buf = req->unaligned_buf;
			req->unaligned_buf=0;
			req->flags=0;
		}
#endif
		request_done(_ep, req, -ESHUTDOWN );
		printk(".");
	}
*/
	while (!list_empty(&_ep->queue)) {
		list_del_init(&_ep->queue);
		printk(".");
	}
HDEBUG_E(0);
}

/* USB Endpoint Operations */
/*
 * The following sections briefly describe the behavior of the Gadget
 * API endpoint operations implemented in the DWC_otg driver
 * software. Detailed descriptions of the generic behavior of each of
 * these functions can be found in the Linux header file
 * include/linux/usb_gadget.h.
 *
 * The Gadget API provides wrapper functions for each of the function
 * pointers defined in usb_ep_ops. The Gadget Driver calls the wrapper
 * function, which then calls the underlying PCD function. The
 * following sections are named according to the wrapper
 * functions. Within each section, the corresponding DWC_otg PCD
 * function name is specified.
 *
 */

/**
 * This function is called by the Gadget Driver for each EP to be
 * configured for the current configuration (SET_CONFIGURATION).
 *
 * This function initializes the dwc_otg_ep_t data structure, and then
 * calls dwc_otg_ep_activate.
 */
static int dwc_otg_pcd_ep_enable(struct usb_ep *_ep,
                 const struct usb_endpoint_descriptor *_desc){
	dwc_otg_pcd_ep_t *ep = 0;
	dwc_otg_pcd_t *pcd = 0;
	unsigned long flags;


HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p,%p)\n", __func__, _ep, _desc );

	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);
	if (!_ep || !_desc || ep->desc ||
	    _desc->bDescriptorType != USB_DT_ENDPOINT) {
		if (!_ep )
			DWC_WARN( "%s, bad ep or descriptor(EP=NULL)\n", __func__);
		if (!_desc)
			DWC_WARN( "%s, bad ep or descriptor(DESC=NULL)\n", __func__);
		if (ep->desc)
			DWC_WARN( "%s, bad ep or descriptor(EP-DESC=NULL)\n", __func__);
		if (_desc->bDescriptorType != USB_DT_ENDPOINT)
			DWC_WARN( "%s, bad ep or descriptor(WRONG TYPE[%d]\n", __func__,_desc->bDescriptorType);


HDEBUG_E(1);
		return -EINVAL;
	}
	if (ep == &ep->pcd->ep[0]){
		DWC_WARN("%s, bad ep(0)\n", __func__);
HDEBUG_E(2);
		return -EINVAL;
	}

	/* Check FIFO size? */
	if (!_desc->wMaxPacketSize) {
		DWC_WARN("%s, bad %s maxpacket\n", __func__, _ep->name);
HDEBUG_E(3);
		return -ERANGE;
	}

	pcd = ep->pcd;
	if (!pcd->driver || pcd->gadget.speed == USB_SPEED_UNKNOWN) {
		DWC_WARN("%s, bogus device state\n", __func__);
HDEBUG_E(4);
		return -ESHUTDOWN;
	}

	SPIN_LOCK_IRQSAVE(&pcd->lock, flags);

	ep->desc = _desc;
	ep->ep.maxpacket = le16_to_cpu (_desc->wMaxPacketSize);

	/*
	 * Activate the EP
	 */
	ep->stopped = 0;
	ep->dwc_ep.is_in = (USB_DIR_IN & _desc->bEndpointAddress) != 0;
	ep->dwc_ep.maxpacket = ep->ep.maxpacket;
	ep->dwc_ep.tx_fifo_num = 0;
	ep->dwc_ep.type = _desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
	if ((_desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_ISOC
	  //||(_desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_INT   //howard
	) {
		/*
		 * if ISOC EP then assign a Periodic Tx FIFO.
		 */
		/** @todo NGS Determine Tx FIFO for periodic EP.
		 *  Currently using 1.
		 */
		ep->dwc_ep.tx_fifo_num = 1;
	}
	/* Set initial data PID. */
	if ((_desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_BULK ) {
		ep->dwc_ep.data_pid_start = 0;
	}

	DWC_DEBUGPL(DBG_PCD, "Activate %s-%s: type=%d, mps=%d desc=%p\n",
	                ep->ep.name, (ep->dwc_ep.is_in ?"IN":"OUT"),
	                ep->dwc_ep.type, ep->dwc_ep.maxpacket, ep->desc );

	dwc_otg_ep_activate( GET_CORE_IF(pcd), &ep->dwc_ep );
	SPIN_UNLOCK_IRQRESTORE(&pcd->lock, flags);
HDEBUG_E(5);
	return 0;
}

/**
 * This function is called when an EP is disabled due to disconnect or
 * change in configuration. Any pending requests will terminate with a
 * status of -ESHUTDOWN.
 *
 * This function modifies the dwc_otg_ep_t data structure for this EP,
 * and then calls dwc_otg_ep_deactivate.
 */
static int dwc_otg_pcd_ep_disable(struct usb_ep *_ep){
	dwc_otg_pcd_ep_t *ep;
	unsigned long flags;


HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p)\n", __func__, _ep);
	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);
	if (!_ep || !ep->desc) {
		DWC_DEBUGPL(DBG_PCD, "%s, %s not enabled\n", __func__,
		       _ep ? ep->ep.name : NULL);
HDEBUG_E(1);
		return -EINVAL;
	}

	SPIN_LOCK_IRQSAVE(&ep->pcd->lock, flags);
	request_nuke( ep );
	dwc_otg_ep_deactivate( GET_CORE_IF(ep->pcd), &ep->dwc_ep );
HDEBUG_C(1,"EP Disabled %p %p\n",ep,ep->desc);
	ep->desc = 0;
	ep->stopped = 1;
	SPIN_UNLOCK_IRQRESTORE(&ep->pcd->lock, flags);

	DWC_DEBUGPL(DBG_PCD, "%s disabled\n", _ep->name);
HDEBUG_E(2);
	return 0;
}


/**
 * This function allocates a request object to use with the specified
 * endpoint.
 *
 * @param _ep The endpoint to be used with with the request
 * @param _gfp_flags the GFP_* flags to use.
 */
static struct usb_request *dwc_otg_pcd_alloc_request(struct usb_ep *_ep, int _gfp_flags){
	dwc_otg_pcd_request_t *req;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p,%d)\n", __func__, _ep, _gfp_flags);
	if (0 == _ep ) {
		DWC_WARN("%s() %s\n", __func__, "Invalid EP!\n");
HDEBUG_E(1);
		return 0;
	}
	req = kmalloc( sizeof(dwc_otg_pcd_request_t), _gfp_flags);
	if (0 == req){
		DWC_WARN("%s() %s\n", __func__, "request allocation failed!\n");
HDEBUG_E(2);
		return 0;
	}
	memset(req, 0, sizeof(dwc_otg_pcd_request_t));
	req->req.dma = DMA_ADDR_INVALID;
	INIT_LIST_HEAD(&req->queue);
HDEBUG_E(3);
	return &req->req;
}

/**
 * This function frees a request object.
 *
 * @param _ep The endpoint associated with the request
 * @param _req The request being freed
 */
static void dwc_otg_pcd_free_request(struct usb_ep *_ep, struct usb_request *_req){
	dwc_otg_pcd_request_t *req;
HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p,%p)\n", __func__, _ep, _req);

	if (0 == _ep || 0 == _req) {
		DWC_WARN("%s() %s\n", __func__, "Invalid ep or req argument!\n");
HDEBUG_E(1);
		return;
	}

	req = container_of(_req, dwc_otg_pcd_request_t, req);
	kfree(req);
HDEBUG_E(2);
}

/**
 * This function allocates an I/O buffer to be used for a transfer
 * to/from the specified endpoint.
 *
 * @param _ep The endpoint to be used with with the request
 * @param _bytes The desired number of bytes for the buffer
 * @param _dma Pointer to the buffer's DMA address; must be valid
 * @param _gfp_flags the GFP_* flags to use.
 * @return address of a new buffer or null is buffer could not be allocated.
 */
static void *dwc_otg_pcd_alloc_buffer(struct usb_ep *_ep, unsigned _bytes,
                                 dma_addr_t *_dma, int _gfp_flags){
	void *buf;
	dwc_otg_pcd_ep_t *ep;
	dwc_otg_pcd_t *pcd = 0;

	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);
	pcd = ep->pcd;
HDEBUG_SE(0);
	DWC_DEBUGPL(DBG_PCDV,"%s(%p,%d,%p,%0x)\n", __func__, _ep, _bytes, _dma, _gfp_flags);

	/* Check dword alignment */
	if ((_bytes & 0x3UL) != 0) {
		DWC_WARN("%s() Buffer size is not a multiple of"
		            "DWORD size (%d)",__func__, _bytes);
	}

	if (GET_CORE_IF(pcd)->dma_enable) {
		buf = dma_alloc_coherent (NULL, _bytes, _dma, _gfp_flags);
	}
	else {
		buf = kmalloc( _bytes, _gfp_flags);
	}

	/* Check dword alignment */
	if (((int)buf & 0x3UL) != 0) {
		DWC_WARN("%s() Buffer is not DWORD aligned (%p)",__func__, buf);
	}
	return buf;
}

/**
 * This function frees an I/O buffer that was allocated by alloc_buffer.
 *
 * @param _ep the endpoint associated with the buffer
 * @param _buf address of the buffer
 * @param _dma The buffer's DMA address
 * @param _bytes The number of bytes of the buffer
 */
static void dwc_otg_pcd_free_buffer(struct usb_ep *_ep, void *_buf,
                                    dma_addr_t _dma, unsigned _bytes){
	dwc_otg_pcd_ep_t *ep;
	dwc_otg_pcd_t *pcd = 0;

HDEBUG_SE(0);
	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);
	pcd = ep->pcd;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p,%p,%0x,%d)\n", __func__, _ep, _buf, _dma, _bytes);

	if (GET_CORE_IF(pcd)->dma_enable) {
		dma_free_coherent (NULL, _bytes, _buf, _dma);
	}
	else {
		kfree( _buf );
	}
}

/**
 * This function is used to submit an I/O Request to an EP.
 *
 * 	- When the request completes the request's completion callback
 * 	  is called to return the request to the driver.
 *	- An EP, except control EPs, may have multiple requests
 *	  pending.
 *	- Once submitted the request cannot be examined or modified.
 *	- Each request is turned into one or more packets.
 *	- A BULK EP can queue any amount of data; the transfer is
 *	  packetized.
 *	- Zero length Packets are specified with the request 'zero'
 *	  flag.
 */
static int dwc_otg_pcd_ep_queue(struct usb_ep *_ep,
                                struct usb_request *_req, int _gfp_flags){
	int prevented = 0;
	dwc_otg_pcd_request_t *req;
	dwc_otg_pcd_ep_t *ep;
	dwc_otg_pcd_t	*pcd;
	unsigned long flags = 0;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p,%p,%d)\n",
	          __func__, _ep, _req, _gfp_flags);

	req = container_of(_req, dwc_otg_pcd_request_t, req);
	if (!_req || !_req->complete || !_req->buf ||
	           !list_empty(&req->queue)) {
		DWC_WARN("%s, bad params\n", __func__);
HDEBUG_E(1);
		return -EINVAL;
	}

	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);
	if (!_ep || (!ep->desc && ep->dwc_ep.num != 0)) {
		if(!_ep)
			DWC_WARN("%s, null ep\n", __func__);
		else
			DWC_WARN("%s, Null Descriptor\n", __func__);
HDEBUG_E(2);
		return -EINVAL;
	}

	pcd = ep->pcd;
	if (!pcd->driver || pcd->gadget.speed == USB_SPEED_UNKNOWN) {
		DWC_DEBUGPL(DBG_PCDV, "gadget.speed=%d\n", pcd->gadget.speed);
		DWC_WARN("%s, bogus device state\n", __func__);
HDEBUG_E(3);
		return -ESHUTDOWN;
	}

	DWC_DEBUGPL(DBG_PCD, "%s queue req %p, len %d buf %p\n",
	      _ep->name, _req, _req->length, _req->buf);

	if (!GET_CORE_IF(pcd)->core_params->opt) {
		if (ep->dwc_ep.num != 0) {
			DWC_ERROR("%s queue req %p, len %d buf %p\n",
			     _ep->name, _req, _req->length, _req->buf);
		}
	}

	SPIN_LOCK_IRQSAVE(&ep->pcd->lock, flags);
#if GADGET_NEEDS_ALIGNED_BUFFERS        //{{{
	/* Obviously the higher application layer submits a non aligned buffer, but we can use only
	 * buffers, which are already 32 bit aligned. This means that we have to copy the buffer
	 * to a aligned one temporarily. We have to save the original buffer pointer additionally, to
	 * be able later to restore the original request.
	 * */

	if (req->req.length && ((unsigned long) req->req.buf) % 4) {
		req->flags=1;

		if(req->unaligned_buf)
			kfree(req->unaligned_buf);
		req->unaligned_buf = req->req.buf;
		req->req.buf = kmalloc(req->req.length, GFP_ATOMIC | GFP_DMA);
		if (!req->req.buf){
			printk("Memory is not enough in skb buffer copy\n");
HDEBUG_E(4);
			return -1;
		}
		memcpy(req->req.buf, req->unaligned_buf, req->req.length);
	}
#endif

#if defined(DEBUG) & defined(VERBOSE)
	dump_msg(_req->buf, _req->length);
#endif

	_req->status = -EINPROGRESS;
	_req->actual = 0;

	/*
	 * For EP0 IN without premature status, zlp is required?
	 */
	if (ep->dwc_ep.num == 0 && ep->dwc_ep.is_in) {
		DWC_DEBUGPL(DBG_PCDV, "%s-OUT ZLP\n", _ep->name);
		//_req->zero = 1;
	}

	/* Start the transfer */
	if (list_empty(&ep->queue) && !ep->stopped) {
		/* EP0 Transfer? */
		if (ep->dwc_ep.num == 0) {
			switch (pcd->ep0state) {
				case EP0_IN_DATA_PHASE:
					DWC_DEBUGPL(DBG_PCD,
					    "%s ep0: EP0_IN_DATA_PHASE\n",
					    __func__);
					break;

				case EP0_OUT_DATA_PHASE:
					DWC_DEBUGPL(DBG_PCD,
					    "%s ep0: EP0_OUT_DATA_PHASE\n",
					    __func__);
					if (pcd->request_config) {
						/* Complete STATUS PHASE */
						ep->dwc_ep.is_in = 1;
						pcd->ep0state = EP0_STATUS;
					}
					break;

				default:
					DWC_DEBUGPL(DBG_ANY, "ep0: odd state %d\n",pcd->ep0state);
					SPIN_UNLOCK_IRQRESTORE(&pcd->lock, flags);
					return -EL2HLT;
			}

			ep->dwc_ep.dma_addr = _req->dma;
			ep->dwc_ep.start_xfer_buff = _req->buf;
			ep->dwc_ep.xfer_buff = _req->buf;
			ep->dwc_ep.xfer_len = _req->length;
			ep->dwc_ep.xfer_count = 0;
			//ep->dwc_ep.sent_zlp = 0;
			ep->dwc_ep.sent_zlp = _req->zero;
			ep->dwc_ep.total_len = ep->dwc_ep.xfer_len;
			dwc_otg_ep0_start_transfer( GET_CORE_IF(pcd),
			         &ep->dwc_ep );
		} else {
			/* Setup and start the Transfer */
			ep->dwc_ep.dma_addr = _req->dma;
			ep->dwc_ep.start_xfer_buff = _req->buf;
			ep->dwc_ep.xfer_buff = _req->buf;
			ep->dwc_ep.xfer_len = _req->length;
			ep->dwc_ep.xfer_count = 0;
			ep->dwc_ep.sent_zlp = 0;
			ep->dwc_ep.total_len = ep->dwc_ep.xfer_len;
			dwc_otg_ep_start_transfer( GET_CORE_IF(pcd),&ep->dwc_ep );
		}
	}

	if ((req != 0) || prevented) {
		++pcd->request_pending;
		list_add_tail(&req->queue, &ep->queue);
		if (ep->dwc_ep.is_in && ep->stopped && !(GET_CORE_IF(pcd)->dma_enable)) {
			/** @todo NGS Create a function for this. */
			diepmsk_data_t diepmsk = { .d32 = 0};
			diepmsk.b.intktxfemp = 1;
			dwc_modify_reg32( &GET_CORE_IF(pcd)->dev_if->dev_global_regs->diepmsk, 0, diepmsk.d32 );
		}
	}

	SPIN_UNLOCK_IRQRESTORE(&pcd->lock, flags);
HDEBUG_E(5);
	return 0;
}

/**
 * This function cancels an I/O request from an EP.
 */
static int dwc_otg_pcd_ep_dequeue(struct usb_ep *_ep,
                                  struct usb_request *_req)
{
	dwc_otg_pcd_request_t *req;
	dwc_otg_pcd_ep_t *ep;
	dwc_otg_pcd_t	*pcd;
	unsigned long flags;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p,%p)\n", __func__, _ep, _req);

	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);
	if (!_ep || !_req || (!ep->desc && ep->dwc_ep.num != 0)) {
		DWC_WARN("%s, bad argument\n", __func__);
HDEBUG_E(1);
		return -EINVAL;
	}
	pcd = ep->pcd;
	if (!pcd->driver || pcd->gadget.speed == USB_SPEED_UNKNOWN) {
		DWC_WARN("%s, bogus device state\n", __func__);
HDEBUG_E(2);
		return -ESHUTDOWN;
	}

	SPIN_LOCK_IRQSAVE(&pcd->lock, flags);
	DWC_DEBUGPL(DBG_PCDV, "%s %s %s %p\n", __func__, _ep->name,
	         ep->dwc_ep.is_in ? "IN" : "OUT",
	         _req);

	/* make sure it's actually queued on this endpoint */
	list_for_each_entry( req, &ep->queue, queue) {
		if (&req->req == _req) {
			break;
		}
	}

	if (&req->req != _req) {
		SPIN_UNLOCK_IRQRESTORE(&pcd->lock, flags);
HDEBUG_E(3);
		return -EINVAL;
	}

	if (!list_empty(&req->queue)) {
#if GADGET_NEEDS_ALIGNED_BUFFERS
		if(req->flags==1&&req->unaligned_buf) {
			memcpy(req->unaligned_buf, req->req.buf, req->req.length);
			kfree(req->req.buf);
			req->req.buf = req->unaligned_buf;
			req->unaligned_buf=0;
			req->flags=0;
		}
#endif
		request_done(ep, req, -ECONNRESET);
	} else {
		req = 0;
	}

	SPIN_UNLOCK_IRQRESTORE(&pcd->lock, flags);

HDEBUG_E(4);
	return req ? 0 : -EOPNOTSUPP;
}

/**
 * usb_ep_set_halt stalls an endpoint.
 *
 * usb_ep_clear_halt clears an endpoint halt and resets its data
 * toggle.
 *
 * Both of these functions are implemented with the same underlying
 * function. The behavior depends on the value argument.
 *
 * @param[in] _ep the Endpoint to halt or clear halt.
 * @param[in] _value
 *	- 1 means set_halt,
 *	- 0 means clear_halt.
 */
static int dwc_otg_pcd_ep_set_halt(struct usb_ep *_ep, int _value){
	int retval = 0;
	unsigned long flags;
	dwc_otg_pcd_ep_t *ep = 0;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCD,"HALT %s %d\n", _ep->name, _value);

	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);
	if (!_ep || (!ep->desc && ep != &ep->pcd->ep[0]) ||
	            ep->desc->bmAttributes == USB_ENDPOINT_XFER_ISOC) {
		if (!_ep )
			DWC_WARN("%s, null ep\n", __func__);
		if ( !ep->desc && ep != &ep->pcd->ep[0])
			DWC_WARN("%s, ep not equal to ep0\n", __func__);
		if( ep->desc->bmAttributes == USB_ENDPOINT_XFER_ISOC)
	        DWC_WARN("%s, bad ep ISOC\n", __func__);
HDEBUG_E(1);
		return -EINVAL;
	}

	SPIN_LOCK_IRQSAVE(&ep->pcd->lock, flags);
	if (!list_empty(&ep->queue)){
		DWC_WARN("%s() %s XFer In process\n", __func__, _ep->name);
HDEBUG_E(2);
		retval = -EAGAIN;
	}
	else if (_value == 0) {
		dwc_otg_ep_clear_stall( ep->pcd->otg_dev->core_if, &ep->dwc_ep );
	} else {
		if (ep->dwc_ep.num == 0) {
			ep->pcd->ep0state = EP0_STALL;
		}
		ep->stopped = 1;
		dwc_otg_ep_set_stall( ep->pcd->otg_dev->core_if,&ep->dwc_ep );
	}
	SPIN_UNLOCK_IRQRESTORE(&ep->pcd->lock, flags);
HDEBUG_E(3);
	return retval;
}


static void dwc_otg_pcd_fifo_flush (struct usb_ep *_ep){
	dwc_otg_pcd_ep_t *ep;
	dwc_otg_core_if_t *_core_if;

	ep = container_of(_ep, dwc_otg_pcd_ep_t, ep);

	if(ep->dwc_ep.is_in){//Tx
		dwc_otg_flush_tx_fifo(ep->pcd->otg_dev->core_if, ep->dwc_ep.num); /* all Tx FIFOs */
	}
}


static struct usb_ep_ops dwc_otg_pcd_ep_ops = {
	.enable		= dwc_otg_pcd_ep_enable,
	.disable	= dwc_otg_pcd_ep_disable,

	.alloc_request	= dwc_otg_pcd_alloc_request,
	.free_request	= dwc_otg_pcd_free_request,

	.alloc_buffer	= dwc_otg_pcd_alloc_buffer,
	.free_buffer	= dwc_otg_pcd_free_buffer,

	.queue		= dwc_otg_pcd_ep_queue,
	.dequeue	= dwc_otg_pcd_ep_dequeue,

	.set_halt	= dwc_otg_pcd_ep_set_halt,
	.fifo_status	= 0,
#ifdef USE_ORIG
	.fifo_flush	= 0,
#else
	.fifo_flush	= dwc_otg_pcd_fifo_flush,
#endif
};




/*  Gadget Operations */
/**
 * The following gadget operations will be implemented in the DWC_otg
 * PCD. Functions in the API that are not described below are not
 * implemented.
 *
 * The Gadget API provides wrapper functions for each of the function
 * pointers defined in usb_gadget_ops. The Gadget Driver calls the
 * wrapper function, which then calls the underlying PCD function. The
 * following sections are named according to the wrapper functions
 * (except for ioctl, which doesn't have a wrapper function). Within
 * each section, the corresponding DWC_otg PCD function name is
 * specified.
 *
 */

/**
 *Gets the USB Frame number of the last SOF.
 */
static int dwc_otg_pcd_get_frame(struct usb_gadget *_gadget){
	dwc_otg_pcd_t *pcd;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p)\n", __func__, _gadget);
	if (_gadget == 0){
HDEBUG_E(1);
		return -ENODEV;
	} else {
		pcd = container_of(_gadget, dwc_otg_pcd_t, gadget);
		dwc_otg_get_frame_number( GET_CORE_IF(pcd) );
	}

HDEBUG_E(2);
	return 0;
}

void dwc_otg_pcd_initiate_srp(dwc_otg_pcd_t *_pcd){
	uint32_t *addr = (uint32_t *)&(GET_CORE_IF(_pcd)->core_global_regs->gotgctl);
	gotgctl_data_t mem;
	gotgctl_data_t val;

HDEBUG_S;
	val.d32 = dwc_read_reg32( addr );
	if (val.b.sesreq) {
		DWC_ERROR("Session Request Already active!\n");
HDEBUG_E(1);
		return;
	}

	DWC_NOTICE("Session Request Initated\n");
	mem.d32 = dwc_read_reg32(addr);
	mem.b.sesreq = 1;
	dwc_write_reg32(addr, mem.d32);

	/* Start the SRP timer */
	dwc_otg_pcd_start_srp_timer( _pcd );
HDEBUG_E(2);
	return;
}

void dwc_otg_pcd_remote_wakeup(dwc_otg_pcd_t *_pcd, int set)
{
	dctl_data_t dctl = {.d32=0};
	volatile uint32_t *addr =
	            &(GET_CORE_IF(_pcd)->dev_if->dev_global_regs->dctl);

HDEBUG_S;
	if (dwc_otg_is_device_mode(GET_CORE_IF(_pcd))) {
		if (_pcd->remote_wakeup_enable) {
			if (set) {
				dctl.b.rmtwkupsig = 1;
				dwc_modify_reg32( addr, 0, dctl.d32 );
				DWC_DEBUGPL(DBG_PCD, "Set Remote Wakeup\n");
				mdelay(1);
				dwc_modify_reg32( addr, dctl.d32, 0 );
				DWC_DEBUGPL(DBG_PCD, "Clear Remote Wakeup\n");
			}
		}
		else {
			DWC_DEBUGPL(DBG_PCD, "Remote Wakeup is disabled\n");
		}
	}

HDEBUG_E(0);
	return;
}

/**
 * Initiates Session Request Protocol (SRP) to wakeup the host if no
 * session is in progress. If a session is already in progress, but
 * the device is suspended, remote wakeup signaling is started.
 *
 */
static int dwc_otg_pcd_wakeup(struct usb_gadget *_gadget)
{
	unsigned long flags;
	dwc_otg_pcd_t *pcd;
	dsts_data_t     dsts;
	gotgctl_data_t  gotgctl;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p)\n", __func__, _gadget);
	if (_gadget == 0){
HDEBUG_E(1);
		return -ENODEV;
	} else {
		pcd = container_of(_gadget, dwc_otg_pcd_t, gadget);
	}
	SPIN_LOCK_IRQSAVE(&pcd->lock, flags);

	/*
	 * This function starts the Protocol if no session is in progress. If
	 * a session is already in progress, but the device is suspended,
	 * remote wakeup signaling is started.
	 */

	/* Check if valid session */
	gotgctl.d32 = dwc_read_reg32(&(GET_CORE_IF(pcd)->core_global_regs->gotgctl));
	if (gotgctl.b.bsesvld) {
		/* Check if suspend state */
		dsts.d32 = dwc_read_reg32(&(GET_CORE_IF(pcd)->dev_if->dev_global_regs->dsts));
		if (dsts.b.suspsts) {
			dwc_otg_pcd_remote_wakeup(pcd, 1);
		}
	}
	else {
		dwc_otg_pcd_initiate_srp(pcd);
	}

	SPIN_UNLOCK_IRQRESTORE(&pcd->lock, flags);
HDEBUG_E(2);
	return 0;
}

#if defined(_USB_LED_) && defined(CONFIG_IFX)
/**
 *Sets the USB LED ON/OFF.
 */
static int dwc_otg_pcd_usb_led(struct usb_gadget *_gadget, int is_on) {
	dwc_otg_pcd_t *pcd;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p)\n", __func__, _gadget);
	if (_gadget == 0){
		return -ENODEV;
	} else {
		pcd = container_of(_gadget, dwc_otg_pcd_t, gadget);
		//dwc_otg_get_frame_number( GET_CORE_IF(pcd) );
		dwc_otg_set_usb_led( GET_CORE_IF(pcd), is_on );
	}

HDEBUG_E(0);
	return 0;
}
#endif //_USB_LED_


static const struct usb_gadget_ops dwc_otg_pcd_ops = {
	.get_frame	 = dwc_otg_pcd_get_frame,
	.wakeup		 = dwc_otg_pcd_wakeup,
	// current versions must always be self-powered
#if defined(_USB_LED_) && defined(CONFIG_IFX)
	.usb_led		= dwc_otg_pcd_usb_led,
#endif //_USB_LED_
};

/**
 * This function updates the otg values in the gadget structure.
 */
void dwc_otg_pcd_update_otg( dwc_otg_pcd_t *_pcd, const unsigned _reset ){

HDEBUG_S;
	if (!_pcd->gadget.is_otg){
HDEBUG_E(1);
		return;
	}
	if (_reset) {
		_pcd->b_hnp_enable = 0;
		_pcd->a_hnp_support = 0;
		_pcd->a_alt_hnp_support = 0;
	}

	_pcd->gadget.b_hnp_enable = _pcd->b_hnp_enable;
	_pcd->gadget.a_hnp_support =  _pcd->a_hnp_support;
	_pcd->gadget.a_alt_hnp_support = _pcd->a_alt_hnp_support;
HDEBUG_E(2);
}

/**
 * This function is the top level PCD interrupt handler.
 */
static irqreturn_t
dwc_otg_pcd_irq(int _irq, void *_dev, struct pt_regs *_r){
HDEBUG_S;
	dwc_otg_pcd_t *pcd = _dev;
	int32_t retval;
#ifdef CONFIG_IFX
	mask_and_ack_ifx_irq (_irq);
#endif

	retval = dwc_otg_pcd_handle_intr( pcd );

HDEBUG_E(0);
	return IRQ_RETVAL(retval);
}

/**
 * PCD Callback function for initializing the PCD when switching to
 * device mode.
 *
 * @param _p void pointer to the <code>dwc_otg_pcd_t</code>
 */
static int32_t dwc_otg_pcd_start_cb( void *_p ){
	dwc_otg_pcd_t *pcd = (dwc_otg_pcd_t *)_p;
	/*
	 * Initialized the Core for Device mode.
	 */
HDEBUG_S;
	if (dwc_otg_is_device_mode( GET_CORE_IF(pcd) )){
		dwc_otg_core_dev_init(GET_CORE_IF(pcd));
	}
HDEBUG_E(0);
	return 1;
}

/**
 * PCD Callback function for stopping the PCD when switching to Host
 * mode.
 *
 * @param _p void pointer to the <code>dwc_otg_pcd_t</code>
 */
static int32_t dwc_otg_pcd_stop_cb( void *_p ){
	dwc_otg_pcd_t *pcd = (dwc_otg_pcd_t *)_p;
	extern void dwc_otg_pcd_stop(dwc_otg_pcd_t *_pcd);

HDEBUG_S;
	dwc_otg_pcd_stop( pcd );
HDEBUG_E(0);
	return 1;
}


/**
 * PCD Callback function for notifying the PCD when resuming from
 * suspend.
 *
 * @param _p void pointer to the <code>dwc_otg_pcd_t</code>
 */

//805211:<IFTW-fchang>.removed
#if 0
static int32_t dwc_otg_pcd_suspend_cb( void *_p ){
	dwc_otg_pcd_t *pcd = (dwc_otg_pcd_t *)_p;

HDEBUG_S;
	if (pcd->driver && pcd->driver->suspend) {
		SPIN_UNLOCK(&pcd->lock);
HDEBUG_C(1);
		pcd->driver->suspend(&pcd->gadget);
HDEBUG_C(2);
		SPIN_LOCK(&pcd->lock);
	}
HDEBUG_E(0);
//	dwc_otg_phy_power_off();
//	dwc_otg_flush_tx_fifo(pcd->otg_dev->core_if,0); /* FLUSH Howard */
	return 1;
}
#endif

//805211:<IFTW-fchang>.added for USB plug in/out issues
static int32_t dwc_otg_pcd_suspend_cb( void *_p ){
	dwc_otg_pcd_t *pcd = (dwc_otg_pcd_t *)_p;
	if (pcd->driver && pcd->driver->suspend) {
  		SPIN_UNLOCK(&pcd->lock);
  		pcd->driver->suspend(&pcd->gadget);

  		SPIN_LOCK(&pcd->lock);
 	}
 	dwc_otg_pcd_stop( pcd );/* Howard */
 	dwc_otg_core_dev_init(GET_CORE_IF(pcd));
 	dwc_otg_flush_tx_fifo(pcd->otg_dev->core_if,0x10); /* FLUSH Howard */
	dwc_otg_flush_rx_fifo(pcd->otg_dev->core_if); /* FLUSH Howard */
	dwc_otg_pcd_resume_cb( _p );
	return 1;
}

/**
 * PCD Callback function for notifying the PCD when resuming from
 * suspend.
 *
 * @param _p void pointer to the <code>dwc_otg_pcd_t</code>
 */
static int32_t dwc_otg_pcd_resume_cb( void *_p )
{
	dwc_otg_pcd_t *pcd = (dwc_otg_pcd_t *)_p;
HDEBUG_S;
	//dwc_otg_phy_power_on();
	if (pcd->driver && pcd->driver->resume) {
		SPIN_UNLOCK(&pcd->lock);
		pcd->driver->resume(&pcd->gadget);

		SPIN_LOCK(&pcd->lock);
	}
	/* Stop the SRP timeout timer. */
	if ((GET_CORE_IF(pcd)->core_params->phy_type != DWC_PHY_TYPE_PARAM_FS) ||
	    (!GET_CORE_IF(pcd)->core_params->i2c_enable)){
		if (GET_CORE_IF(pcd)->srp_timer_started) {
			GET_CORE_IF(pcd)->srp_timer_started = 0;
			del_timer( &pcd->srp_timer );
		}
	}
HDEBUG_E(0);
	return 1;
}


/**
 * PCD Callback structure for handling mode switching.
 */
static dwc_otg_cil_callbacks_t pcd_callbacks = {
	.start = dwc_otg_pcd_start_cb,
	.stop = dwc_otg_pcd_stop_cb,
	.suspend = dwc_otg_pcd_suspend_cb,
	.resume_wakeup = dwc_otg_pcd_resume_cb,
	.p = 0, /* Set at registration */
};

/**
 * This function is called when the SRP timer expires.  The SRP should
 * complete within 6 seconds.
 */
static void srp_timeout( unsigned long _ptr ){
	gotgctl_data_t gotgctl;
	dwc_otg_core_if_t *core_if = (dwc_otg_core_if_t *)_ptr;
	volatile uint32_t *addr = &core_if->core_global_regs->gotgctl;

HDEBUG_S;
	gotgctl.d32 = dwc_read_reg32(addr);

	core_if->srp_timer_started = 0;

	if ((core_if->core_params->phy_type == DWC_PHY_TYPE_PARAM_FS) &&
	       (core_if->core_params->i2c_enable)){
		DWC_PRINT( "SRP Timeout\n");

		if ((core_if->srp_success) && (gotgctl.b.bsesvld)){
			if (core_if->pcd_cb && core_if->pcd_cb->resume_wakeup ) {
				core_if->pcd_cb->resume_wakeup(core_if->pcd_cb->p);
			}
			/* Clear Session Request */
			gotgctl.d32 = 0;
			gotgctl.b.sesreq = 1;
			dwc_modify_reg32( &core_if->core_global_regs->gotgctl, gotgctl.d32, 0);
			core_if->srp_success = 0;
		}
		else {
			DWC_ERROR( "Device not connected/responding\n");
			gotgctl.b.sesreq = 0;
			dwc_write_reg32(addr, gotgctl.d32);
		}
	}
	else if (gotgctl.b.sesreq){
		DWC_PRINT( "SRP Timeout\n");

		DWC_ERROR( "Device not connected/responding\n");
		gotgctl.b.sesreq = 0;
		dwc_write_reg32(addr, gotgctl.d32);
	} else {
		DWC_PRINT( " SRP GOTGCTL=%0x\n", gotgctl.d32);
	}
HDEBUG_E(0);
}

/**
 * Start the SRP timer to detect when the SRP does not complete within
 * 6 seconds.
 *
 * @param _pcd the pcd structure.
 */
void dwc_otg_pcd_start_srp_timer(dwc_otg_pcd_t *_pcd ){
	struct timer_list *srp_timer = &_pcd->srp_timer;
	GET_CORE_IF(_pcd)->srp_timer_started = 1;
HDEBUG_S;
	init_timer( srp_timer );
	srp_timer->function = srp_timeout;
	srp_timer->data = (unsigned long)GET_CORE_IF(_pcd);
	srp_timer->expires = jiffies + (HZ*6);
	add_timer( srp_timer );
HDEBUG_E(0);
}

/**
 * Tasklet
 *
 */
extern void start_next_request( dwc_otg_pcd_ep_t *_ep );

static void start_xfer_tasklet_func (unsigned long data)
{
	dwc_otg_pcd_t *pcd = (dwc_otg_pcd_t*)data;
	dwc_otg_core_if_t *core_if = pcd->otg_dev->core_if;

	int i;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV, "Start xfer tasklet\n");
	for (i=0; i<core_if->dev_if->num_eps; i++) {
		depctl_data_t diepctl;
		diepctl.d32 = dwc_read_reg32( &core_if->dev_if->in_ep_regs[i]->diepctl);
		if (pcd->ep[i].queue_sof) {
			pcd->ep[i].queue_sof = 0;
			start_next_request (&pcd->ep[i]);
			// break;
		}
	}

HDEBUG_E(0);
	return;
}

static struct tasklet_struct start_xfer_tasklet = {
	.next = NULL,
	.state = 0,
	.count = ATOMIC_INIT(0),
	.func = start_xfer_tasklet_func,
	.data = 0,
};


/**
 * This function initialized the pcd Dp structures to there default
 * state.
 *
 * @param _pcd the pcd structure.
 */
void dwc_otg_pcd_reinit(dwc_otg_pcd_t *_pcd){
	static const char * names[] = {
	    "ep0", "ep1", "ep2", "ep3",
	    "ep4", "ep5", "ep6", "ep7",
	    "ep8", "ep9", "ep10", "ep11",
	    "ep12", "ep13", "ep14", "ep15",
	};
	uint32_t i;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV, "%s(%p)\n", __func__, _pcd);

	INIT_LIST_HEAD (&_pcd->gadget.ep_list);
	_pcd->gadget.ep0 = &_pcd->ep [0].ep;
	_pcd->gadget.speed = USB_SPEED_UNKNOWN;

	INIT_LIST_HEAD (&_pcd->gadget.ep0->ep_list);
	/**
	 * Initialize the EP structures.
	 */
	for (i = 0; i < _pcd->num_eps; i++) {
		dwc_otg_pcd_ep_t *ep = &_pcd->ep[i];
		/* Init EP structure */
		ep->desc = 0;
		ep->pcd = _pcd;
		ep->stopped = 1;

		/* Init DWC ep structure */
		ep->dwc_ep.num = i;
		ep->dwc_ep.active = 0;
		ep->dwc_ep.tx_fifo_num = 0;

		/* Control until ep is actvated */
		ep->dwc_ep.type = DWC_OTG_EP_TYPE_CONTROL;
		ep->dwc_ep.maxpacket = MAX_PACKET_SIZE;
		ep->dwc_ep.dma_addr = 0;
		ep->dwc_ep.start_xfer_buff = 0;
		ep->dwc_ep.xfer_buff = 0;
		ep->dwc_ep.xfer_len = 0;
		ep->dwc_ep.xfer_count = 0;
		ep->dwc_ep.sent_zlp = 0;
		ep->dwc_ep.total_len = 0;
		ep->queue_sof = 0;

		/* Init the usb_ep structure. */
		/**
		 * @todo NGS: Add direction to EP, based on contents
		 * of HWCFG1.  Need a copy of HWCFG1 in pcd structure?
		 * sprintf( ";r
		 */
		ep->ep.name = names[i];
		ep->ep.ops = &dwc_otg_pcd_ep_ops;
		/**
		 * @todo NGS: What should the max packet size be set to
		 * here?  Before EP type is set?
		 */
		ep->ep.maxpacket = MAX_PACKET_SIZE;
		list_add_tail (&ep->ep.ep_list, &_pcd->gadget.ep_list);
		INIT_LIST_HEAD (&ep->queue);
	}
	/* remove ep0 from the list.  There is a ep0 pointer.*/
	list_del_init (&_pcd->ep[0].ep.ep_list);

	_pcd->ep0state = EP0_DISCONNECT;
	_pcd->ep[0].ep.maxpacket = MAX_EP0_SIZE;
	_pcd->ep[0].dwc_ep.maxpacket = MAX_EP0_SIZE;
	_pcd->ep[0].dwc_ep.type = DWC_OTG_EP_TYPE_CONTROL;
HDEBUG_E(0);
}

/**
 * This function releases the Gadget device.
 * required by device_unregister().
 *
 * @todo Should this do something?  Should it free the PCD?
 */
#ifndef CONFIG_IFX // winder.
static void dwc_otg_pcd_gadget_release(struct device *_dev){
HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p)\n", __func__, _dev);
HDEBUG_E(0);
}
#endif

/**
 * This function initialized the PCD portion of the driver.
 *
 */
int __init dwc_otg_pcd_init(struct lm_device *_lmdev){
	static char pcd_name[] = "dwc_otg_pcd";
	dwc_otg_pcd_t *pcd;
	dwc_otg_device_t *otg_dev = lm_get_drvdata(_lmdev);
	int retval = 0;


HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV,"%s(%p)\n",__func__, _lmdev );
	/*
	 * Allocate PCD structure
	 */
	pcd = kmalloc( sizeof(dwc_otg_pcd_t), GFP_KERNEL);
	if (pcd == 0) {
HDEBUG_E(1);
		return -ENOMEM;
	}
	memset( pcd, 0, sizeof(dwc_otg_pcd_t));
	spin_lock_init( &pcd->lock );

	otg_dev->pcd = pcd;
	s_pcd = pcd;
	pcd->gadget.name = pcd_name;

#ifdef LINUX26
	strcpy(pcd->gadget.dev.bus_id, "gadget");
#endif
	pcd->otg_dev = lm_get_drvdata(_lmdev);

#ifdef LINUX26
	pcd->gadget.dev.parent = &_lmdev->dev;
	pcd->gadget.dev.release = dwc_otg_pcd_gadget_release;
#endif
	pcd->gadget.ops = &dwc_otg_pcd_ops;

	/* If the module is set to FS or if the PHY_TYPE is FS then the gadget
	 * should not report as dual-speed capable.  replace the following line
	 * with the block of code below it once the software is debugged for
	 * this.  If is_dualspeed = 0 then the gadget driver should not report
	 * a device qualifier descriptor when queried. */
	if ((GET_CORE_IF(pcd)->core_params->speed == DWC_SPEED_PARAM_FULL) ||
	    ((GET_CORE_IF(pcd)->hwcfg2.b.hs_phy_type == 2) &&
	     (GET_CORE_IF(pcd)->hwcfg2.b.fs_phy_type == 1) &&
	     (GET_CORE_IF(pcd)->core_params->ulpi_fs_ls))){
		pcd->gadget.is_dualspeed = 0;
	}
	else {
		pcd->gadget.is_dualspeed = 1;
	}

	if ((otg_dev->core_if->hwcfg2.b.op_mode == DWC_HWCFG2_OP_MODE_NO_SRP_CAPABLE_DEVICE) ||
	    (otg_dev->core_if->hwcfg2.b.op_mode == DWC_HWCFG2_OP_MODE_NO_SRP_CAPABLE_HOST) ||
	    (otg_dev->core_if->hwcfg2.b.op_mode == DWC_HWCFG2_OP_MODE_SRP_CAPABLE_DEVICE) ||
	    (otg_dev->core_if->hwcfg2.b.op_mode == DWC_HWCFG2_OP_MODE_SRP_CAPABLE_HOST)){
		pcd->gadget.is_otg = 0;
	}
	else {
		pcd->gadget.is_otg = 1;
	}

	// winder. temporary solution.
	if (otg_dev->core_if->core_params->otg_cap == DWC_OTG_CAP_PARAM_NO_HNP_SRP_CAPABLE)
		pcd->gadget.is_otg = 0;

	pcd->driver = 0;
#ifdef LINUX26
	/* Register the gadget device */
	device_register( &pcd->gadget.dev );
#endif

	/*
	 * Initialized the Core for Device mode.
	 */
	if (dwc_otg_is_device_mode( GET_CORE_IF(pcd) )) {
		dwc_otg_core_dev_init( GET_CORE_IF(pcd) );
	}

	/*
	 * Initialize EP structures
	 */
	pcd->num_eps = otg_dev->core_if->hwcfg2.b.num_dev_ep;
	dwc_otg_pcd_reinit( pcd );

	/*
	 * Register the PCD Callbacks.
	 */
	dwc_otg_cil_register_pcd_callbacks( otg_dev->core_if, &pcd_callbacks, pcd );

	/*
	 * Setup interupt handler
	 */
	DWC_DEBUGPL( DBG_ANY, "registering handler for irq%d\n", _lmdev->irq);
	retval = request_irq(_lmdev->irq, dwc_otg_pcd_irq, SA_SHIRQ, pcd->gadget.name, pcd);
	if (retval != 0) {
		DWC_ERROR("request of irq%d failed\n", _lmdev->irq);
		kfree (pcd);
HDEBUG_E(2);
		return -EBUSY;
	}

	/*
	 * Initialize the DMA buffer for SETUP packets
	 */
	if (GET_CORE_IF(pcd)->dma_enable) {
		pcd->setup_pkt = dma_alloc_coherent (NULL, sizeof (*pcd->setup_pkt) * 5, &pcd->setup_pkt_dma_handle, 0);
		pcd->status_buf = dma_alloc_coherent (NULL, sizeof (uint16_t), &pcd->status_buf_dma_handle, 0);
	}
	else {
		pcd->setup_pkt = kmalloc (sizeof (*pcd->setup_pkt) * 5, GFP_KERNEL);
		pcd->status_buf = kmalloc (sizeof (uint16_t), GFP_KERNEL);
	}
	if (pcd->setup_pkt == 0) {
		kfree (pcd);
HDEBUG_E(3);
		return -ENOMEM;
	}

	/* Initialize tasklet */
	start_xfer_tasklet.data = (unsigned long)pcd;
	pcd->start_xfer_tasklet = &start_xfer_tasklet;

HDEBUG_E(4);
	return 0;
}

int usb_gadget_unregister_driver(struct usb_gadget_driver *_driver);

/**
 * Cleanup the PCD.
 */
void dwc_otg_pcd_remove( struct lm_device *_lmdev ){
	dwc_otg_device_t *otg_dev = lm_get_drvdata(_lmdev);
	dwc_otg_pcd_t *pcd = otg_dev->pcd;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCDV, "%s(%p)\n", __func__, _lmdev);
	/*
	 * Free the IRQ
	 */
	free_irq( _lmdev->irq, pcd );

	/* start with the driver above us */
	if (pcd->driver) {
		/* should have been done already by driver model core */
		DWC_WARN("driver '%s' is still registered\n", pcd->driver->driver.name);
		usb_gadget_unregister_driver( pcd->driver);
	}

#ifdef LINUX26 // winder.
	device_unregister(&pcd->gadget.dev);
#endif

	if (GET_CORE_IF(pcd)->dma_enable) {
		dma_free_coherent (NULL, sizeof (*pcd->setup_pkt) * 5, pcd->setup_pkt, pcd->setup_pkt_dma_handle);
		dma_free_coherent (NULL, sizeof (uint16_t), pcd->status_buf, pcd->status_buf_dma_handle);
	}
	else {
		kfree (pcd->setup_pkt);
		kfree (pcd->status_buf);
	}
#ifdef CONFIG_IFX
#if   defined(CONFIG_DANUBE)
	set_bit (0, DANUBE_PMU_PWDCR);
	set_bit (6, DANUBE_PMU_PWDCR);
#elif defined(CONFIG_AMAZON_SE)
	set_bit (0, AMAZON_SE_PMU_PWDCR);
	set_bit (6, AMAZON_SE_PMU_PWDCR);
#endif
#endif
	kfree( pcd );
	otg_dev->pcd = 0;
HDEBUG_E(0);
}

/**
 * This function registers a gadget driver with the PCD.
 *
 * When a driver is successfully registered, it will receive control
 * requests including set_configuration(), which enables non-control
 * requests.  then usb traffic follows until a disconnect is reported.
 * then a host may connect again, or the driver might get unbound.
 *
 * @param _driver The driver being registered
 */
int usb_gadget_register_driver(struct usb_gadget_driver *_driver){
	int retval;

HDEBUG_S;
	DWC_DEBUGPL(DBG_PCD, "registering gadget driver '%s'\n", _driver->driver.name);

HDEBUG_C(1);
	if (!_driver || _driver->speed == USB_SPEED_UNKNOWN ||
	    !_driver->bind ||
	    !_driver->unbind ||
	    !_driver->disconnect ||
	    !_driver->setup) {
		DWC_DEBUGPL(DBG_PCDV,"EINVAL\n");
HDEBUG_E(1);
		return -EINVAL;
	}
HDEBUG_C(2);
	if (s_pcd == 0) {
	DWC_DEBUGPL(DBG_PCDV,"ENODEV\n");
HDEBUG_E(2);
		return -ENODEV;
	}
HDEBUG_C(3);
	if (s_pcd->driver != 0) {
	DWC_DEBUGPL(DBG_PCDV,"EBUSY (%p)\n", s_pcd->driver);
HDEBUG_E(3);
		return -EBUSY;
	}
HDEBUG_C(4);

	/* hook up the driver */
	s_pcd->driver = _driver;
#ifdef LINUX26
	s_pcd->gadget.dev.driver = &_driver->driver;
#endif

	DWC_DEBUGPL(DBG_PCD, "bind to driver %s\n", _driver->driver.name);
HDEBUG_C(5);
	retval = _driver->bind(&s_pcd->gadget);
HDEBUG_C(6);
	if (retval) {
HDEBUG_C(7);
		DWC_ERROR("bind to driver %s --> error %d\n",
                           _driver->driver.name, retval);
		s_pcd->driver = 0;
#ifdef LINUX26
		s_pcd->gadget.dev.driver = 0;
#endif
HDEBUG_E(4);
		return retval;
	}
HDEBUG_C(8);
	DWC_DEBUGPL(DBG_ANY, "registered gadget driver '%s'\n",
	                _driver->driver.name);

#ifdef DWC_DEVICE_ONLY // howard, late enable test
#ifndef USE_ORIG
	dwc_otg_enable_global_interrupts( GET_CORE_IF(s_pcd) );
	//dwc_otg_phy_power_on();
#endif
#endif

HDEBUG_E(5);

	return 0;
}
EXPORT_SYMBOL(usb_gadget_register_driver);

/**
 * This function unregisters a gadget driver
 *
 * @param _driver The driver being unregistered
 */
int usb_gadget_unregister_driver(struct usb_gadget_driver *_driver){
	//DWC_DEBUGPL(DBG_PCDV,"%s(%p)\n", __func__, _driver);

HDEBUG_S;
	if (s_pcd == 0) {
		DWC_DEBUGPL(DBG_ANY, "%s Return(%d): s_pcd==0\n", __func__, -ENODEV);
HDEBUG_E(1);
		return -ENODEV;
	}
	if (_driver == 0 || _driver != s_pcd->driver) {
		DWC_DEBUGPL( DBG_ANY, "%s Return(%d): driver?\n", __func__, -EINVAL);
HDEBUG_E(2);
		return -EINVAL;
	}

	_driver->unbind(&s_pcd->gadget);
	s_pcd->driver = 0;

	DWC_DEBUGPL(DBG_ANY, "unregistered driver '%s'\n", _driver->driver.name);
HDEBUG_E(3);
	return 0;
}
EXPORT_SYMBOL(usb_gadget_unregister_driver);

#endif /* DWC_HOST_ONLY */
